import paragonwl
import time
from time import strftime,gmtime 
import os


from Globals import *


def captureTimeConfiguration ( par1, captureTime):

	# capture configuration time
	par1.paragonset("Capture Control Mode","FIXED")  
	par1.paragonset("Capture Control FixedPeriod", "USER")
	par1.paragonset("Capture Control UserPeriod" ,str(captureTime))





def syncEConfiguration (par1):

	if  par1.paragonget( "Capture SyncE WanderCaptEnable" ) == "TRUE":
		# select packet generation with SSM code of QL_PRC
		par1.paragonset( "PacketGeneration #0 Esmc #0 SsmType", "QL-PRC")

	else: 
		# select packet generation with SSM code of QL_DNU
		par1.paragonset ( "PacketGeneration #0 Esmc #0 SsmType", "QL-DNU/DUS")

	




def openCat ( par1 , measure ):

	# opening the Cat - fetching the results

	if measure == "PTP": 
		par1.paragonset ( "Cat Show", "TRUE" )
		par1.paragonset ( "Cat 1588TimeError" )
		par1.waitforcat()
	elif measure == "1PPS":
		par1.paragonset ("Cat Show", "TRUE")
		par1.paragonset ("Cat 1PPS")
		par1.waitforcat()
	else:
		print ("INVALID MEASURE SELECTED")
    


def startMSE (par1,captureTime ):
	par1.paragonset ("MasterSlave Master #0 Enabled", "TRUE")



#
# Start MSE and wait deviceSettlingTime seconds.
# Set deviceSettlingTime to zero if no settling is required
# If captureFilename is not "", then take a capture and save the capture to the specified file
# captureFilename should be the complete path to the file and include the extension
def startMSEandWaitonSettling ( par1, noiseTransferType, deviceSettlingTime =0, captureFilename ="", isDemoMode = False):

#  For some reason these need set before MSE runs
#
#
	par1.paragonset ("MasterSlave FlowFilter ImpairDelReq", "FALSE")
	par1.paragonset ("MasterSlave FlowFilter ImpairMulticastAnnounce", "TRUE")
	par1.paragonset ("MasterSlave FlowFilter ImpairMulticastSync", "TRUE")
	par1.paragonset ("MasterSlave FlowFilter ImpairMulticastDelay", "TRUE")
	par1.paragonset ("MasterSlave FlowFilter ImpairMulticastAllSlaves", "TRUE")
	par1.paragonset ("MasterSlave FlowFilter ImpairSet", "TRUE")
   
	XXXDebugTrace ("Starting MSE...")
	par1.paragonset ("MasterSlave Master #0 Enabled","TRUE")


	# Check that we have a 1pps reference
	if is1ppsReferenceDetected(par1) == False:
		log ("FATAL ERROR: No 1pps reference has been detected.")
		quit()
	
	XXXDebugTrace ( "1pps reference is good...")
	

	# (ENT-44) If running a T-BC test need to check both ports are receiving packets
	# if performing a T-TSC test (PTP-1PPS or SyncE-1PPS ) need to check Port 1 only
	#in the below block of code (incl. the variable), port 1 refers to the port labelled as port 2 on the device/GUI.  Port 0 is 'Port 1' on device/GUI
	port1packetsMandatory = True

	if noiseTransferType == 'PTP-1PPS' or noiseTransferType == 'SyncE-1PPS':
		port1packetsMandatory = False

	# Check that we are seeing packets
	meas = par1.getMeasurementConfiguration()

	if meas.portPacketsReceived(0) == False:
		log ("FATAL ERROR: No packets are being received on Port 0. Please check your cabling and device configuration")
		quit()	

	if meas.portPacketsReceived(1) == False and port1packetsMandatory == True:
		log ("FATAL ERROR: No packets are being received on Port 1. Please check your cabling and device configuration")
		quit()
    
	#gives approriate console message
	if port1packetsMandatory == False:
		XXXDebugTrace("Good packets being received on Port 0. Packets not required on port 1 for T-TSC measurment...")
	else:
		XXXDebugTrace("Good packets being received on both ports...")


	# If captureFilename is specified, start a capture
	# This will capture for the specified duration i.e. the settling time

	if captureFilename != "": 
		XXXDebugTrace ("Capturing / waiting for device to settle - %s seconds" % str(deviceSettlingTime ))
		packetCapture(par1,deviceSettlingTime,isDemoMode,captureFilename)
	else:	
	    # wait on device settling before starting impairment
	    XXXDebugTrace ("Waiting for device to settle" + str(deviceSettlingTime) + "seconds")
	    time.sleep(deviceSettlingTime)  
		
	

def is1ppsReferenceDetected(par1):
	status = False
	# Give it 10 seconds to appear

	i = 0
	while (i < 10):
		status = par1.paragonget("InstrumentStatus Interface 1ppsRefLock Detected")
		if status == True:
			break
		time.sleep (1)
	
		i = i + 1

	return status

def is1ppsMeasurementInputDetected(par1):
	status = False
	# Give it 10 seconds to appear
	
	i = 0
	while ( i < 10 ):
		status = par1.paragonget("InstrumentStatus Interface 1ppsMeasLock Detected")
		if status == "TRUE": 
			break
		time.sleep(1)

		i = i + 1
	return status


def stopMSE ( par1,  captureTime ):
	par1.paragonset ("MasterSlave Master #0 Enabled", "False")





def packetCapture( par1, captureTime,isDemoMode = False, filename = ""):

	if ( isDemoMode == FALSE):
		captureTimeConfiguration( par1,  captureTime )
		par1.paragonset ("MasterSlave StartMeasurement", "TRUE")
	else:
		log ("Demo mode: calling starttimingcapture NOT StartMeasurement")
		par1.starttimingcapture
	
	
	time.sleep ( captureTime )
	
	if ( isDemoMode == False ):

		downLoadActiveCount = 240


		while ( par1.paragonget("Capture Active") == "TRUE"  and (downLoadActiveCount > 0 )):

			time.sleep(1)
			downLoadActiveCount = downLoadActiveCount - 1
			dac  = ("%3d" % downLoadActiveCount)
			XXXDebugTrace ("%s\ Download Active count:\t %s\r" % (currentDateTime(), "FALSE"))
		
		# Force a newline
		XXXDebugTrace( "" )
		res = par1.paragonget (" Capture Active")
		XXXDebugTrace( "%s After stop loop Capture status\t: %s Download Active Count\t: %d" % (currentDateTime() , res, downLoadActiveCount))
	
	
	# Save the capture to the specified file
	if  filename != "" :
		filepath = os.getcwd() + ( "%s" %filename )
		XXXDebugTrace ("Exporting capture to %s" % filepath)
		
		try:
		
			par1.exportdata ("%s" % filepath)

		except IOError:
			log ("Failed to export capture file %s capture status %s This is not fatal - continuing execution." % (filename, par1.paragonget("Instrument Status Capture IsRunning")))
			
		
		
	
	
#	XXXDebugTrace "packetCapture: Stopping MSE..."
#	par1.paragonset (MasterSlave Master #0 Enabled FALSE



def ObjectCleanUp():
	
	
	del ParagonX 
	del Socket 
	del Result 
	del MeasurementAnalysis 
	del Measurement 
	del Port 
	del References 
	del MasterSlaveSetUp 
	logFileFid.close()


def openLogFile (folder, file ): 
	global logFileFid

	exists = os.path.exists(folder) 
	if exists == False :
		os.mkdir( folder)
	
	logFileFid = open(file, "w")


def log (text): 
	global logFileFid
	
	print (text,end='\n',flush= True)
	logFileFid.write (text+"\n")
	logFileFid.flush()


#################################
###
###   DJWAT REMOVED FOR THE MOMENT
##
#################################
#def  getCallerFunctionName (infoString):
#
#	caller = "UNDEFINED"
#	# If the caller is in an object, then the format will be something like:
#	#   ::oo::Obj<num>::objId functionName
#	# e.g. ::oo::Obj44::step0 startStimulus <params>
#	# If the caller isn't in an object then we just get the function name and params
#	# So, if infoString doesn't start with "::oo", then take the first item in the list as the caller name
#	# If infoString starts with "::oo", then extract the ObjId and the function name
#	if {[string first "::oo::" $infoString] == 0} {
#		# Caller is in an object
#		set parts [split $infoString ":"]
#		set remainder [lindex $parts 6]
#		set remainingParts [split $remainder " "]
#		set objId [lindex $remainingParts 0]
#		set callerFunction [lindex $remainingParts 1]
#		set caller "$objId.$callerFunction"
#	} else {
#		# Caller name is first in the list
#		set caller [lindex $infoString 0]
#	}
#	return $caller


def logLevel (level, text):
#	#set callerFunctionName [lindex [info level -1] 0]
#	set callerFunctionName [getCallerFunctionName [info level -1]]
	l = "!!!! %s (%s) !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" %("",level)
	log ("")
	log ("%s" %l)
	log (text)

	if level == "FATAL":
		log ("Aborting...")
		quit()


# Use an optional parameter for newline or nonewline

def  XXXDebugTrace (text , nl = True):
	global debugTrace
	global logFileFid
	
	callerFunctionName = ""

	#set callerFunctionName [lindex [info level -1] 0]
	#set callerFunctionName [info level -1]
	#set callerFunctionName [getCallerFunctionName [info level -1]]
	t = ("********** DEBUG %s: %s" % ( callerFunctionName, text))
	if (nl == True):
	    logFileFid.write(t+ "\n")
	else:
		logFileFid.write(t)
	logFileFid.flush()

	if debugTrace == True:
		if (nl):
			print (t + "\n")
		else:
			print (t)
			


# Get the current time
def currentDateTime():
	
	thistime = strftime("%Y-%m-%d:%S:%H-%M-%S", gmtime())
	return thistime


## Get the minimum value in the supplied arguments
#proc getMin args {
#  if {[llength $args] == 1} {set args [lindex $args 0]}
#    lindex [lsort -real $args] 0
#}

# get the maximum value in the supplied arguments
#def getMax (args):
#	if args.llength == 1:
#
#    if {[llength $args] == 1} {set args [lindex $args 0]}
#    lindex [lsort -real $args] end
#}


#######################################
##  TCL has built in error exception handling
##
##  add this to allow existing code to call error without refactoring

def error( errorStr):
	raise Exception (errorStr)


#######################################
##  Function to convrt an Args parameter
##  into Json format for output to RESTapi

	
def argsToJSON(arg):
    i = iter(arg); d = dict(zip(i, i))
    return json.dumps(d)