version = "08.03.03"

#########################################################################################################
##*** Enhanced Noise Transfer script 8.03.03 ***
##*** updated 15.03.24 
## Changes:
##  - Removed import for graphing (and therefore the dependent modules required for this)
##*** Enhanced Noise Transfer script 8.03.02 ***
##*** updated 23.03.21 
## Changes:
##  - When ReProcess = TRUE, does not attempt to set anything on physical Paragon X
##  - Modifed bug where settling time measurement was put in incorrect folder
##  
##
##*** Enhanced Noise Transfer script 8.03.01 ***
##*** updated 2021.03.09
## Changes:
##  - Modified CSV import to handle both CAT v25.2 and V26 - the V26 adds an extra line
##  - Fixed missing comma bug with CSV export to results.csv
##
##*** Enhanced Noise Transfer script 8.03.00 ***
##*** updated 2021.02.18
## Changes:
##  - Added 1PPS impedance 
##  - Made presence of packets on port 2 non-mandatory condition when performing SyncE-1PPS or PTP-1PPS test (T-TSC testing)
##  - Changed the use of argument names 'type' to pass noiseTransferType as "type" appears to be a reserved word,
##    although this wasn't causing an issue it may in future releases
##
## *** updated 2021.02.10
## Changes:
##	- Released version
##  - Minor bug fixes
##
#### *** Enhanced Noise Transfer script 8.01.00 ***
## *** updated 2020.03.10
## Changes:
##	- Improved handling of Link Status checking - when one or more links is missing
##  - Removed redundant check for Port 2 Link Status check when performing SyncE-1PPS or PTP-1PPS test (T-TSC testing)
##
## *** Enhanced Noise Transfer script 8.00.00 ***
## *** updated 2020.01.07
## Changes:
##	- Added support for uncoupled interfaces
##  - using --PortConfig1/2  parameter block
##
## *** Enhanced Noise Transfer script 7.01.00 ***
## *** updated 2019.11.19
## Changes:
##	- Initial python version based on TCL version 7.00.00
##

## ##############################################################################
## Settings
## ------------------------------------------------------------------------------
## The script allows a number of settings to be made. These are a mix of instrument
## and script configuration
##
## There are three possible sources for settings:
## - Defaults: as defined in Defaults.tcl
## - Command Line: set using key,value pairs as arguments to the script
## - Config File: A file can be specified on the command line which contains user settings
##
## The precedence for settings is defined by the order of processing:
## - All settings are initially taken from the defaults
## - If a config file is specified, then the settings in that file are processed
##		These settings over-ride the defaults
## - Once any config file is processed, command line arguments are processed
##		Command line arguments over-ride both the config file and defaults
## The exception to the above is that the Config File setting is taken ONLY from the command line
##
## Settings are defined as a series of <key, value> pairs
## The key is the name of the setting; the value is used to initialise that setting
## In the Defaults.tcl and any user-defined config file, the key,value pairs are defined
## in a python structure known as a dictionary. See the Defaults.py file for an example
##
## From the command line the key,value pairs are entered as a series of arguments where
## the key name is specified and this is followed by the value
# ## Example: G.8373.2.py Type PTP-PTP Class 50 DeviceSettlingTime 50
##
## Default settings will be used for parameters that are not specified.
## Note that the Defaults.tcl file must define a default value for all possible settings.
##
## Examples:
##	 G.8273.2.py 
##		Executes the script using the defaults defined in Default.py
##   G.8273.2.py ConfigFile config.py
## 		Executes the script using the defaults but settings are over-ridden by those in config.py
##	G.8273.2.py ConfigFile config.py Type SyncE-PTP
##		Executes the script using the defaults with settings over-ridden by those in config.py and
##       with Type set to SyncE-PTP on the command line 
##
## The keys and possible values for all valid settings are listed below.
## For numeric values, the type is shown; check the remote control manual for range information.
## The default value for each setting is defined in the Defaults.tcl file.
##
## InstrIp		IP address of the P-X												<IP Addr>
## GuiIp		IP address of the PC running the P-X GUI							<IP Addr> or <localhost>
##              If GuiIp is not "localhost", then it is assumed that the PC is 
## 				remote to the script host.
##              In this case, results will be retrieved using http requests
## GuiPort		The port to use to connect to the P-X GUI							<integer>
## Type			Noise transfer stimulus and measurement								PTP-BOTH, PTP-PTP, PTP-1PPS, SyncE-BOTH, SyncE-PTP, SyncE-1PPS
## SyncEPoints	List specifying which points to use for SyncE stimulus				List of points e.g. "0 4 5 11". Points are indexed from zero
## PtpPoints	List specifying which points to use for PTP stimulus				List of points e.g. "1 5 7 8 9". Points are indexed from zero
## CableDelay 	Delay (in ns) due to cable from BC master to P-X slave 				<integer>
## LineRate 		Line rate 														1GBE, 100BASET, 10GBE
## Interface 	Interface 															ELECTRICAL, OPTICAL, XFP, SFPPLUS
## Reference	Source of frequency reference										EXT_10M, EXT_R75_E1, EXT_R100_T1, EXT_R100_E1, EXT_K64, INT
## TwoStep		Whether the device under test is a two-step clock					TRUE, FALSE
## SettlingTime Time (in secs) to allow the device to settle after MSE starts		<integer>
## RecoveryTime Time (in secs) to allow the device to recover after noise			<integer> 
##              starts	
## 1ppsThreshold Voltage threshold for 1pps measurement input 						<float>
## AutoNegotiate Enable or disable auto negotiation 								TRUE, FALSE
## SyncE		Enable or disable SyncE measurement during test						TRUE, FALSE
## RemoteCaptureFolder	The folder in which to store capture files on the 			<path/filename>
##						remote Windows PC.
## RemoteProfileFolder	The folder from which prfoiles will be loaded on the		<path/filename>
##						remote Windows PC
## Debug         Enable or disable additional debug information						TRUE, FALSE
## ReProcess	 Allows the script to run with P-X in demo mode and to				TRUE, FALSE
##               re-process previously captured data
##               Uses pre-existing files in specified directory.
## Filter		 Defines the filter type to use. For Calnex use only				IIR, LSQ, NONE
## SimulatedData Generates test data to run through the filter.						TRUE, FALSE
##                   For Calnex use only
## Graphing		Enable graphical UI for results										TRUE, FALSE
## GraphingYAxis	Plot results as either Gain(dB) or Amplitude(ns)				GAIN, AMPLITUDE
## ConfigFile	File containing user-defined settings. The file must exist			<filename>
##					in the same folder as this script
##
## Note: 
##	Error checking on the input parameters is limited.
##	In particular, couplings between parameters are not checked e.g. some line rates are not compatible with some interfaces
##   Incompatible parameter settings may either result in incorrect performance or errors in the main body of the script
##   It is the caller's responsibility to ensure that the parameter names and values are correct.
##	Keys are case sensitive as are those values passed directly to P-X
##   SimulatedResults will not work with Filter=NONE
##
## TODO: Make error checking of arguments more robust
##------------------------------------------------------------------------------
## Design notes:
## - The argument list is passed into this script as a (possibly) long list of arguments.
##	This list is then treated as a dictionary. A dictionary is a list of <key,value> pairs.
##	Dictionary functions can then be used to get the value for any given key
## - The "defaults" list is used as a dictionary. The keys in the list are those used to
##	look for command line arguments
## - A new dictionary is then created that contains the combination of defaults,
##	config file and command line settings
##------------------------------------------------------------------------------
import sys, os
import time
import csv


#from  Globals import *
import Globals as gvar

import paragonwl
from paragonwl import *

from measurements import * 
from Initialization import *
from TimeNoiseTransfer import *
from Commonfunctions import *
from ParametersCheck import *

import Commonfunctions as cf

import traceback, logging



## ##############################################################################
## Globals
## ------------------------------------------------------------------------------
## Debugging and test mode flags should NOT be changed from defaults when testing devices

#global parameters
gvar.demoMode = False
gvar.debugTrace = False
gvar.simulatedData = False

# ###############################################################################

# ##############################################################################
# Parse command Line and any associated user Config File

commandLineParsing()



#global filterType
## Remote indicator and hostname (GUI IP) are global
## When remoteHost is true, http requests will be used to retrieve data from the remote PC
#global remoteHost
#global guiHostname

gvar.debugTrace =  (gvar.parameters['Debug'] != 'FALSE')
gvar.demoMode = (gvar.parameters['ReProcess'] != 'FALSE')
gvar.simulatedData = (gvar.parameters['SimulatedData'] != 'FALSE')
filterType = gvar.parameters['Filter']

remoteHost = False
## ------------------------------------------------------------------------------

## ##############################################################################
## Test type
## ------------------------------------------------------------------------------
## Type of noise transfer test:
##
##   PTP-BOTH     -  input stimulus PTP impairment  output measure: PTP and 1pps
##   PTP-PTP      -  input stimulus PTP impairment  output measure: PTP
##   PTP-1PPS     -  input stimulus PTP impairment  output measure: 1PPS
##
##   SyncE-BOTH   -  input stimulus SYNCE Wander stimulus  output measure: PTP and 1pps
##   SyncE-PTP    -  input stimulus SYNCE Wander stimulus  output measure: PTP
##   SyncE-1PPS   -  input stimulus SYNCE Wander stimulus  output measure: 1PPS
##
noiseTransferType = gvar.parameters ['Type']

## ##############################################################################
## Initialise logging
## ------------------------------------------------------------------------------
## This is done as soon as possible so that output can be logged for most of the script
initialiseLogging( gvar.demoMode , noiseTransferType )

## ##############################################################################
## Masurement/Instrument Configuration
## ------------------------------------------------------------------------------
## Always enable 1pps
## 
onePps = "AccuracyCaptEnable"
syncE = gvar.parameters ['SyncE']
## pdh wander: T1WanderCaptEnable, E1WanderCaptEnable, M2WanderCaptEnable, None
pdhWander = "None"
## sync to an external 1pps
syncToExternal1pps = False
## Voltage level of 1pps Measurement signal 
oneppsLevel = gvar.parameters['1ppsThreshold']
## Termination type of 1pps Measurement signal
oneppsImpedance = gvar.parameters['1ppsImpedance']
## DUT to Paragon-x Cable Delay
cableDelay = gvar.parameters ['CableDelay']
## line rate: 1GBE, 100BASET, 10GBE
#DJWAT-AE7
lineRatePort1 = gvar.parameters['LineRatePort1']
lineRatePort2 = gvar.parameters['LineRatePort2']

## interface 
##
## ELECTRICAL  - for LineRate 100BASET or 1GBE
## OPTICAL     - for LineRate 1GBE   SFP  
## XFP         - For LineRate 10GBE  SFP+    /* Sets lineRate to be 10GBE */
## SFPPLUS     - for LineRate 10GBE  XFP     /* Sets lineRate to be 10GBE */
##
#  
#DJWAT-AE-7
interfacePort1 = gvar.parameters['InterfacePort1']
interfacePort2 = gvar.parameters['InterfacePort2']

## clock source: EXT_10M, EXT_R75_E1, EXT_R100_T1, EXT_R100_E1, EXT_K64, INT
referenceFrequency = gvar.parameters['Reference']
## change Ethernet to auto negotiate

#DJWAT-AE-7
autoNegotiatePort1 = gvar.parameters['AutoNegotiatePort1']
autoNegotiatePort2 = gvar.parameters['AutoNegotiatePort2']



## 1-step or 2-step DUT
twoStep = gvar.parameters['TwoStep']



## ##############################################################################
## Configure instrument connection
## ------------------------------------------------------------------------------
## If guiIp (guiHostname) is not set to localhost, then it is assumed that the 
## P-X application is running on a remote host
##
instrIpAddress = gvar.parameters['InstrIp']
guiHostname = gvar.parameters['GuiIp']
instrPort = "9990"
guiPort = gvar.parameters['GuiPort']

if guiHostname != "localhost":
	gvar.remoteHost = TRUE

remoteCaptureFolder = gvar.parameters['RemoteCaptureFolder']
remoteProfileFolder = gvar.parameters['RemoteProfileFolder']
## ------------------------------------------------------------------------------

## ##############################################################################
## Based on the settings, check that the required packages are installed
## ------------------------------------------------------------------------------
#set reqdPackages ""
## rest required if using remotely hosted Paragon-X
#if {$guiHostname != "localhost"} {
#	if { [catch {package require rest} ] } {
#		set reqdPackages "$reqdPackages rest"
#	}
#}
## Tk, img and rbc required if Graphing is enabled
#if {$graphing} {
#	if { [catch {package require Tk} ] } {
#		set reqdPackages "$reqdPackages Tk"
#	}
#	if { [catch {package require Img} ] } {
#		set reqdPackages "$reqdPackages Img"
#	}
#	if { [catch {package require rbc} ] } {
#		set reqdPackages "$reqdPackages rbc"
#	}
#}

#if {$reqdPackages != ""} {
#	puts "\nOne or more packages must be installed for the options you have selected."
#	puts "The packages are: $reqdPackages"
#	puts "If you are using ActiveState ActiveTcl, use 'teacup install <packageName>' to install these packages"
#	puts "If you are using another Tcl distribution, please consult its documentation for installing packages"
#	exit
#}

## ##############################################################################
## Do the test. Steps are:
## 	- Create the required results folders and files
##	- Configure P-X
##	- Do the test
##	- Clean up
## ------------------------------------------------------------------------------
##

#import djwat
#from djwat import *

#djwattest()

par1 = ParagonX() 
#fred = par2.getMeasurementConfiguration()
#par1.connect("%s %s %s %d" % (instrIpAddress, guiHostname, guiPort , instrPort))
par1.connect(instrIpAddress, guiHostname, instrPort , guiPort)


#try:
		# Connect to Paragon-X and create instance
		#par1 = ParagonX() 
		#par1.connect("%s %s %d %d" % (instrIpAddress, guiHostname, instrPort , guiPort))
#except: 
		
#        print ("An Error occurred on connect", end='\n',flush= True)
#        sys.exit()
	
import sys, traceback, logging

logging.basicConfig(level=logging.ERROR)


 
try:
#	# Do some basic initialisation of the script
	noiseTransferInitialise (par1 , noiseTransferType , gvar.parameters , version ,gvar.demoMode ,gvar.remoteHost)
#	
#	# Reset the Paragon-X to a factory default state
	if gvar.demoMode == False:
		log ("Resetting Paragon-X...")
		par1.paragonset("Rst", "TRUE") 
#	
		par1.paragonset( "OperatingMode", "SERVICES")

		#	# Configure measurements, ports and reference
		#
		# 



		meas = Measurement(par1)
		par1.setMeasurementConfiguration(meas)
		meas.measurementConfiguration( pdhWander, onePps, syncE)

		
		# DJWAT-PX-AE-7
		# Now set up the Ports individually

		meas.portSetLineRate(lineRatePort1, 0)
		meas.portSetLineRate(lineRatePort2, 1)
		meas.portSetInterface(interfacePort1,0)
		meas.portSetInterface(interfacePort2,1)
		
		meas.referencesSetClockSource(referenceFrequency)

		#   DJWAT TO BE ADDED
		# 

		# Back to back measurements with a Gen 2 box needs 1PPS selection made in interfaces
		# comment out if not testing back to back on a rev 2
		#par1.paragonset("Physical RefOutPort","1PPS")



	#		
	#	# Configure 1pps measurement port - consider moving it into the appropriate class
		par1.paragonset("Physical AuxInputThreshold", str(oneppsLevel ))
	# 	# Configure 1pps impedance (ENT-44)
		par1.paragonset("Physical AuxInputTermination", str(oneppsImpedance ))

	#	# Configure sync to external 1pps - consider moving it into the measurement class
		par1.paragonset ("MasterSlave Master #0 SyncToExternal1pps",  str(syncToExternal1pps))
	#	
	#	# setting the Ethernet Auto Negotiate - defaults to FALSE, user must command line override for TRUE

		# DJWAT-PX-AE-7

		#  Need to make autoNegotiate port dependent

		#if lineRate != "10GBE" or "":
		#    par1.paragonset("Physical EthAutonegotiate", autoNegotiate)

		if lineRatePort1 != "10GBE":
			par1.paragonset("Physical #0 EthAutonegotiate", autoNegotiatePort1)

		if lineRatePort2 != "10GBE":
			par1.paragonset("Physical #1 EthAutonegotiate", autoNegotiatePort2)	

	#	# Select the phase profile - consider moving it into the measurement class
		par1.paragonset(" MasterSlave StandardsProfile",  "G.8275.1_PHASE_PROFILE")

	
	#	# Configure MSE
		meas.masterSetEncapsulation ("ETHERNET")
		meas.masterConfigureMulticast() 
		meas.masterSetMessagesFrequencies(8, 16)
		par1.paragonset ("MasterSlave Master #0 TwoStep", str(twoStep) )

		meas.slaveSetEncapsulation("ETHERNET")
		meas.slaveConfigureMulticast()
		meas.slaveSetAllowedMasters(16)
		meas.setCableDelay(cableDelay)

	##  MULTICAST ADDRESS CHANGES
	##
	## By Default, MulticastAddress is the Forwardable one, only bneed to change if commandline/config requests NonForwardable


		if gvar.parameters['MasterMcastAddress'] == "NONFORWARDABLE":
			par1.paragonset ("MasterSlave Master #0 MulticastEthMACAddress", "01 80 c2 00 00 0e")

		if gvar.parameters['SlaveMcastAddress'] == "NONFORWARDABLE":
			par1.paragonset ("MasterSlave Slave MulticastEthMACAddress", "01 80 c2 00 00 0e")


	#	
	#	# Before we start the tests, check some of the basics...
	#	if running a T-BC test need to check both links status, if performing a T-TSC test need to check Port 1 only

		if gvar.parameters['Type'] == 'PTP-1PPS' or gvar.parameters['Type'] == 'SyncE-1PPS':
			if meas.portLinkIsUp(0) == 'FALSE':
				log ("FATAL ERROR: Port1 link is down. Please check that your cabling is correct")
				sys.exit()

		elif meas.portLinkIsUp(0) == 'FALSE' or meas.portLinkIsUp(1) == 'FALSE' :
			log ("FATAL ERROR: One or both links are down. Please check that your cabling is correct")
			sys.exit()
		
		log ("Links are up...")
	#	
	#	# Configure the domain number - consider moving it into the measurement class
		par1.paragonset( "MasterSlave Master #0 DomainNumber", "24")
		par1.paragonset( "MasterSlave Slave DomainNumber", "24")

	# timeNoiseTransfer does the noise transfer measurement
	timeNoiseTransfer ( par1 , noiseTransferType, gvar.demoMode, gvar.simulatedData)

	par1.disconnect()
#	
#	#Object cleanup
	#ObjectCleanUp()
except:
	#    on error { information detailedInformation } {
	#	puts $information
	#	#log $detailedInformation
	print ("Error in Main loop",end='\n',flush=True)
	logging.exception("Text")
	par1.disconnect()
	#ObjectCleanUp()

