# ########################################################################################################
# NoiseGeneration.tpy
# Created: 6th July 2018
# Author: GM
# --------------------------------------------------------------------------------------------------------
# Measures the time error output from the DUT in the absence of any noise on the input
# --------------------------------------------------------------------------------------------------------
# Parameters:
#	par1: An instance of a paragon object
# Return values:
#
# Pre-Conditions:
# The instrument is already configured for G.8273.2 noise generation measurements
# MSE is enabled and has been running long enough to allow the DUT to settle
# --------------------------------------------------------------------------------------------------------
from Commonfunctions import *



def  noiseGenerationInitialise():
	i = 1   # dummy initialisation to 
	



def noiseGenerationDoCapture (par1,captureDuration,captureFolder,captureFile,demo):
	#set captureFile "noiseGeneration.cpd"
	XXXDebugTrace ("Starting packet capture: %s seconds" % captureDuration)

	if demo == False:
		packetCapture(par1, captureDuration, demo)
	

	# explicitly stop capture, possible differences in GUI behavious
	# just in case catch, should an older version throw an error on stopping an already stopped capture
	XXXDebugTrace ("Stopping packet capture...")
	
	try:
		par1.stopcapture()
#		puts "capture stopped"
	except:
		i = 2    # do nothing if capture start fails

	if demo == False:
		f = ""
		#append f [pwd] "/$folder/" $captureFile
		f = "%s/%s" % (captureFolder,captureFile)

		XXXDebugTrace ("Exporting capture to %s" % f)

		try:
			par1.exportdata("%s" % f )
		except:
			logLevel ("WARNING" "Failed to export capture file %s (capture status: %s). This is not fatal - continuing execution." % (captureFile, par1.paragonget("InstrumentStatus Capture IsRunning")))
		
	



def noiseGenerationGetStats(TEType, resultObject):
	resultsDict = {}
	
	if TEType == "2Way":
		resultsDict[ "range"] = resultObject.getMax_Min()
		resultsDict[ "max"] = resultObject.getMax()
		resultsDict[ "min"] = resultObject.getMin()
		resultsDict[ "mean"] = resultObject.getMean()
		resultsDict[ "fwdMessages"] = resultObject.getFwdMessages()
		resultsDict[ "revMessages"] = resultObject.getRevMessages()
		resultsDict[ "fwdRate"] = resultObject.getForwardRate()
		resultsDict[ "revRate"] = resultObject.getReverseRate()
	elif TEType == "T1":
		resultsDict[ "range"] = resultObject.getMax_Min()
		resultsDict[ "max"] = resultObject.getMax()
		resultsDict[ "min"] = resultObject.getMin()
		resultsDict[ "mean"] = resultObject.getMean()
		resultsDict[ "fwdMessages"] = resultObject.getMessages()
		resultsDict[ "revMessages"] = NA
		resultsDict[ "fwdRate"] =  resultObject.getRate()
		resultsDict[ "revRate"] = NA		
	elif TEType == "T4":
		resultsDict[ "range"] = resultObject.getMax_Min()
		resultsDict[ "max"] = resultObject.getMax()
		resultsDict[ "min"] = resultObject.getMin()
		resultsDict[ "mean"] = resultObject.getMean()
		resultsDict[ "fwdMessages"] =  NA
		resultsDict[ "revMessages"] = resultObject.getMessages()
		resultsDict[ "fwdRate"] = NA
		resultsDict[ "revRate"] = resultObject.getRate()		
	elif TEType == "1PPS":
		resultsDict[ "range"] = resultObject.getMax_Min()
		resultsDict[ "max"] = resultObject.getMax()
		resultsDict[ "min"] = resultObject.getMin()
		resultsDict[ "mean"] = resultObject.getMean()
	else:
		logLevel ("FATAL" "TEType %s unrecognised" % TEType) 
		#exit
	return resultsDict
	


def noiseGenerationGetResults (par1,ptpOr1pps, TEType, statistic):

	# addMeasurement will fail if the CAT cannot select the appropriate slot
	# Catch any fail, log an error message and terminate
	
	try:
		meas = par1.addMeasurement ("%s.%s.%s" % (ptpOr1pps,TEType,statistic), "TE" ,TEType)

	except:
		logLevel ("FATAL" ,"There are no %s (%s) results\n\
		Please make sure that you can successfully make time error measurements manually before running this script"% (ptpOr1pps, TEType))
		#log "noiseGenerationGetResults (FATAL): There are no $ptpOr1pps ($TEType) results"
		#log "Please make sure that you can successfully make time error measurements manually before running this script"
	
	measResult = meas.createResult()
	allResults = noiseGenerationGetStats(TEType,measResult)
	result = allResults[statistic]
	del measResult 
	del meas 
	return result


def noiseGeneration (par1,captureFolder,captureFilename,argsDict,demo,reProcessFolder):
	captureDuration = argsDict["captureDuration"] 
	noiseGenerationDoCapture(par1,captureDuration,captureFolder,captureFilename,demo)

	if demo == True:
		XXXDebugTrace ("Demo mode - Importing %s/%s" % (reProcessFolder, captureFilename))	
		par1.importdata ("%s/%s" % (reProcessFolder,captureFilename))	
	
	
	measurements = argsDict["measurements"]
	# In TCL Measurements is a dictionary with keys of either "PTP" or "1pps"
	# The value associaed with the key is a list of T1/T4/2Way followed by the statistic
	# For 1pps, the list is "1pps" followed by the statistic
	
	
	resultsList =[]
	resultsDict = {}
	if 'PTP' in measurements:

		measList = measurements['PTP']


		XXXDebugTrace ("Opening CAT (PTP)")
		openCat(par1, 'PTP')

		for i in range(0,len(measList),2):
			source = measList[i]
			metric = measList[i+1]
			result  = noiseGenerationGetResults(par1,'PTP',source ,metric)
			resultsDict['%s%s' %(source,metric)] = result
	


	if '1PPS' in measurements:
		measList = measurements['1PPS']	
		XXXDebugTrace ("Opening CAT (1PPS)")
		openCat(par1, '1PPS')

		for i in range(0,len(measList),2):
			source = measList[i]
			metric = measList[i+1]
			result  = noiseGenerationGetResults(par1,'1PPS', source ,metric)
			resultsDict['%s%s' %(source,metric)] = result
			i= i+2


	return resultsDict


