proc P100G_checkNoiseTransferCapture {c stim meas} {
	if {[$c isNoiseTransfer]} {
		$c printHeaderNoiseTransfer {log}
		XXXDebugTrace "Total Noise Transfer Table Duration: [$c getNoiseTransferTotalDuration]"
		# This is a Noise Transfer capture. Now check that the stimulus matches
		if {[$c validateStimulus log $stim] != TRUE} {
			log "[$c getCaptureFilename] file is not valid. Aborting..."
			exit
		}
		if {[$c validateMeasurement log $meas] != TRUE} {
			log "Capture file is not valid. Aborting..."
			exit
		}
	} else {
		log "[$c getCaptureFilename] is not a capture from a Noise Transfer measurement. Aborting..."
		exit
	}
}

#
# Reads in the P100G file and extracts information about the transfer test from the header
# This is then used to create a stimulus list for the rest of the script
#
proc P100G_parseCaptureFile {seq sList} {
	set P100GFile ""
	set measurement [$seq getMeasurement]
	set stimulus [$seq getStimulus]

#	if {$measurement == "PTP" || $measurement == "BOTH"} {
#		set captureFile "Ptp1588100.cdf"
#	} elseif {$measurement == "1PPS"} {
#		set captureFile "OnePpsAccuracy100.cdf"
#	} else {
#		log "P100G_parseCaptureFile: Invalid measurement ($measurement). Aborting..."
#		exit
#	}

	# The new stimulus list can come from either PTP or 1pps - should be the same in any case
	if {$measurement == "PTP" || $measurement == "BOTH"} {
		set captureFile "Ptp1588100.cdf"
		XXXDebugTrace "Loading P100G file: $captureFile"
		P100GCapture create ptpCapture "./ReprocessP100G" $captureFile
		$seq p100G_setPtpCapture ptpCapture
		ptpCapture printHeaderGeneral {log}
		P100G_checkNoiseTransferCapture ptpCapture $stimulus "PTP"
		# Need to modify the list from the file header into a format we can use in the script
		set newStimList [P100G_modifyStimulusList ptpCapture $sList]
	}
	if {$measurement == "1PPS" || $measurement == "BOTH"} {
		set captureFile "OnePpsAccuracy100.cdf"
		XXXDebugTrace "Loading P100G file: $captureFile"
		P100GCapture create 1ppsCapture "./ReprocessP100G" $captureFile
		$seq p100G_set1ppsCapture 1ppsCapture
		1ppsCapture printHeaderGeneral {log}
		P100G_checkNoiseTransferCapture 1ppsCapture $stimulus "1PPS"
		# Need to modify the list from the file header into a format we can use in the script
		set newStimList [P100G_modifyStimulusList 1ppsCapture $sList]
	}

	# Need to modify the list from the file header into a format we can use in the script
	#set newStimList [P100G_modifyStimulusList ptpCapture $sList]
	
#	append P100GFile [pwd] "/ReprocessP100G/$captureFile"
#	XXXDebugTrace "Loading P100G file: $P100GFile"
#	P100GCapture create capture "./ReprocessP100G" $captureFile
#	capture printHeaderGeneral {log}
#	if {[capture isNoiseTransfer]} {
#		capture printHeaderNoiseTransfer {log}
#		XXXDebugTrace "Total Noise Transfer Table Duration: [capture getNoiseTransferTotalDuration]"
		# This is a Noise Transfer capture. Now check that the stimulus matches
#		if {[capture validateStimulus log $stimulus] != TRUE} {
#			log "$P100GFile file is not valid. Aborting..."
#			exit
#		}
#		if {[capture validateMeasurement log "PTP"] != TRUE} {
#			log "Capture file is not valid. Aborting..."
#			exit
#		}
				
		# Need to modify the list from the file header into a format we can use in the script
#		set newStimList [P100G_modifyStimulusList capture $sList]
#	} else {
#		log "$P100GFile is not a capture from a Noise Transfer measurement. Aborting..."
#		exit
#	}
	
	#XXXDebugTrace $newStimList
	return $newStimList
}

proc P100G_createResultsFile { file } {
	set resultsFile $file
	set resultsFile [string map {".csv" "_Results.csv"} $resultsFile] 
	
	return $resultsFile
}

proc P100G_readResultsFromFile {catFile measure} {
	XXXDebugTrace "P100G_readResultsFromFile: $catFile $measure"

	if {$measure == "T4"} {
		return [P100G_readPtpResultsFromFile $catFile]
	} elseif {$measure == "1pps"} {
		return [P100G_read1ppsResultsFromFile $catFile]
	} else {
		log "ERROR: $measure is not a valid measurement type. Aborting."
		exit
	}
}	

proc P100G_readPtpResultsFromFile {catFile} {
	XXXDebugTrace "P100G_readPtpResultsFromFile: $catFile"

	set catFile [open $catFile]
	set x 0
	set y {}
	set times {}
	set data {}
	set index 0

	while {1} {
		set line [gets $catFile]
		#puts $line
		set line [split $line ";"]
		set timeColumnValue [lindex $line 0]
		set dataColumnValue [lindex $line 1]
		#puts "$index - t:$timeColumnValue d:$dataColumnValue"
		if { $dataColumnValue == ""  || $dataColumnValue == "DATA END" } {
			close $catFile
			break
		}
		if {$x > 4} {
			#dict lappend y $timeColumnValue $dataColumnValue
			lappend times [expr $timeColumnValue / 1e9]
			lappend data $dataColumnValue

			incr index
		} 
		incr x
	}

	# y now contains the the timestamp and measured amplitude in ns for each sampled point
	lappend y $times $data

	return $y
}

proc P100G_read1ppsResultsFromFile {catFile} {
	variable graphing
	XXXDebugTrace "read1ppsResultsFromP100GFile: $catFile"

	set catFile [open $catFile]
	set x 0
	set y {}
	set times {}
	set data {}
	set index 0

	while {1} {
		set line [gets $catFile]
		#puts $line
		#set line [split $line ";"]
		#set timeColumnValue [lindex $line 0]
		set dataColumnValue $line
		#puts "$index - t:$timeColumnValue d:$dataColumnValue"
		# Check the data column to look for the end of the file.
		if { $dataColumnValue == ""  || $dataColumnValue == "DATA END" } {
			close $catFile
			break
		}
		# Skip the header - this should be smarter. Maybe look for numeric value in timestamp?
		if {$x > 6} {
			#dict lappend y $timeColumnValue $dataColumnValue
			#lappend times [expr $timeColumnValue / 1e9]
			lappend data $dataColumnValue

			incr index
			lappend times $index
		} 
		incr x
	}

	# y now contains the the timestamp and measured amplitude in ns for each sampled point
	lappend y $times $data

	return $y
}
# ############################################################################################################################################################


proc P100G_getStepResults {data seq stepNum meas} {
	variable graphing
	# We need to figure out where the segment we want is in the data
	# stepNum is the step we are interested in
	# We need to add up the durations of all previous segments to get the start point
	# Then use the duration for stepNum to find the end
	set i 0
	set start 0
	set end 0
	set range {}
	foreach seqStep $seq {
		set i [lindex $seqStep 0]
		set step [lindex $seqStep 1]

		set freq [$step getFreq]
		set Fs [$step getSampleRate]
		set d [$step getDuration]
		
#	foreach tp $stimulusList 
#		set freq [lindex $tp 0]
#		set Fs [lindex $tp 2]
#		set d [lindex $tp 3]

		if {$meas == "T4"} {
			set end [expr {int($end + $d*$Fs)}]
		} elseif {$meas == "1PPS" || $meas == "1pps"} {
			set end [expr {int($end + $d)}]
		} else {
			log "P100G_getStepResults: Invalid measurement type - $meas"
		}
		if {$i == $stepNum} {
			break
		}
		set start [expr {$end + 1}]
		incr i
	}

	set expectedDataLength [expr $end - $start + 1]
	set times [lindex $data 0]
	set values [lindex $data 1]
	set timeRange [lrange $times $start $end]
	set valueRange [lrange $values $start $end]
	set actualDataLength [llength $valueRange]
	XXXDebugTrace "P100G_getStepResults: $meas start $start, end $end, range $expectedDataLength, actual data length $actualDataLength"
	
	if {$actualDataLength < $expectedDataLength} {
		log "*** Warning: Data available for step $stepNum ($actualDataLength) is less than expected ($expectedDataLength) ***"
	}

	if {$graphing} {
		updateDataPlot $timeRange $valueRange $freq $meas
	}


	#puts "[llength $timeRange], [llength $valueRange]"
	lappend range $timeRange $valueRange
	return $range
}

proc P100G_determinePassFail {in ll ul} {
	if {$ll != "NA" && $in < $ll} {return "FAIL(lower)"}
	if {$ul != "NA" && $in > $ul} {return "FAIL(upper)"}
	return PASS
}

# Need to modify the list from the file header into a format we can use in the script
# The file header has freq,ampl,duration,settling,guard
# This needs to be changed into freq,ampl,sampleRate,duration,LL,UL,file
# Note that this assumes that original stimulusList frequencies and order match that in the file - dangerous but OK for now
# It is assumed that all steps have the same settling time
proc P100G_modifyStimulusList {cap originalList} {
	#log $originalList
	set newList ""
	set sList [$cap getNoiseTransferStimulusList]
	set i 0
	foreach step $sList {
		set f [lindex $step 0]
		set a [lindex $step 1]
		set d [lindex $step 2]
		set deviceRecoveryTime [lindex $step 3] 
		set guardTime [lindex $step 4] 
		set ll [lindex [lindex $originalList $i] 4]
		set ul [lindex [lindex $originalList $i] 5]
		set newStep "$f $a 16.0 $d $ll $ul NA"
		set newList [lappend newList $newStep]
		incr i
	}
	return $newList
}


proc P100G_exportFileFromCat {ptp_or_1pps fileBase} {
	global P100GNoiseTransferFile

	if {$ptp_or_1pps == "PTP"} {
		set meas "T4"
	} elseif {$ptp_or_1pps == "1PPS"} {
		set meas "1PPS"
	} else {
		log "*** ERROR *** exportFromCat: $ptp_or_1pps is not a valid measurement. Aborting..."
		exit
	}
	XXXDebugTrace "exportFileFromCAT: $fileBase, $meas"
		
	par1 paragonset Cat SelectSlot $meas
	set newCatFile $fileBase
	# Remove the .csv
	set newCatFile [ string map {".csv" ""} $fileBase ]
	set catFilePostfix "_$meas"
	append catFilePostfix "_Cat.csv"
		#puts $catFilePostfix
	append newCatFile $catFilePostfix
	#set catFile [ string map {".csv" "_Cat.csv"} $catFile ]

	set catFileAddress ""

	# If the script isn't running on a remote host, then provide the full path to the results
	append catFileAddress [pwd] "/" $newCatFile
		
	#XXXDebugTrace "exportFileFromCat: $catFileAddress"
	par1 paragonset Cat TimeErrorCsv Export $catFileAddress
	
	# Dirty hack - CAT seems to be exporting to its own export folder.
	# Not sure why this isn't taking the full path but hack it for the moment
	#	set exportedFile $env(HOME)
	#	append exportedFile "/Documents/Calnex/Cat/Exports/"
	#	append exportedFile [file tail $catFileAddress]
	#	XXXDebugTrace "exportFileFromFile: $exportedFile"
	#	file rename -force $exportedFile $catFileAddress
	
	return $catFileAddress
}

proc P100G_getFullResults {seq measure} {
	if {$measure == "PTP"} {
		set capture [$seq p100G_getPtpCapture]
		set meas "T4"
		set sRate 16.0
		$seq setRecoveryTime [$capture getNoiseTransferRecoveryTime]
		$seq setGuardTime [$capture getNoiseTransferGuardTime]
	} elseif {$measure == "1PPS"} {
		set capture [$seq p100G_get1ppsCapture]		
		set meas "1pps"
		set sRate 1.0
		$seq setRecoveryTime [$capture getNoiseTransferRecoveryTime]
		$seq setGuardTime [$capture getNoiseTransferGuardTime]
	} else {log "P100G_getFullResults: $measure is not a valid measurement type"}
	
	XXXDebugTrace "Opening the CAT ($measure)... [$capture getCaptureFilename]"
	par1 paragonset Cat OpenFile "[pwd]/[$capture getCaptureFilename]"
	par1 waitforcat
	# export file from Cat
	#set catFile [ P100G_exportFileFromCat par1 "P100G/P100GNoiseTransfer.csv" "T4"]
	set catFile [ P100G_exportFileFromCat $measure "P100G/P100GNoiseTransfer.csv"]
	set fullResults [ P100G_readResultsFromFile $catFile $meas]
	set samplesInCapture [llength [lindex $fullResults 1]]
	set totalDuration [expr $samplesInCapture / $sRate]
	XXXDebugTrace "Total number of samples in capture: $samplesInCapture; Duration: $totalDuration"

	return $fullResults
}

proc P100G_processSequence {sequence fullResults} {
	set measurement [sequence getMeasurement]
	set stimulus [sequence getStimulus]

	if {$measurement == "PTP" || $measurement == "BOTH"} {
		set measure "PTP"
		set metric "T4"
		set sRate 16.0
	} elseif {$measurement == "1PPS" || $measurement == "BOTH"} {    
		set measure "1PPS"
		set metric "1pps"
		set sRate 1.0
	} else {
		log "P100G_processSequence: invalid measurement $measurement"
	}

	set counter 0
	set seq [sequence getSequence]
	#log $seq
	foreach seqStep $seq {
#			foreach tp $stimulusList 
		set i [lindex $seqStep 0]
		set step [lindex $seqStep 1]

		set inputFreq [$step getFreq]
		set inputAmpl [$step getAmpl]
		#set inputSampleRate [$step getSampleRate]
		set captureTime [$step getDuration]
		set lowerLimit [$step getLL]
		set upperLimit [$step getUL]

		log ""
		log "[currentDateTime]\tStep $i Stimulus:\t$inputFreq\t$inputAmpl\t$captureTime"
			
		set freqWithUnderbar [string map {"." "_"} $inputFreq] 		
		set file "NoiseTransfer_$freqWithUnderbar.csv"

		#log ""
		#log "[currentDateTime]\tStep $counter Stimulus:\t$tp"
		# creating the result file 
		set resultsFile [ P100G_createResultsFile "P100G/$file" ]
		XXXDebugTrace "Results File: $resultsFile"
		if {[file exists "./$resultsFile"]} {
			XXXDebugTrace "Results File: $resultsFile already exists - deleting..."
			file delete "./$resultsFile"
		}
		#set results [getP100GStepResults $fullResults $stimulusList $counter "T4"]
		set results [P100G_getStepResults $fullResults $seq $counter $metric]
		# Remove a specified amount of data from the capture
		# This allows the device to recover after being hit by the noise
		# The recovery time should be set as a parameter
		#set results [removeRecoveryTime $results $deviceRecoveryTime 16 $inputFreq $measurement]
		set rTime [sequence getRecoveryTime]
		set gTime [sequence getGuardTime]
		#### DIRTY HACK FOR OLD CDF FILES. REMOVE ASAP
		set rTime 30
		set gTime 10
		set results [removeRecoveryTime $results $rTime $sRate $inputFreq $measurement]
		set results [removeGuardTime $results $gTime $sRate $inputFreq $measure]
				#set resultList [filterLeastSquares PTP "" $results $resultsFile $inputFreq $inputAmpl $inputSampleRate]
		set resultList [filterLeastSquares $stimulus $results $resultsFile $inputFreq $inputAmpl $sRate]
			
		set pkpkAmplitudeIn [ lindex $resultList 0] 
		set amplitudeOut  [ lindex $resultList 1] 
		set frequency [ lindex $resultList 2]
	    
		# Calculate the gain
		set gain [ expr 20 * log10( [expr $amplitudeOut / $pkpkAmplitudeIn ])]
		
		# Work out pass/fail
		# There is one pass/fail column: pass, fail (lower), fail (upper)
		set passFail [P100G_determinePassFail $amplitudeOut $lowerLimit $upperLimit]
		
		# save results to file
		if {$measure == "PTP"} {			
			savePtpResultsToFile [sequence getFidResults] $file $frequency $pkpkAmplitudeIn $amplitudeOut $gain $lowerLimit $upperLimit $passFail
		} else {
			save1ppsResultsToFile [sequence getFidResults] $file $frequency $pkpkAmplitudeIn $amplitudeOut $gain $lowerLimit $upperLimit $passFail			
		}

		log "[currentDateTime]\tStep $counter Measurement:\t[format %10.8f $frequency]\t $pkpkAmplitudeIn\t [format %.2f $amplitudeOut] \t [format %.2f $gain]\t $passFail"
			
		incr counter
	}
	
}