from filterFunction import *
from filterLeastSquares import *
from filterFunction import *
from filterLeastSquares import *
from Commonfunctions import *
from Commonfunctions import *
from NoiseGeneration import *
from math import log10
from collections import OrderedDict

#import P100GUtils

# Graphing is not supported, do not import this
# from graphing import *

import time
from time import strftime,gmtime 
import os
import math
import requests, json
import Globals as gvar

# Pi is used in a couple of places. Make it global
pi = 3.1415926535897931
global demoMode
global simulatedData


# ########################################################################################################
# Description of filenames and locationos
#
# At each stage of the test, various files are created and used.
# The names of folders and files are determined either in the sequence object or the step object
# (depending on whether the scope of the folder/file is the entire sequence or a single step).
# The location and names of files is dependent on whether the GUI is running on a remote host
# There are also some couplings with demo mode and reprocessing.
#
# Profiles Folder: (sequence)
#	Local: 		Current folder (where the script is running)
#	Remote: 	Passed in as an argument to the script
# Profiles File: (step)
#				Held in stimulusList
# Logs Folder: (sequence)
# 				[pwd]/Logs
# Log File: (sequence)
#	Normal: 	Log <noiseTransferType> <dateStart>-T-<timeStart>.txt
#	Demo: 		Log <noiseTransferType> Demo.txt
# Results Folder: (sequence)
#   Normal: 	[pwd]/Results/Results <noiseTransferType> <dateStart>-T-<timeStart>
#	Demo: 		[pwd]/Results/Results <noiseTransferType> Demo
# Overall Results File: (sequence)
# 	Normal: 	<Results Folder>/Results <dateStart>-T-<timeStart>.csv
# 	Demo: 		<Results Folder>/Results Demo.csv
# Overall Results GUI Picture: (sequence)
#				<Results Folder>/Results.bmp
#
# The test process is essentially:
# 	Capture (save to file)
#	Load into CAT
#	Export from CAT (into file)
#	Process file (results saved to file)
#	Individual step results are stored in the voerall results file above
#
# The folders and files used at each stage above are:
#
# Capture Folder: (step)
#	Normal: 			<Results Folder> (as above)
#	Demo, not Remote: 	[pwd]/ReProcess
#	Remote: 			Passed in as an argument to the script
# Capture File: (step)
#						<noiseTransferType> NoiseTransfer <freq>Hz.clxz
# CAT Export Folder: (step)
#	Local:				<Results Folder>
#	Remote:				Defaults to Calnex/.../CAT/Export
# CAT Export File: (step)	
#						NoiseTransfer_<freqWithUnderbar>_<meas>_Cat.csv		where <meas> is T4 or 1PPS etc
# Filtered CAT Data Folder: (step)
#						<Results Folder>
# Filtered CAT Data: (step)
#						NoiseTransfer_<freqWithUnderbar>_Results.csv	
# 

# #######################################################################################################
#   STIMULUS 
#	Either PTP or SyncE will be applied as a noise stimulus.
#   The stimulus applied is defined in stimulusList.
#
#   PTP:   Freq, pk-pk ampl, packet sample rate, capture duration, lower limit, upper limit, PDV file
#   SYNCE: Freq, pk-pk ampl, packet sample rate, capture duration, lower limit, upper limit
#
#	The frequencies, amplitudes and limits are based on those defined in amendment TD64 Rev2 to G.8273.2
#	The SyncE noise amplitudes at lower frequencies are higher than those suggested in the standard.
#	Using higher amplitudes provides better signal-to-noise ratio whilst being within the tolerable levels for the DUT.
#	Note that the limits are defined in Table VI.4 (PTP) and VI.5 (SyncE).
#	Since the script uses the least squares filter, the limits are defined with N=10 for PTP and N=25 for SyncE
#	The limits are checked against the measured pk-pk amplitude (as opposed to the gain)
#
#	NOTES: 
#	1. 	The amplitude of the noise applied for a PTP input is defined in the standards as 200ns.
#		The profiles loaded by P-X are however applied in only one direction (to Syncs).
#		So, to achieve 200ns 2Way noise, 400ns is applied to the Sync messages.
#		In other words, the profiles used have a pk-pk amplitude of 400ns.
#		The table below has been changed to reflect the fact that what we are actually applying is
#		2Way noise of 200ns (it used to be 400ns).
#	2.	The sample rate and amplitudes must be represented as floating point to make sure subsequent 
#		maths operations are not rounded to integers. 
# #######################################################################################################





#
# When stimulus is PTP
# Limits below are from Appendix VI in the standard using N=10
stimulusListPtp =	OrderedDict ({
	'0': { "Frequency": 0.00390625 , "amplitude":200.0 , "samples": 16.0, "duration": 800, "LL": 130, "UL":	215, "PDVFile": "NoiseTransfer_0_00390625Hz.csv" },
	'1': { "Frequency": 0.0078125  , "amplitude":200.0 , "samples": 16.0, "duration": 800, "LL": 130, "UL":	215, "PDVFile": "NoiseTransfer_0_0078125Hz.csv"  },
	'2': { "Frequency": 0.015625   , "amplitude":200.0 , "samples": 16.0, "duration": 400, "LL": 130, "UL":	215, "PDVFile": "NoiseTransfer_0_015625Hz.csv"   },
	'3': { "Frequency": 0.03125    , "amplitude":200.0 , "samples": 16.0, "duration": 250, "LL": 130, "UL":	215, "PDVFile": "NoiseTransfer_0_03125Hz.csv"    },
	'4': { "Frequency": 0.0615625  , "amplitude":200.0 , "samples": 16.0, "duration": 250, "LL": NA , "UL":  215, "PDVFile": "NoiseTransfer_0_0615625Hz.csv" },
	'5': { "Frequency": 0.123125   , "amplitude":200.0 , "samples": 16.0, "duration": 250, "LL": NA , "UL":  140, "PDVFile": "NoiseTransfer_0_123125Hz.csv"  },
	'6': { "Frequency": 0.24625    , "amplitude":200.0 , "samples": 16.0, "duration": 250, "LL": NA , "UL":  90 , "PDVFile": "NoiseTransfer_0_24625Hz.csv"   },
	'7': { "Frequency": 0.4925     , "amplitude":200.0 , "samples": 16.0, "duration": 250, "LL": NA , "UL":  50 , "PDVFile": "NoiseTransfer_0_4925Hz.csv"    },
	'8': { "Frequency": 0.985      , "amplitude":200.0 , "samples": 16.0, "duration": 250, "LL": NA , "UL":  35 , "PDVFile": "NoiseTransfer_0_985Hz.csv"     },
	'9': { "Frequency": 1.985      , "amplitude":200.0 , "samples": 16.0, "duration": 250, "LL": NA , "UL":  30 , "PDVFile": "NoiseTransfer_1_985Hz.csv"     } 
	})


stimulusListSyncE = OrderedDict	({
	'0': { "Frequency": 0.00391 , "amplitude":2000.0 , "samples":16.0, "duration": 800, "LL": NA, "UL":  225 }, 
	'1': { "Frequency": 0.00781 , "amplitude":2000.0 , "samples":16.0, "duration": 800, "LL": NA, "UL":  340 },
	'2': { "Frequency": 0.01563 , "amplitude":2000.0 , "samples":16.0, "duration": 400, "LL": NA, "UL":  630 },
  	'3': { "Frequency": 0.03125 , "amplitude":1000.0 , "samples":16.0, "duration": 250, "LL": NA, "UL":  565 }, 
	'4': { "Frequency": 0.06156 , "amplitude":500.0  , "samples":16.0, "duration": 250, "LL": NA, "UL":  545 }, 
	'5': { "Frequency": 0.12313 , "amplitude":250.0  , "samples":16.0, "duration": 250, "LL": 150, "UL": 285 }, 
	'6': { "Frequency": 0.24625 , "amplitude":250.0  , "samples":16.0, "duration": 250, "LL": 150, "UL": 285 }, 
	'7': { "Frequency": 0.4925  , "amplitude":250.0  , "samples":16.0, "duration": 250, "LL": 150, "UL": 285 }, 
	'8': { "Frequency": 0.985   , "amplitude":250.0  , "samples":16.0, "duration": 250, "LL": 150, "UL": 285 }, 
	'9': { "Frequency": 1.985   , "amplitude":250.0  , "samples":16.0, "duration": 250, "LL": NA, "UL":  285 }, 
	'10': { "Frequency": 3.985  , "amplitude":250.0  , "samples":16.0, "duration": 250, "LL": NA, "UL":  285 }, 
	'11': { "Frequency": 7.985  , "amplitude":250.0  , "samples":16.0, "duration": 250, "LL": NA, "UL":  285 }
	}) 


stimulusListSyncE_old = OrderedDict ({
	'0': { "Frequency": 0.00391, "amplitude": 250.0, "samples": 16.0, "duration": 800,"LL": NA, "UL":  45  }, 
	'1': { "Frequency": 0.00781, "amplitude": 250.0, "samples": 16.0, "duration": 800,"LL": NA, "UL":  65  },
	'2': { "Frequency": 0.01563, "amplitude": 250.0, "samples": 16.0, "duration": 400,"LL": NA, "UL":  100 },
	'3': { "Frequency": 0.03125, "amplitude": 250.0, "samples": 16.0, "duration": 250,"LL": NA, "UL":  160 }, 
	'4': { "Frequency": 0.06156, "amplitude": 250.0, "samples": 16.0, "duration": 250,"LL": NA, "UL":  285 }, 
	'5': { "Frequency": 0.12313, "amplitude": 250.0, "samples": 16.0, "duration": 250,"LL": 150, "UL": 285 }, 
	'6': { "Frequency": 0.24625, "amplitude": 250.0, "samples": 16.0, "duration": 250,"LL": 150, "UL": 285 }, 
	'7': { "Frequency": 0.4925 , "amplitude": 250.0, "samples": 16.0, "duration": 250,"LL": 150, "UL": 285 }, 
	'8': { "Frequency": 0.985  , "amplitude": 250.0, "samples": 16.0, "duration": 250,"LL": 150, "UL": 285 }, 
	'9': { "Frequency": 1.985  , "amplitude": 250.0, "samples": 16.0, "duration": 250,"LL": NA, "UL":  285 }, 
	'10': { "Frequency": 3.985 , "amplitude": 250.0, "samples": 16.0, "duration": 250,"LL": NA, "UL":  285 }, 
	'11': { "Frequency": 7.985 , "amplitude": 250.0, "samples": 16.0, "duration": 250,"LL": NA, "UL":  285 }
	  })


####################################
##  GLOBALS
####################################

global activesequence

# P100G analysis is a bit of a hack. Some vars to support it...
#global P100GNoiseTransferFile
#set P100GNoiseTransferFile TRUE
# T4 or 1pps
#set P100GMeas "1pps"
#set P100GStimulus "SyncE"

def initialiseLogging ( demoMode,  noiseTransferType):
	folderLogs = "Logs"
	dt = currentDateTime()

	dtList = dt.split(':')




	if demoMode == False:
		fileLog = ("%s/Log-%s-%s-T-%s.txt" % (folderLogs, noiseTransferType,dtList[0], dtList[2]))
	else:
		fileLog = ("%s/Log_%s_Demo.txt" % (folderLogs, noiseTransferType))
	

	openLogFile ( folderLogs,  fileLog)
	log ("Log will be stored in:\t\%s" % fileLog)




def noiseTransferInitialise ( paragonObject1, noiseTransferType, parameters, scriptVersion, demoMode, remoteHost ):
	global P100GNoiseTransferFile
	global stimulusListPtp
	global stimulusListSyncE
	global stimulusListSyncE_old
	global sequence
	global stimulus
	global simulatedData
	global graphing

	#Setting this to false so hooks can be built into the graphing part for processing - this was done in the timeNoiseTransfer definition also so doing it here - Billy
	P100GNoiseTransferFile = False

	
	# If we are processing a P100G file, then load in the code to extract deltails from the header
	#if {$P100GNoiseTransferFile} { source "P100GCapture.tcl" }
	
  

	#transferSequence create sequence $type $parameters $demoMode $remoteHost

	sequence = transferSequence( paragonObject1, noiseTransferType, parameters,demoMode, remoteHost)

	#set stimulus [sequence getStimulus]
	stimulus = sequence.getStimulus()

	# A setting allows the user to run only a sub-set of points
	# Get the setting and construct the stimulusList appropriately
	syncEPoints = parameters[ 'SyncEPoints']
		
	ptpPoints     = parameters['PtpPoints']
	reProcess     = parameters['ReProcess']
	simulatedData = parameters['SimulatedData']
	graphing	  = parameters['Graphing']


	stimulusList = []

	selectedStimulusList = []

	inputDictionary = OrderedDict()
	stimulusListUsedDuringTest = OrderedDict()
	stimulusDictionary = OrderedDict()
	
	if reProcess == "TRUE": 
		# Read in the stimulusList.py file generated when originally running the test
		# This file needs to have been copied into the ReProcess folder
		# If the file does not exist, then assume that the OLD list was used
		# NOTE: The list read in is a dictionary this needs processed into a list 
		#        before passing it on to the TransferSequence.LoadStepList

		slFile = ("./ReProcess/stimulusList_%s.py" % stimulus )

		exists = os.path.isfile(slFile) 
	
		if exists:
			XXXDebugTrace ("stimulusList read from %s" % slFile )
			try:
				with open (slFile) as f:
					exec (f.read(), inputDictionary)
			except:
				print("Python syntax error in file (%s)" % filename,end='\n',flush= True)
				exit()
				
			if len(inputDictionary['stimulusListUsedDuringTest']) == 0:
				print ("Error: File %s does not contain a dictionary named 'stimulusListUsedDuringTest"% filename,end='\n',flush= True)
				sys.exit()
				
			stimulusListUsedDuringTest =  inputDictionary['stimulusListUsedDuringTest']

		else:
			XXXDebugTrace ("%s  not found. Using hard-coded version" % slFile )

			if stimulus == "PTP":
				stimulusListUsedDuringTest = stimulusListPtp
			elif stimulus == "SYNCE": 
				stimulusListUsedDuringTest = stimulusListSyncE_old
			else:
				logLevel ( "FATAL" ,"unrecognised stimulus %s" %  stimulus)
			
		
	stimulusList = []
	
	if stimulus == "PTP": 
		points = ptpPoints
		stimulusDictionary = stimulusListPtp
	elif stimulus == "SYNCE":
		points = syncEPoints
		stimulusDictionary = stimulusListSyncE
	else:
		logLevel ("FATAL", "unrecognised stimulus - %s" % stimulus)
	

	# Parse the P100G file to extract the stimulus list
	#if {$P100GNoiseTransferFile == TRUE} {
	#	set stimulusList [P100G_parseCaptureFile sequence $stimulusList]
	#}


	#  Work through chosen points from Parameters{} list
	#  start with this as the working list
	
	for p in points:
		selectedStimulusList.append(stimulusDictionary[str(p)])

		
	stimulusList = selectedStimulusList

	if reProcess == "FALSE":
		outputStimulusFileAsPython(sequence, stimulusList)
	
	# Now check and see if it needs overriden by the ReProcess choices
	#

	# if Reprocess TRUE and SimulatedData FALSE then select entries from  the file
	# generated during the actual test
    #  NOTE:  Cannot Guarantee the order of Dictionary entries, so check for ascending frequencies and reorder if necessary
	
	selectedStimulusList = []

	if reProcess == "TRUE" and simulatedData == "FALSE":

		selectedStimulusListDuringTest = OrderedDict()
		
		max = len (stimulusListUsedDuringTest)
		for i in range(max):
			try:
				selectedStimulusList.append(stimulusListUsedDuringTest[str(i)])
			except:
				ignore = True


			
		stimulusList = selectedStimulusList


	if reProcess == "TRUE" and simulatedData != "FALSE":
		 stimulusList = selectedStimulusList
	
	sequence.loadStepList(paragonObject1,stimulusList)

	
	#sets the graphing parameter, called by future scripts
	global graphingEnabled
	if graphing == "TRUE" or graphing == "True":
		graphingEnabled = True
	else:
		graphingEnabled = False

	

		


	log ( "*************************************\n")
	log ("Enhanced Noise Transfer\t%s\n" % scriptVersion)
	log ( "*************************************\n")
	log	("")

	sequence.printParameters()
	
	serialNumber = paragonObject1.getInstrumentSerialNumber()
	version = paragonObject1.getInstrumentVersion()
	
	#####################################
	###  DJWAT REVISIT - IS THIS NEEDED
	#####################################
	###############################################################################
	# Configure graphing
	#------------------------------------------------------------------------------
	# Only include graphing if required - needs a couple of packages installed
	#	set g [dict get $parameters "Graphing"]
	#	if {$g == "TRUE" or $g == "True"} {
	#		XXXDebugTrace "Including graphing.tcl"
	#		source "./graphing.tcl"
	#		# Graph initialisation is done later for P100G
	#		if {$P100GNoiseTransferFile != TRUE} {
	#			configureGraphing [sequence getStimulus] [sequence getMeasurement] [sequence getStepList] $parameters $serialNumber $version [sequence getStartDate] [sequence getStartTime]
	#		} else {
	#			set meas [sequence getMeasurement]
	#			if {$meas == "PTP" or $meas == "BOTH"} {
	#				set capture [sequence p100G_getPtpCapture]
	#			}
	#			if {$meas == "1PPS"} {
	#				set capture [sequence p100G_get1ppsCapture]				
	#			}
	#			configureGraphing [sequence getStimulus] [sequence getMeasurement] [sequence getStepList] $parameters [$capture getSetting serial] [$capture getSetting softwareVersion] [$capture getSetting startTime] [$capture getSetting startTime]
	#		}
	#	}
	#}

	######### BILLY #############################

	limitsOn 			= parameters['LimitsEnabled']
	yAxis				= parameters['GraphingYAxis']

	if graphingEnabled == True:
		XXXDebugTrace ("Graphing set to true")

		#Repeating this as it's in the TCL 
		if P100GNoiseTransferFile == False:
			configureGraphing(sequence.getStimulus(), sequence.getStepList(), sequence.getMeasurement(), limitsOn, yAxis, parameters, serialNumber, version, sequence.getStartDate(), sequence.getStartTime())
		else:
			#P100G stuff in the TCL above to be added at some point if required
			log("Sorry, this version of ENT does not currently support P100G")

	#############################################

def outputStimulusFileAsPython (seq, sList ):
	stim = seq.getStimulus()
	folder = seq.getFolderResults()	
	sListFileName = "stimulusList_%s.py" % stim
	sListPath = "%s/%s"%(folder,sListFileName)
	#		
	#		# If the file already exists, delete it
	if os.path.isfile(sListPath):
		os.remove(sListPath)
		log ("Existing stimulus list file %s has been deleted"%sListPath)
	

	fid = open(sListPath,"a")
	fid.write ("# THIS FILE HAS BEEN GENERATED BY THE G.8273.2 SCRIPT. DO NOT DELETE OR MODIFY\n")
	fid.write ("stimulusListUsedDuringTest = {\n")	


	localList = sList.copy()
	if stimulus == "PTP":
		maxlen = 10
	else:
		maxlen = 12	
	
	
	for i in range(len(sList)):
		if (localList[i]['LL'] == -1 ):
			localList[i]['LL'] = "NA"
		fid.write("\t\t '%d': %s ,\n"%(i,localList[i]))



	fid.write("}")
	fid.flush()
	fid.close()



def doBaselineNoiseGenerationMeasurement (par1,measurement, isDemo ,simulatedData):	
	# Capture duration is set here but should maybe be set at a higher level
	# Packet rate is used to calculate the number of packet we expect in the capture
	# This should probably come from a higher level but right now, hard-code here
	
	revisit = 1
	
	
	captureDuration = 20
	packetRate =16
	# The noiseGeneration function takes a dictionary as input
	# This specifies the capture time and the desired measurements
	noiseGenerationDict = {"captureDuration": captureDuration}
	# We are running at 16pkts/second and so we would expect 20*16 messages
	# Allow for 10% slop
	expectedMessageCount = (captureDuration * packetRate)
	expectedMessageThreshold = (expectedMessageCount - (expectedMessageCount*0.2))

	# In TCL the NoiseGenerationMesDict. contained dictionaries within dictionaries
	# This present problems for Python dictionaries as it requires "key" names for dictionay entries
	# so map this for measurements as Dictionary entries with Lists as values


	noiseGenerationMeasDict= {}

	if measurement == "PTP" or measurement == "BOTH":
		noiseGenerationMeasDict['PTP'] =  ['T4', 'range', 'T4', 'revMessages']		
	
	if measurement == "1PPS" or measurement == "BOTH":
		noiseGenerationMeasDict['1PPS'] = ['1PPS', 'range', '1PPS', 'min', '1PPS', 'max']		
	
	noiseGenerationDict['measurements'] = noiseGenerationMeasDict
	#puts $noiseGenerationDict
	

	cwd =os.getcwd()
	# The noiseGeneration function returns a dictionary containing the requested results
	#set resultsDict [noiseGeneration par1 "[pwd]//[sequence getFolderResults]" "BaselineNoiseGeneration.clxz" $noiseGenerationDict $isDemo "[pwd]//ReProcess/"]
	resultsDict  = noiseGeneration (par1,  "%s/%s" % (cwd, sequence.getFolderResults()), "baselineNoiseGeneration.clxz", noiseGenerationDict, isDemo, "%s/ReProcess/" % cwd)
	
	XXXDebugTrace ("Baseline measurements - %s" %  resultsDict)

	if simulatedData == False:
		if measurement == "PTP" or measurement == "BOTH":
			revMessages = resultsDict['T4revMessages']

			if float (revMessages) < expectedMessageThreshold and (measurement == "PTP" or measurement == "BOTH"):
				logLevel ("FATAL", "The are %s Del-Req/Del-Resp messages in the capture\n \
				There should be approximately %f messages\n \
				Please check your configuration manually before running this script" % (revMessages, expectedMessageCount))
			
		
		if measurement == "1PPS" or measurement == "BOTH":
			oneppsRange = resultsDict['1PPSrange']
			oneppsMin   = resultsDict['1PPSmin']
			oneppsMax   = resultsDict['1PPSmax']

			if float (oneppsRange) < 0.5 and float (oneppsMin) < 0.5 and float(oneppsMin) > -0.5:
				logLevel ("WARNING","The 1pps baseline measurement is questionable\nThis may not be an issue but you should check your results and configuration")
			
		
	else:
		log ("Simulated data - not checking baseline noise measurement...\n")
	

def timeNoiseTransfer( par1, noiseTransferType, demoMode, simulatedData ): 
	global P100GNoiseTransferFile
	P100GNoiseTransferFile = False
	global P100GMeas
	global P100GStimulus
	global graphingEnabled
	global sequence

	stimulus = sequence.getStimulus()
	measurement= sequence.getMeasurement()
	
	#ENT 48
	if demoMode == False:
		par1.paragonset( "MasterSlave Impairments ProfileReplay MasterTx Corruption Enable", "FALSE")
		par1.paragonset( "MasterSlave Impairments ProfileReplay MasterRx Corruption Enable" ,"FALSE")

	if stimulus == "PTP":
		# If PTP is the stimulus, then enable delay impairments
		par1.paragonset( "MasterSlave Impairments VariableDelay MasterTx Enable", "TRUE")
	

	# If we are processing a P100G capture, it contains all the table steps
	# So, load the cdf into the CAT and export the data.
	# As we loop through each point in the table, the relevant section from
	# the capture will be used.
	# Note that the PTP and 1PPS sections here are effectively copies of each other
	# This could be substantially re-factored 
	if P100GNoiseTransferFile  == True:
		#source "P100GCapture.tcl"
		
		# Get the capture file 
		# If the measurement is PTP, then look in ReprocessP100G for Ptp1588100.cdf
		# If the measurement is 1pps, then look in ReporcessP100G for OnePpsAccuracy100.cdf
		# If the measaurement is BOTH, then use both of the above.
		if measurement == "PTP" or measurement == "BOTH":
			measure = "PTP"

			fullResults  = P100G_getFullResults ( sequence.measure())
			P100G_processSequence = sequence.fullResults()

			#set sList [sequence getStepList]
			###set capture [sequence p100G_getPtpCapture]
			# Dirty hack for testing - override the values in the capture. Need to figure out how to deal with these properly		

			#if graphing:
				#savePlotWindow ("P100G/", Results)
				
		
		if measurement == "1PPS" or measurement == "BOTH":    
			measure = "1PPS"

			fullResults = P100G_getFullResults (sequence.measure())
			P100G_processSequence = sequence.fullResults()

			# For testing - override the values in the capture
			#deviceRecoveryTime = 5 
			#guardTime = 15

			#			if measurement != "BOTH":
				#configureGraphing $stimulus $measurement $sList "" [$capture getSetting serial] [$capture getSetting softwareVersion] [$capture getSetting startTime] [$capture getSetting startTime]
			
			

			#if graphing == True:
				#savePlotWindow ( "P100G/", Results)
			
			
			#del capture 
			return
		
		
			
		
	counter = 0

	#
	#    Start MSE first then wait on any device settling Time
	#    ** DO NOT START MSE if in DEMO mode
	#

	if demoMode == False and simulatedData == False:
		# select packet generation with SSM code of QL_PRC
		par1.paragonset ("PacketGeneration #0 Esmc #0 SsmType",  "QL-PRC")
		par1.paragonset ("PacketGeneration #0 Enable", "TRUE")

		captureFilename  = ("%s\%s" % ("\\" + sequence.getFolderResults(),"settlingTime.clxz"))
		startMSEandWaitonSettling(par1, noiseTransferType, sequence.getSettlingTime(),captureFilename,demoMode)
	

	if simulatedData == False:
		log ("\n*** Baseline Noise Generation Measurement ***\n")
		doBaselineNoiseGenerationMeasurement (par1, measurement,  demoMode , simulatedData)
		
	
	log ( "\n*** Stimulus = %s ---> DUT ---> Measure = %s ***\n" % (stimulus, measurement))	
	
	seq = sequence.getSequence()
	i = 0
	#log0 $seq
	for seqStep in seq:

		inputFreq = seqStep.getFreq()
		inputAmpl = seqStep.getAmpl()
		inputSampleRate = seqStep.getSampleRate()
		captureTime = seqStep.getDuration()
		lowerLimit = seqStep.getLL()
		upperLimit = seqStep.getUL()
		log ("\n")
		log ("%s" % currentDateTime() + " Step %d " %counter + "Stimulus:\t%f\t%f\t%d" % (inputFreq,inputAmpl,captureTime))

		tp = sequence.getStepList()
		
		seqStep.startStimulus()
		seqStep.doCapture()
		seqStep.stopStimulus()

		if measurement == "PTP" or measurement == "BOTH":
			measure = "PTP"
			XXXDebugTrace ("Opening CAT (PTP)...")
			openCat (par1, "PTP")

			seqStep.exportFileFromCat ("PTP")

			# Read the results (either from file or from simulated data)
			# This also removes the recovery time
			seqStep.getResults ("PTP")
			
			seqStep.applyFilter ("PTP")
			seqStep.calculateFinalResults ("PTP")
			
			if measurement != "BOTH":
				seqStep.saveAndDisplayFinalResults ("PTP")
			
			log ("%s" % currentDateTime() + "\tStep %d " %counter + measure +"Meas:\t %10.8f" % seqStep.getFreq() + "\t %f" % seqStep.getPtpAmplitudeOut() + "\t  %.2f" % seqStep.getPtpAmplitudeOut() + " \t %.2f" %  seqStep.getPtpGain() + "\t %s" % seqStep.getPtpPassFail())

		
		
		if measurement == "1PPS" or measurement == "BOTH":    
			measure = "1PPS"
			XXXDebugTrace ("Opening CAT (1pps)...")
			openCat (par1, "1PPS")

			seqStep.exportFileFromCat ("1PPS")
			seqStep.getResults ("1PPS")
			seqStep.applyFilter ("1PPS")
			seqStep.calculateFinalResults ("1PPS")
	
			if measurement != "BOTH":
				seqStep.saveAndDisplayFinalResults ("1PPS")
		
			log ("%s" % currentDateTime() + "\tStep %d " %counter + measure +"Meas:\t %10.8f" %  seqStep.getFreq() + "\t %f" % seqStep.getoneppsAmplitudeOut() + "\t  %.2f" % seqStep.getoneppsAmplitudeOut() + " \t %.2f" %  seqStep.getoneppsGain() + "\t %s" % seqStep.getoneppsPassFail())


			par1.clearDictionaryMeasurements()
		 

		if measurement == "BOTH":
			seqStep.saveAndDisplayFinalResults ("BOTH")
		

		counter = counter + 1

    

	#
	#    FINISHED - so stop  MSE  
	# #ENT 48
	if gvar.demoMode == False:
		stopMSE (par1, captureTime) 


	if graphingEnabled == True:
		#gets the csv filename - used to base the .png on
		overall_results_csv = sequence.getOverallResultsFilename()
		savePlotImage(overall_results_csv)
	
	
	log ("\nTest Complete")
	del sequence 



# ############################################################################################################################################################
# Functions to save results to file and to update the UI
def saveResultsToFile ( openedResultsGeneralFile, f, frequency, pkpkAmplitudeIn, ptpAmplitudeOut, ptpGain, oneppsAmplitudeOut, oneppsGain, ll, ul, ptpPass, oneppsPass):

	global graphingEnabled
	
	resultsInfo = ""
	#set file [ string map {".csv" ""} $f ]

	file = f.replace (".csv","")

	# Need to trap LL as string NA, TCL can cope but python cannot
	if ll == 'NA':
		llStr = ll
	else:
		llStr = "%d" % ll	
	
	#if (ptpGain < 0):
	#	ptpGainStr = "-%f"% ptpGain
	#else:
	#	ptpGainStr = "%f" % ptpGain
	

	#f (oneppsGain < 0):
	#	oneppsGainStr = "-%f"% oneppsGain
	#else:
	#	oneppsGainStr = "%f" % oneppsGain
	

	#resultsInfo = "\n" + file + ", %f" % frequency + (", %f" %  pkpkAmplitudeIn) + ", " + ("%.2f" % ptpAmplitudeOut)	+ ", " +  ptpGainStr 
	resultsInfo = "\n" + file + ", %f" % frequency + (", %f" %  pkpkAmplitudeIn) + ", " + ("%.2f" % ptpAmplitudeOut)	+ ", " + "%f" % ptpGain
	#resultsInfo = resultsInfo +  ", " + ("%.2f" % oneppsAmplitudeOut) + ", " + oneppsGainStr + llStr + (",  %d " % ul) + ", "	+ ptpPass + ", " + oneppsPass
	#ENT-47 - added comma in this row
	resultsInfo = resultsInfo +  ", " + ("%.2f" % oneppsAmplitudeOut) + ", " + "%f" % oneppsGain + ", " + llStr + (",  %d " % ul) + ", "	+ ptpPass + ", " + oneppsPass
	
	if graphingEnabled == True:
		updatePtpMeasurement(frequency, ptpAmplitudeOut, ptpGain, ptpPass)
		update1ppsMeasurement(frequency, oneppsAmplitudeOut, oneppsGain, oneppsPass)
	
	openedResultsGeneralFile.write ( resultsInfo)


def savePtpResultsToFile ( openedResultsGeneralFile, f, frequency, pkpkAmplitudeIn, ptpAmplitudeOut, ptpGain, ll, ul, ptpPass):
	global graphing
	
	resultsInfo = ""
	#set file [ string map {".csv" ""} $f ]
	file = f.replace (".csv","")

	# Need to trap LL as string NA, TCL can cope but python cannot
	if ll == 'NA':
		llStr = ll
	else:
		llStr = "%d" % ll	
	

	resultsInfo = "\n" +file + ", %f" % frequency + (", %f" %  pkpkAmplitudeIn) + ", " + "%.2f" % ptpAmplitudeOut + ", " +  "%f" % ptpGain +  ", " + llStr + (",  %d " % ul) + ", " + ptpPass


	if graphingEnabled == True:
		updatePtpMeasurement(frequency, ptpAmplitudeOut, ptpGain, ptpPass)
	
	openedResultsGeneralFile.write(resultsInfo)


def save1ppsResultsToFile ( openedResultsGeneralFile, f, frequency, pkpkAmplitudeIn, oneppsAmplitudeOut, oneppsGain, ll, ul, oneppsPass):
	global graphing
	
	resultsInfo = ""
	#set file [ string map {".csv" ""} $f ]
	file = f.replace (".csv","")

	if ll == 'NA':
		llStr = 'NA'
	else:
		llStr = "%d" % ll	

	resultsInfo = "\n" + file + ", %f " % frequency + (", %f  " % pkpkAmplitudeIn) + ", " + "%.2f"  % oneppsAmplitudeOut + ", " +  "%f" % oneppsGain +  ", " + llStr + ", %d" % ul + ", " +  oneppsPass
	
	XXXDebugTrace ("%s" % resultsInfo)

	if graphingEnabled == True:
		update1ppsMeasurement(frequency, oneppsAmplitudeOut, oneppsGain, oneppsPass)
	
	openedResultsGeneralFile.write( resultsInfo)

# ############################################################################################################################################################

# ############################################################################################################################################################
# Functions to read results from files exported from the CAT

# --------------------------------------------------------------------------------
# Read results from the CAT export
# ---------------------------------------------------------------------------------
# This reads the test results from the csv file exported from the Cat
# The file contains 2 columns: timestamps, data
#
# Input Parameters
# - catFile: 	 	The file name
# - measure:		PTP or 1PPS
# Output
# - data: 		List containing 2 lists. The first is a list of timestamps;
#					the second is a list of amplitude values.
#					For PTP, the timestamps come from the file
#					For 1PPS, the timestamps are generated - one per second
def readResultsFromLocalFile (catFile, measure, freq):
	results = ""
	XXXDebugTrace ("%s %s" % (catFile ,measure))

	if measure == "PTP":
		results = readPtpResultsFromLocalFile (catFile ,freq)
	elif measure == "1PPS":
		results = read1ppsResultsFromLocalFile (catFile ,freq)
	else:
		log ("ERROR: %s  is not a valid measurement type. Aborting." % measure) 
		exit
	

	if (len(results) != 2) or (len(results[0]) <= 1):
		log ("FATAL ERROR (reeadResultsFromLocalFile): ")
		log ("\tThere are insufficient points in the data. Check that your instrument and DUT configuration is correct.")
		log ("\tYou should verify your set up by running a manual test before re-running the script.")
		log ("Aborting...")
		exit
	

	return (results)
	

def readPtpResultsFromLocalFile (catFilename, freq):
	global graphing
	XXXDebugTrace ("%s" %catFilename)

	catFile = open (catFilename, "r")
	x = 0
	y = []
	timesOut = []
	dataOut = []
	index = 0
	lineout = []
	#sets data start line to some chunk into the file, this will be reduced when it's hit
	dataStartLine = 50

	while (1):
		try:
		# Check for an empty file...
			line = catFile.readline()
			
			lineout = line.split(";")
			
		except:
				log ("FATAL ERROR (reeadPtpResultsFromLocalFile): ")
				log ("\tProblem reading %s"  %catFile)
				log ("\tCheck that your instrument and DUT configuration is correct.")
				log ("\tYou should verify your set up by running a manual test before re-running the script.")
				log ("Aborting...")
				catFile.close()
				exit
		


		timeColumnValue = lineout [0]

		#ENT-45 sets the line that data starts, this differs between CATv25 and CATv26 due to header change
		#assumes we will only ever hit one "timestamp" and the data starts on the line after it
		if timeColumnValue == 'timestamp':
			dataStartLine = x

		#puts "$index - t:$timeColumnValue d:$dataColumnValue"
		# Only checking the data column to look for the end of the file. Not sure if checking the timestamp column is necessary.
		if timeColumnValue == ""  or timeColumnValue == "DATA END":
			catFile.close()
			break
		

		dataColumnValue = lineout[1]

		#ENT-45 if we've gone past the header, then append value
		if x > dataStartLine:
			#dict lappend y $timeColumnValue $dataColumnValue
			timesOut.append(float(timeColumnValue) / 1e9)
			dataOut.append (float(dataColumnValue))

			index= index + 1
		
		x= x + 1
	

	# y now contains the the timestamp and measured amplitude in ns for each sampled point
	y.append(timesOut)
	y.append(dataOut)

	#if graphing:
		#updateDataPlot (timesOut, dataOut, freq, "PTP")
	

	return (y)




def read1ppsResultsFromLocalFile (catFilename ,freq):
	global graphing
	XXXDebugTrace ("%s" % catFilename)

	catFile = open (catFilename,"r")
	x = 0
	y = []
	timesOut =[]
	dataOut = []
	index = 0
	lineout = []
	#sets data start line to some chunk into the file, this will be reduced when it's hit
	dataStartLine = 50


	while (1): 

		try:
			line = catFile.readline()
			lineout = line.split(";")				
		except:
			log ("FATAL ERROR (read1ppsResultsFromLocalFile): ")
			log ("\tProblem reading %s" % catFile)
			log ("\tCheck that your instrument and DUT configuration is correct.")
			log ("\tYou should verify your set up by running a manual test before re-running the script.")
			log ("Aborting...")
			catFile.close()
			exit
		#puts $line
		#set line [split $line ";"]
		#set timeColumnValue [lindex $line 0]

		dataColumnValue  = lineout[0]

		#ENT-45 sets the line that data starts, put in case the header size changes in future
		#assumes we will only ever hit one "timestamp" and the data starts on the line after it
		if dataColumnValue == 'value':
			dataStartLine = x

		#puts "$index - t:$timeColumnValue d:$dataColumnValue"
		# Check the data column to look for the end of the file.
		if dataColumnValue == ""  or dataColumnValue == "DATA END":
			catFile.close()
			break
		
	
		# ENT-45 if we've gone past the header, then append value
		if x > dataStartLine:
			#dict lappend y $timeColumnValue $dataColumnValue
			#lappend times [expr $timeColumnValue / 1e9]
			dataOut.append(float(dataColumnValue))

			index = index + 1
			timesOut.append(index)
		
		x = x + 1
	

	# y now contains the the timestamp and measured amplitude in ns for each sampled point
	y.append(timesOut)
	y.append(dataOut)

	#if graphing:
		#updateDataPlot (timesOut, dataOut, freq, "1pps")
	

	return (y)

def removeRecoveryTime (results, t, f, freq, meas): 
	global graphing
	
	rangeOut = []
	#set numSamples [expr {int([expr {$t*$f}])}]
	numSamples = int(t*f)

	timesIn = results [0]
	dataIn = results [1]
	XXXDebugTrace (" (%f %f)  - %d  samples from  %d " % (t, f , numSamples, len(dataIn)))

	if len(dataIn) <= numSamples: 
		logLevel ("FATAL" ,\
		"There are insufficient points in the data. Check that your instrument and DUT configuration is correct.\n"\
		"You should verify your set up by running a manual test before re-running the script.")
	
	#set range [lrange $results $numSamples end]
	#set timeRange [lrange $times $numSamples end]
	#set dataRange [lrange $data $numSamples end]
	

	# Appears to be no equivalent to lrange in python so try for loops

	timeRange = []
	dataRange = []
	endrange =  len(timesIn) 

	count = numSamples
	while count  < endrange:
		timeRange.append(timesIn[count])
		dataRange.append(dataIn[count])
		count = count + 1


	#for  i in range (numSamples,endrange):
	#	#timerange.append(times[i])
	#	#datarange.append(data[i])
	#	timerange.append(times[i])  
	#	datarange.append (data[i])

	XXXDebugTrace (" (%f %f)  - %d  now have %d samples" % (t, f , numSamples, len(dataIn)))



	#puts $range
	rangeOut.append (timeRange)
	rangeOut.append (dataRange)

	#if graphing:
		#updateDataPlot (timeRange, dataRange, freq, meas)
	
	return (rangeOut)


def removeGuardTime (results, t, f, freq, meas):
	global graphing
	
	range = []
	numSamples = int(t*f)
	XXXDebugTrace (" (%f %f ) - %d samples" % (t,f,numSamples))

	times = results[0]
	data = results[1]

	if len(data) <= numSamples:
		logLevel ("FATAL" ,\
		"There are insufficient points in the data. Check that your instrument and DUT configuration is correct.\n/"\
		"You should verify your set up by running a manual test before re-running the script.")
	

	#set range [lrange $results $numSamples end]
	#set timeRange [lrange $times 0 end-$numSamples]
	#set dataRange [lrange $data 0 end-$numSamples]

	# Appears to be no equivalent to lrange in python so try for loops

	endrange = len(times) - numSamples

	for  i in range (endrange):
		timerange.append(times[i])
		datarange.append(data[i])


	#puts $range
	range.append (timeRange)
	range.append (dataRange)

	#if graphing:
		#updateDataPlot (timeRange, dataRange, freq, meas)
	
	return (range)

def generateSimulatedResults (Fsample, freq, amplitude, duration):
	global pi
	global graphing
	XXXDebugTrace ("%f %f %f %f" % (Fsample, freq, amplitude, duration))

	times = []
	data = []
	y = []
	sigma = 6
	N = Fsample * duration

	# This function outputs +/-amplitude whereas we want pk-pk amplitude so half it
	ampl = amplitude / 2
	
	for i in range (N):
		t = i/Fsample
		x = ampl * math.sin(2*pi*freq*t) + 5000
		u  = random.random()*sigma
		v = x+u
		times.append(t)
		data.append(v)
		#lappend y [expr $x+$u]
	
	#return [array get y]
	y.append (times)
	y.append (data)
	#if graphing:
		#updateDataPlot (times, data, freq, "PTP")
	
	
	return (y)


class transferStep():
	global catBaseFile	# The file exported from the CAT
	global catPtpFile		# The file exported from the CAT
	global cat1ppsFile	# The file exported from the CAT
	global resultsFile	# The filename of the filtered data file
	global profileFile	# The filename of the profile to be applied for PTP stimulus
	#global profileFolder	# The folder in which profiles are stored
	global captureFile	# The file where the capture data is stored
	global captureFolder
	global freq
	global ampl
	global sRate
	global duration
	global ll
	global ul

	global recoveryTime
	#global impairmentProfileFile
	#global impairmentProfileFolder
	global stepList
	global parentSequence
	global resultsBeforeFilter
	global resultsAfterFilter
	global ptpAmplitudeOut
	global ptpGain
	global ptpPassFail
	global oneppsAmplitudeOut
	global oneppsGain
	global oneppsPassFail
	global sequence
	
	def __init__(self, paragonObject1, tp,parent):	
		global parentSequence	
		self.stepList = tp
		self.freq = tp['Frequency']
		self.ampl = tp['amplitude']
		self.sRate = tp['samples']
		self.duration = tp['duration']
		self.ll = tp['LL']
		self.ul = tp['UL']
		if "SyncE" not in parent.transferType:
			self.profileFile = tp['PDVFile']
		self.parentSequence = parent
		self.resultsBeforeFilter = "UNDEFINED"
		self.resultsAfterFilter ="UNDEFINED"
		self.par1 = paragonObject1
		
		#set remote [$parent isRemoteHost]
		#set demo [$parent isDemoMode]
		
		self.InitialiseFilesAndFolders()
		
		#if {!$remote} {
		#	set profileFolder [pwd]
		#} else {
		#	set profileFolder [parentSequence getRemoteProfileFolder]
		#}
		#set impairmentProfileFile [lindex $tp 6]

		# For remote export from the CAT, we need to remove periods from freq
		#set freqWithUnderbar [string map {"." "_"} $freq] 		
		#set catFile "NoiseTransfer_$freqWithUnderbar.csv"
		
		#set captureFile ""
		#append captureFile "[$parent getTransferType] NoiseTransfer $freq" "Hz.clxz"

		#set self.captureFolder ""
		#if {!$remote and !$demo} {		
		#	append self.captureFolder [pwd] "/[$parent getFolderResults]/"
		#} elseif {!$remote and $demo} {
		#	append self.captureFolder [pwd] "/ReProcess/"
		#} elseif {$remote} {
		#	append self.captureFolder $remoteCaptureFolder "/"		
		#}
		#XXXDebugTrace "$self.captureFolder $captureFile"

		
		
		#log "Step created: $dataFile"
	
	
	
	def print (self,p):
		 p (self.stepList)
	
	#def getDataFilename {} 		{return $dataFile
	def getResultsFilename(self):
		return self.resultsFile
	def getFreq(self): 				
		return self.freq
	def getAmpl(self):
		return self.ampl
	def getDuration(self):
		return self.duration
	def getSampleRate(self):
		return self.sRate
	def getLL(self):
		return self.ll
	def getUL(self):
		return self.ul	
	def getPtpAmplitudeOut(self):
		return self.ptpAmplitudeOut
	def getPtpGain(self):
		return self.ptpGain
	def getPtpPassFail(self): 		
		return self.ptpPassFail
	def getoneppsAmplitudeOut(self): 	
		return self.oneppsAmplitudeOut
	def getoneppsGain(self):
		return self.oneppsGain
	def getoneppsPassFail(self): 		
		return self.oneppsPassFail

	def startStimulus(self):		
		stimulus = self.parentSequence.getStimulus()
		demo = self.parentSequence.isDemoMode()
		simulatedData = self.parentSequence.getSimulatedData()

		XXXDebugTrace ("%s" %stimulus)
		if stimulus == "PTP":
			#set fileAddress $profileFolder "/" $profileFile
			fileAddress = "%s/%s" % (self.parentSequence.getFolderProfiles(),self.profileFile)
			XXXDebugTrace ("%s" % fileAddress)

			if demo == False and simulatedData == "FALSE":
				self.par1.importimpairmentdata ( 2, fileAddress)

				XXXDebugTrace ("Starting impairment...%s" % fileAddress)
				self.par1.startimpairment()
			
		elif stimulus == "SYNCE":		
			# Convert amplitude from ns to us
			inputAmplWander = self.ampl / 1000
	
			self.par1.paragonset ("WanderGeneration Tolerance Single Frequency",  str(self.freq))
			self.par1.paragonset ("WanderGeneration Tolerance Single Amplitude" , str(inputAmplWander))

			self.par1.paragonset ("WanderGeneration Tolerance Single Enable",  "START")

		else:
			log ("*** ERROR *** startStimulus: Unrecognised stimulus (%s ) - must be PTP or SYNCE. Aborting..." %stimulus)
			exit
		
	

	def stopStimulus(self):
		stimulus = self.parentSequence.getStimulus()
		if stimulus == "PTP":
			XXXDebugTrace ("Stopping impairment...")
			self.par1.stopimpairment()
		else:
			XXXDebugTrace ("Stopping SyncE wander...")
			# This will stop wander generation at the next 360deg zero crossing
			# Should poll here - might take some time
			self.par1.paragonset ("WanderGeneration Tolerance Single Enable", "FORCESTOP")

			loop  = "RUNNING"
			while loop != "IDLE":
				loop =  self.par1.paragonget ("WanderGeneration Tolerance Single State ")
				#log "."
			
		
	

	
	def doCapture(self):
		#global duration, recoveryTime
		global captureFolder, captureFile

		demo = self.parentSequence.isDemoMode()
		simulatedData = self.parentSequence.getSimulatedData()
		self.recoveryTime = self.parentSequence.getRecoveryTime()

		XXXDebugTrace ("Starting packet capture - %s + %s seconds"% (self.duration,self.recoveryTime))
		
		if demo == False and simulatedData == "FALSE":
			packetCapture (self.par1,  (self.duration + self.recoveryTime), demo)
		elif demo == True and simulatedData == "FALSE":	
			# Demo mode: load files from the Demo folder
			
			f = "%s\%s" % (self.captureFolder, self.captureFile)
			if  os.path.isfile(f):
				self.par1.importdata(f)
				#ENT 48
				if gvar.demoMode == False:
					XXXDebugTrace ("Demo Mode - Stopping MSE...")
					self.par1.paragonset ("MasterSlave Master #0 Enabled",  "FALSE")
			else:
				logLevel ("FATAL"," %s  does not exist" % f)
				exit
			
		

		# explicitly stop capture, possible differences in GUI behavious
		# just in case catch, should an older version throw an error on stopping an already stopped capture
		XXXDebugTrace ("Stopping packet capture...")
		try: 
			self.par1.stopcapture()
		#			puts "capture stopped"
		except:
			i = i +1   # dummy catch , just don't want scrip to fail if stop capture fails

		if demo == False and simulatedData == "FALSE":
			
			fileName = "%s\%s" % (self.captureFolder, self.captureFile)
			#append f [pwd] "/$folder/" $self.captureFile
			
			XXXDebugTrace ("Exporting capture to %s" % fileName )
			try:

				self.par1.exportdata(fileName)
			
			except:
				isrunning = self.par1.paragonget( "InstrumentStatus Capture IsRunning")

				log ("Failed to export capture file %s (capture status: %s). This is not fatal - continuing execution." % (self.captureFile,isrunning))
			
	
	# Measurement must be PTP or 1PPS
	def exportFileFromCat (self,ptp_or_1pps): 
		global P100GNoiseTransferFile

		simulatedData = self.parentSequence.getSimulatedData()

		if simulatedData == "FALSE":
			if ptp_or_1pps == "PTP":
				meas = "T4"
				file = self.catPtpFile
			elif ptp_or_1pps == "1PPS":
				meas = "1PPS"
				file = self.cat1ppsFile
			else:
				log ("*** ERROR *** exportFromCat: %s  is not a valid measurement. Aborting..." % ptp_or_1pps)
				exit
			
			XXXDebugTrace ("%s, %s" %(file, meas))
		
			try:
				
				self.par1.paragonset ("Cat SelectSlot", meas )

			except:
				log ("FATAL ERROR (exportFileFromCat): ")
				log ("\tProblem selecting $meas in CAT")
				log ("\tCheck that your instrument and DUT configuration is correct.")
				log ("\tYou should verify your set up by running a manual test before re-running the script.")
				log ("Aborting...")
				exit			
			
			
		
		#		set newCatFile $catFile
		# Remove the .csv
		#		set newCatFile [ string map {".csv" ""} $catFile ]
		#		set catFilePostfix "_$meas"
		#		append catFilePostfix "_Cat.csv"
		#puts $catFilePostfix
		#		append newCatFile $catFilePostfix
		#set catFile [ string map {".csv" "_Cat.csv"} $catFile ]

			catFileAddress = ""

			remote = self.parentSequence.isRemoteHost()

			if remote == False:
				# If the script isn't running on a remote host, then provide the full path to the results
				catFileAddress = "%s/%s/%s" % (os.getcwd(), self.parentSequence.getFolderResults(), file)
			else:
				# If the script is running on a remote host, then just use the filename
				# In this case, the file will be written to the default location - CAT/Export
			
			#######################
			##  DJWAT REVISIT
			#######################
			
			#	append catFileAddress [file tail $file]
				catFileAddress = file
	
		#	
		#	GM Changes
		#	
		#	#self.par1 paragonset Cat TimeErrorCsv Export $catFileAddress;      # GM: This should be the full data export. Not supported in v18.00

		#	self.par1 paragonset Cat SelectSlot "2Way"
		#	self.par1 paragonset Cat TIMEERROR ExportFullCsv $catFileAddress;  # GM: Trace data
	
			#######################
			##  DJWAT REVISIT
			#######################
			
			#XXXDebugTrace "exportFileFromCat: $catFileAddress"
			XXXDebugTrace ("exportFileFromCat: catFileAddress")
			
			#  end of revisit  i.e comment it out again
			catFileAddress = catFileAddress.replace("\\","\/")
			self.par1.paragonset( "Cat TimeErrorCsv Export", "%s" %catFileAddress)
	
		# Dirty hack - CAT seems to be exporting to its own export folder.
		# Not sure why this isn't taking the full path but hack it for the moment
			#######################
			##  DJWAT REVISIT
			#######################
		#	if P100GNoiseTransferFile
		#		set exportedFile $env(HOME)
		#		append exportedFile "/Documents/Calnex/Cat/Exports/"
		#		append exportedFile [file tail $catFileAddress]
		#		XXXDebugTrace "$exportedFile"
		#		file rename -force $exportedFile $catFileAddress
			
		
	
	
	def getResults (self,ptp_or_1pps):

		if ptp_or_1pps == "PTP":
			meas = "T4"
			file  = self.catPtpFile
			sampleRate = 16.0
		elif ptp_or_1pps == "1PPS":
			meas = "1PPS"
			file = self.cat1ppsFile
			sampleRate = 1.0
		else:
			log ("*** ERROR *** getResults: %s  is not a valid measurement. Aborting..." % ptp_or_1pps)
			exit
		
		#XXXDebugTrace "getResults: $file, $meas"

		simulatedData = self.parentSequence.getSimulatedData()

		if simulatedData != "GENERATE":
			#set results [readResultsFromFile $catFile $measure $inputFreq]
			results = self.ReadResultsFromFile (ptp_or_1pps)
			XXXDebugTrace ("Using data from CAT")
		else:
			results = self.generateSimulatedResults (self.sRate, freq, ampl, self.sduration)
			XXXDebugTrace ("Using simulated data")
		
			
		# Remove a specified amount of data from the capture prior to filtering
		# This allows the device to recover after being hit by the noise
		# The recovery time should be set as a parameter
		#set results [removeRecoveryTime $results $deviceRecoveryTime $inputSampleRate $inputFreq $measure]
		self.resultsBeforeFilter = removeRecoveryTime (results, self.parentSequence.getRecoveryTime(), sampleRate,  self.getFreq(), meas)
	
	
	
	#  Download file from PC running remote Paragon-X GUI/Cat
	
	def download_file(self,folder_type, src_folder, filename, RemoteIpAddr, dest_folder):
		CONTENT_BIN = r"application/octet-stream"
		CONTENT_TEXT = r"text/csv"


		INST = "http://" + RemoteIpAddr + ":1337/api/"

		content_type = ""
		fid = None

		remote_file = src_folder + "/" + filename
		url = "cat/filecommander/download/" + folder_type + "?AsAttachment=true&FileId=" + filename

		local_file = "%s\%s" %(dest_folder , filename)
    
		if folder_type == "SessionsFolder" or folder_type == "ReportFolder":
			content_type = CONTENT_BIN
		elif folder_type == "ExportFolder":
			#content_type = CONTENT_TEXT
			content_type = CONTENT_BIN
		else:
			content_type = CONTENT_TEXT
    		
		try:
			response = requests.get(
           		        "{0}{1}".format(INST, url),
               		    data="",
                   		headers={'Content-Type': content_type})
			response.raise_for_status()
			if content_type == CONTENT_BIN:
				fid = open(local_file, "wb")
				fid.write(response.content)
			else:
				fid = open(local_file, "w")
				# The request function is adding extra CR  '\r', need to remove them
				#outStr=""
				#outStr = str(response.text)
				#outStr = outStr.replace('\r','')
				#fid.write(outStr)
				fid.write(response.content)
		except Exception as exc:

			print(exc)
			print("Unable to download '" + filename + "' from '" + src_folder + "'")

		if fid != None:
			fid.close()
	
	
	# Private
	def ReadResultsFromFile (self,ptp_or_1pps):
		global guiHostname
		global cat1ppsFile


		remote = self.parentSequence.isRemoteHost()
		measure = self.parentSequence.getMeasurement()
		resultsFolder = self.parentSequence.getFolderResults()
		simulatedData  = self.parentSequence.getSimulatedData()

		if ptp_or_1pps == "PTP":
			meas = "T4"
			file = self.catPtpFile
		elif ptp_or_1pps == "1PPS":
			meas = "1PPS"
			file = self.cat1ppsFile
		else:
			logLevel ("FATAL", "%s  is not a valid measurement" % ptp_or_1pps)
			exit
		
		XXXDebugTrace ( "%s, %s" %  (file, meas))
		
		fileAddress = "%s\%s" % (resultsFolder, file)
		
		if simulatedData != "FALSE" and simulatedData != "GENERATE" and simulatedData != "TRUE":
			# We are evaluating noise performance
			# Load a file that has the noise type included just before the file extension
			#set filenameNoExt [file rootname $file]
			#set fileExt [file extension $file]


			filenameNoExt, fileExt = os.path.splitext(file)
			
			newFilename = "%s_%s %s" % (filenameNoExt, simulatedData ,fileExt)
			XXXDebugTrace ( "Simulated data - %s, %s" % (newFilename, meas))
			
			fileAddress = "%s\ReProcess\%s" % (os.getcwd(),newFilename)
			
		
		
		#XXXDebugTrace ("ReadResultsFromFile: remoteHost %s, %s" % (remote, fileAddress))
	
		if remote == False:
			return (readResultsFromLocalFile (fileAddress, ptp_or_1pps, self.freq))
		else:
			# The P-X GUI and CAT are hosted on a remote PC
			# The file has been exported on this PC to the CAT/Export folder - CHECK THIS
			# Use the CAT REST API to get the contents of the file
			# THis will be returned in the same format as the csv and so we should process it the same way
			XXXDebugTrace ("Fetching results from remote PC...")
			self.download_file("ExportFolder", ".", os.path.basename(fileAddress), gvar.parameters['GuiIp'] , resultsFolder)
			return (readResultsFromLocalFile (fileAddress, ptp_or_1pps, self.freq))



	def applyFilter (self,ptp_or_1pps):

		if self.resultsBeforeFilter == "UNDEFINED":
			log ("resultsBeforeFilter not defined. Aborting...")
			exit
		

		filter = self.parentSequence.getFilterType()
		stim = self.parentSequence.getStimulus()
		resultsFolder = self.parentSequence.getFolderResults()

		resultsPath = "%s\%s" % (resultsFolder,self.resultsFile)

		XXXDebugTrace ("%s %s %s %s" % (stim, filter, self.resultsFile, ptp_or_1pps))
		if filter == "LSQ":
			if ptp_or_1pps == "PTP":
				self.resultsAfterFilter = filterLeastSquares (stim, self.resultsBeforeFilter, resultsPath, self.freq, self.ampl,self.sRate)
			else:
				self.resultsAfterFilter = filterLeastSquares (stim, self.resultsBeforeFilter, resultsPath, self.freq, self.ampl, 1.0)
			
		elif filter == "IIR":
			if ptp_or_1pps == "PTP":
				# Get the sample rate - this is needed for the IIR filter
				# We assume that the sample rate is the equivalent of the Sync rate. For MSE BC testing, this is probably reasonable
				self.par1.addMeasurement ("measTE2Way", "TE", "2Way")
				measTE2Way =  self.par1.getMeasurement ("measTE2Way" )
				measTE2WayResult = measTE2Way.createResult()
				inputSampleRate = measTE2WayResult.getForwardRate()
				#log $max_min
				self.resultsAfterFilter = filterIIR (stim, file, self.resultsBeforeFilter, self.resultsFile, self.freq, self.ampl, self.sRate)
			else:
				log ("ERROR: Unsupported filter for 1pps. Filter type: %s. Aborting..." % filter)
				exit
			 
		elif filter == "NONE":
			if ptp_or_1pps == "PTP":
				# No filtering so just get the max-min result for T4 from the CAT
				self.par1.addMeasurement ("measTET4", "TE", "T4")
				measTET4 =  self.par1.getMeasurement ("measTET4" )
				measTET4Result = measTET4.createResult()
				max_min = measTET4Result.getMax_Min()
				#log $max_min
				# Create the result list (bit artifical but it maintains the filter interface)
				self.resultsAfterFilter=  [ self.ampl, max_min, self.freq]
			else: 
				# No filtering so just get the max-min result for 1pps from the CAT
				y = self.resultsBeforeFilter[1]
				# Result is the range of the data
				#puts [getMax $y]
				#puts [getMin $y]
				max_min = max(y) - min(y)
				#log $max_min
				# Create the result list (bit artifical but it maintains the filter interface)
				self.resultsAfterFilter =  [self.ampl, max_min,self.freq]
			
		else:
			log ("ERROR: Unrecognised filter type: %s. Aborting..." % filter)
			exit
		

	

	
	def calculateFinalResults (self,ptp_or_1pps):
		#XXXDebugTrace ("calculateFinalResults: %s" % ptp_or_1pps)
		if self.resultsAfterFilter == "UNDEFINED":
			log ("resultsAfterFilter not defined. Aborting...")
			exit
		

		measurement = self.parentSequence.getMeasurement()

		if ptp_or_1pps == "PTP":
			self.ptpAmplitudeOut = self.resultsAfterFilter[1] 
			#XXXDebugTrace ("calculateFinalResults: %s" %  self.ptpAmplitudeOut)
	    
			# Calculate the gain
			if  self.ptpAmplitudeOut == 0:
				self.ptpGain = -100.00
			else:
				self.ptpGain =  (float) (20 * math.log10( self.ptpAmplitudeOut / self.ampl)	)		
			
			#XXXDebugTrace ("calculateFinalResults: %s" % self.ptpGain)
		
			# Work out pass/fail
			# There is one pass/fail column: pass, fail (lower), fail (upper)
			self.ptpPassFail = self.DeterminePassFail ( self.ptpAmplitudeOut)
			#XXXDebugTrace ("calculateFinalResults: %s" % self.ptpPassFail)
		
			# save results to file 
			#			if measurement != "BOTH":
			#				#savePtpResultsToFile (openedFileResultsGeneral, file, frequency, pkpkAmplitudeIn,  self.ptpAmplitudeOut, self.ptpGain, lowerLimit, upperLimit, self.ptpPassFail)
			#				savePtpResultsToFile (sequence.getFidResults(), self.resultsFile, freq, ampl , self.ptpAmplitudeOut, self.ptpGain, ll, ul, self.ptpPassFail)
			#			
		elif ptp_or_1pps == "1PPS":
			self.oneppsAmplitudeOut  = self.resultsAfterFilter[1] 
			# calculate gain
			if self.oneppsAmplitudeOut == 0:
				self.oneppsGain = -100.00
			else:
				self.oneppsGain = (float) (20 * math.log10(self.oneppsAmplitudeOut / self.ampl))
			
			#print ("After gain calc AmplitudeIn $f amplitudeOut  $f  frequency $f gain $f" % (pkpkAmplitudeIn,amplitudeOut,frequency,gain")) 

			# Work out pass/fail
			# There is one pass/fail column: pass, fail (lower), fail (upper)
			self.oneppsPassFail = self.DeterminePassFail (self.oneppsAmplitudeOut)
		
			# save results to file 
			#			if measurement != "BOTH":
				#save1ppsResultsToFile (openedFileResultsGeneral, file, frequency, pkpkAmplitudeIn, oneppsAmplitudeOut $oneppsGain $lowerLimit $upperLimit $oneppsPassFail
			#				save1ppsResultsToFile (sequence.getFidResults(), self.resultsFile, freq, ampl, oneppsAmplitudeOut, oneppsGain, ll, ul, oneppsPassFail)
			#			}
		
		else:
			log ("calculateFinalResults: Argument (%s) not supported. Aborting..." % ptp_or_1pps)
			exit
		
	
	
	
	def saveAndDisplayFinalResults (self,meas):
		global sequence
		#XXXDebugTrace( "saveAndDisplayFinalResults: %" % ptp_or_1pps)
		if meas == "PTP":
			savePtpResultsToFile (sequence.getFidResults(), self.resultsFile, self.freq, self.ampl,  self.ptpAmplitudeOut, self.ptpGain, self.ll, self.ul, self.ptpPassFail)
		elif meas == "1PPS":
			save1ppsResultsToFile (sequence.getFidResults(), self.resultsFile, self.freq, self.ampl, self.oneppsAmplitudeOut, self.oneppsGain, self.ll, self.ul, self.oneppsPassFail)
		elif meas == "BOTH":
			saveResultsToFile (sequence.getFidResults(), self.resultsFile, self.freq, self.ampl,  self.ptpAmplitudeOut,  self.ptpGain, self.oneppsAmplitudeOut, self.oneppsGain, self.ll, self.ul, self.ptpPassFail, self.oneppsPassFail)		
		else:
			log ("calculateFinalResults: Argument (%s) not supported. Aborting..." % meas)
			exit
		

	
	
	
	
	
		# Private
	def DeterminePassFail (self,invalue):
		
		if self.ll != "NA" and invalue < self.ll:
			return ("FAIL(lower)")
		if self.ul != "NA" and invalue > self.ul: 
			return ("FAIL(upper)")
		return ("PASS")
	

		# Private
	def InitialiseFilesAndFolders(self):
		global parentSequence
		global freq
		global demo
		global remote
		global captureFolder, captureFile
		global catPtpFile, cat1ppsFile

		remote = self.parentSequence.isRemoteHost()
		demo = self.parentSequence.isDemoMode()

		#		if {!$remote} {
		#			set profileFolder [pwd]
		#		} else {
		#			set profileFolder [$parentSequence getRemoteProfileFolder]
		#		}

		# For remote export from the CAT, we need to remove periods from freq
		freqWithUnderbar = str(self.freq)
		freqWithUnderBar = freqWithUnderbar.replace ("." ,"_") 		
		self.catBaseFile = "NoiseTransfer_" + freqWithUnderbar + ".csv"
		

		self.resultsFile = self.catBaseFile.replace (".csv", "_Results.csv") 

		# Create filenames of each possible measurement
		# Remove the .csv
		self.catPtpFile = self.catBaseFile.replace(".csv", "")
		#catFilePostfix = "_T4"
		#append catFilePostfix "_Cat.csv"
		#append self.catPtpFile $catFilePostfix

		self.catPtpFile = self.catPtpFile + "_T4_Cat.csv"

		self.cat1ppsFile =self.catBaseFile.replace(".csv", "") 

		#set catFilePostfix "_1PPS"
		#append catFilePostfix "_Cat.csv"
		#append self.cat1ppsFile $catFilePostfix
		
		self.cat1ppsFile = self.cat1ppsFile + "_1PPS_Cat.csv"
		
		#self.captureFile =""
		#append self.captureFile "[$parentSequence getTransferType] NoiseTransfer $freq" "Hz.clxz"

		self.captureFile = self.parentSequence.getTransferType() + "NoiseTransfer" + "%f"%self.freq + "Hz.clxz"

		self.captureFolder = ""
		if remote == False and demo == False:		
			self.captureFolder = os.getcwd() + "\\" + self.parentSequence.getFolderResults() + "\\"
		elif remote == False and demo == True:
			self.captureFolder = os.getcwd() + "\ReProcess\\"
		elif remote == True:
			#append self.captureFolder $remoteCaptureFolder "\"		
			self.captureFolder = self.parentSequence.getRemoteCaptureFolder()
		
		#XXXDebugTrace ("%s %s" % (captureFolder, captureFile))

	
class  transferSequence():
	global folderProfiles
	global folderResults 
	global folderTestRunResults
	global fileOverallResults
	global fidOverallResults
	global transferType
	global stimulus
	global measurement
	global parametersDictionary
	global dateStart
	global timeStart
	global stepList
	global remoteHost
	global remoteCaptureFolder
	global remoteProfileFolder
	global P100G_ptpCapture
	global P100G_1ppsCapture
	global recoveryTime
	global guardTime
	
# local context
	demoMode = False
	P100G_ptpCapture = False
	P100G_1ppsCapture = False
	
	# now initialise them 

	folderResults = "Results"
	folderTestRunResults ="UNDEFINED"
	fileOverallResults = "UNDEFINED"
	fidOverallResults = 0
	transferType = "UNDEFINED"
	stimulus = "UNDEFINED"
	measurement = "UNDEFINED"
	parametersDictionary = {}
	stepList=[]
	#remoteHost = False
	remoteCaptureFolder = "UNDEFINED"
	remoteProfileFolder = "UNDEFINED"


	recoveryTime  = 20
	guardTime = 20
	

	def __init__(self,paragonObject1, noiseTransferType,params,demo,remote):
		global parametersDictionary
		global stimulus

		self.activesequence = []
		#XXXDebugTrace "transferSequence: creating... "
		self.parametersDictionary = params
		self.remoteHost = remote
		self.demoMode = demo
		self.paragonOject = paragonObject1
		
		systemTime = gmtime()
		self.dateStart = strftime("%Y-%m-%d", gmtime())
		self.timeStart = strftime("%H-%M-%S",gmtime())

		self.transferType = noiseTransferType

		self.recoveryTime = self.parametersDictionary ['RecoveryTime']
		self.guardTime = "UNDEFINED"
		
		self.InitialiseFilesAndFolders()
		
		#		# Profiles folder
		#		if {!$remote} {
		#			set folderProfiles [pwd]
		#		} else {
		#			set folderProfiles [dict get $parametersDictionary "RemoteProfileFolder"]
		#		}

		#		# Results folder and sub-folders
		#		set folderResults "Results"		

		#		if {!$demoMode} {
		#			set folderTestRunResults "Results $noiseTransferType $dateStart-T-$timeStart"
		#		} else {
		#			set folderTestRunResults "Results $noiseTransferType Demo"
		#		}
		#		set fileOverallResults "$folderResults/$folderTestRunResults/Results $dateStart-T-$timeStart.csv"

		#		createResultsFolders $folderResults $folderTestRunResults

		log ("\nStart: %s %s\n" % (self.dateStart ,self.timeStart))
		#		log "Results will be stored in:\t\"$folderResults\\$folderTestRunResults\""
		
		if "PTP-" in self.transferType:
			self.stimulus = "PTP"
		elif  "SyncE-" in self.transferType:
			self.stimulus="SYNCE"
		

		if "-PTP" in self.transferType:
			self.measurement = "PTP"
		elif "-1PPS" in self.transferType:
			self.measurement = "1PPS"
		elif "-BOTH" in self.transferType:
			self.measurement = "BOTH"
		
		return 
	#		set fidOverallResults [ open $fileOverallResults "a"]
	#		my WriteHeaderToResultsFile
	
	def __del__(self):
		global fidOverallResults
		self.fidOverallResults.close()
	
	def loadStepList (self,paragonObject,sList): 
		self.stepList = sList
		counter = 0

		# Dictionary to be used to contain all of the created instances  has to be big enough for the largest number
		#             of entries. This is SyncE-SYNCE at 11

		Steps= {'Step0':0,'Step1':0,'Step2':0,'Step3':0,'Step4':0,'Step5':0,'Step6':0,'Step7':0,'Step8':0,'Step9':0,'Step10':0,'Step11':0}
		for tp in sList:

			Steps['Step%d'%counter] = transferStep (paragonObject,tp , self) 
			s =  Steps['Step%d'%counter] 

			self.addStep(counter,s)
			counter = counter + 1
	


	def getStepList(self):
		return self.stepList
	
	###########################################################
	# ## DJWAT REVISIT
	# ######################################
	# 
	# def outputStepListAsTcl {} {
		#set sListFilename "stimulusList.tcl"
	#	set sListFilename "stimulusList_$stimulus.tcl"
	#	set sListPath "$folderResults/$folderTestRunResults/$sListFilename"
		
	#	# If the file already exists, delete it
	#	if {[file exists $sListPath]} {
	#		file delete -force $sListPath
	#		log "Existing stimulus list file \"$sListPath\" has been deleted"
	#	}
	#
	#		set fid [open $sListPath "a"]
	##		
	#		puts $fid "# THIS FILE HAS BEEN GENERATED BY THE G.8273.2 SCRIPT. DO NOT DELETE OR MODIFY"
	#		puts $fid "set stimulusListUsedDuringTest \{ "
	#		foreach tp $stepList {
	#			puts -nonewline $fid "\{ "
	#			puts -nonewline $fid "[lindex $tp 0] "
	#			puts -nonewline $fid "[lindex $tp 1] "
	#			puts -nonewline $fid "[lindex $tp 2] "
	#			puts -nonewline $fid "[lindex $tp 3] "
	#			puts -nonewline $fid "[lindex $tp 4] "
	#			puts -nonewline $fid "[lindex $tp 5] "
	#			puts $fid "[lindex $tp 6] \}"
	#		}
	#		puts $fid " \}"
	#		flush $fid
	#		close $fid
	#	}
	
	def addStep (self,index,  step):

		#puts "adding step: $step"

		self.activesequence.append(step)
		#puts $activesequence
	
	def getSequence(self):

		return self.activesequence
	
	def print (self,p):

		for s in self.activesequence:
			i = s[0]
			step = s[1]
		#
		#  DJWAT REVISIT
		# 
		# 			$step print $p
		
	
	

	def p100G_setPtpCapture(self,c): 
		self.P100G_ptpCapture = c
		
	def p100G_getPtpCapture(self): 
		return self.P100G_ptpCapture

	def p100G_set1ppsCapture(self,c): 
		self.P100G_1ppsCapture = c
	
	def p100G_get1ppsCapture(self): 	
		return self.P100G_1ppsCapture
	
	def getTransferType(self):
	 	return self.transferType

	def getStimulus(self): 		
		return self.stimulus

	def getMeasurement(self): 	
		return self.measurement
	
	def getStartTime(self): 		
		return self.timeStart
	
	def getStartDate(self): 		
		return self.dateStart

	def getSettlingTime(self): 	
		return self.parametersDictionary ["SettlingTime"]
	
	def setRecoveryTime (self,t): 	
		self.recoveryTime = t

	def getRecoveryTime(self): 
		return self.recoveryTime
	
	def setGuardTime (self,t): 
		self.guardTime = t
	
	def getGuardTime(self): 	
		return self.guardTime

	def getFilterType (self): 	
		return self.parametersDictionary["Filter"]
	
	def getFidResults(self): 	
		return self.fidOverallResults
	
	def getFolderResults(self): 	
		return "%s\%s" % (self.folderResults,self.folderTestRunResults)

	def getOverallResultsFilename(self):
		return self.fileOverallResults

	def printParameters(self):
		log ("Transfer Sequence Parameters:")
		
		ord = OrderedDict (sorted(self.parametersDictionary.items()))
		for key, value in ord.items():

		# if Port1Config is TRUE then Port2Config is also TRUE
		# Need to stop duplicates of Interface,AutoNegotiate and LineRate for
		# uncoupled (Port1Config)  and coupled values	

			if  (key == 'LineRate' or key == 'Interface' or key == 'AutoNegotiate'):
				if gvar.debugTrace == True:
					log ("\tPort %s : %s " % (key, value))
				else:
					log ("\tPort %s" % key)

			else:

				if type(value) is str:	
					log ("\t%s : %s" % ( key , value))
				elif type(value) is int:
					log ("\t%s : %d" % ( key , value))
				elif type(value) is float:		
					log ("\t%s : %f" % ( key , value))
				elif type(value) is list:
					log ("\t%s : %s" % ( key , value ))



	def getDataFilename(self):
		#		return $dataFile
		self.fileName = ""
		return (self.fileName)

	def getResultsFilename(self):
	#		return $resultsFile
		self.resultsFile = ""
		return (self.resultsFile)	

	def isRemoteHost(self):
		return self.remoteHost
	
	def getRemoteCaptureFolder(self):  	
		return self.parametersDictionary ["RemoteCaptureFolder"]

	def getSimulatedData(self):  			
		return self.parametersDictionary ["SimulatedData"]

	def getFolderProfiles(self): 		
		return self.folderProfiles
		
	def isDemoMode(self): 				
		return self.demoMode


	
		# Private
	def WriteHeaderToResultsFile(self):
		global fidOverallResults
		#set systemTime [clock seconds]
		#set d [clock format $systemTime -format "%d %b %Y"]
		#set t [clock format $systemTime -format %H:%M:%S]
		
		resultsInfo = ""
		if self.transferType == "PTP-BOTH" or self.transferType == "SyncE-BOTH":

			# ENT-47, 1pps pass field to end and explcitly mention 1pps pass and ptp pass
			self.resultsInfo = "\nNoise Transfer %s, %s %s \n Profile ,Freq (Hz) , Amplitude (p-p ns) , PTP Measured (p-p ns)   ,  PTP Gain (dB)  , 1pps Measured (p-p ns) ,  1pps Gain (dB)  ,  Lower Limit (p-p ns) , Upper Limit (p-p ns)  , PTP Pass/Fail, 1pps Pass/Fail"  % (self.transferType, self.dateStart, self.timeStart)
		
		elif self.transferType == "PTP-PTP" or self.transferType == "SyncE-PTP":

			self.resultsInfo  = "\nNoise Transfer %s, %s %s \n Profile ,Freq (Hz) , Amplitude (p-p ns) , PTP Measured (p-p ns)   ,  PTP Gain (dB)  , Lower Limit (p-p ns)   ,   Upper Limit (p-p ns)   ,   Pass/Fail"  % (self.transferType, self.dateStart, self.timeStart)
		
		elif self.transferType == "PTP-1PPS" or self.transferType == "SyncE-1PPS":
		
			self.resultsInfo =  "\nNoise Transfer %s, %s %s \n Profile ,Freq (Hz) , Amplitude (p-p ns) ,   1pps Measured (p-p ns)  ,   1pps Gain (dB)   ,   Lower Limit (p-p ns)   ,   Upper Limit (p-p ns)   ,   Pass/Fail" % (self.transferType, self.dateStart, self.timeStart)
		
		else:
			logLevel ( "FATAL", "WriteHeaderToResultsFile: Unknown Type - %s . Aborting..." % self.transferType)
			exit
		
		self.fidOverallResults.write( self.resultsInfo)

	# Private
	def InitialiseFilesAndFolders(self):
		global fidOverallResults

		# Profiles folder
		if self.remoteHost == False:
			self.folderProfiles = os.getcwd()
		else:
			self.folderProfiles = self.parametersDictionary ["RemoteProfileFolder"]
			

		# Results folder and sub-folders
		self.folderResults = "Results"		

		if self.demoMode == False:
			self.folderTestRunResults = ("Results_%s_%s-T-%s" %  (self.transferType ,self.dateStart,self.timeStart))
		else:
			self.folderTestRunResults = ("Results_%s_Demo" % self.transferType )

		self.fileOverallResults = ("%s/%s/Results_%s-T-%s.csv" % (self.folderResults, self.folderTestRunResults,self.dateStart,self.timeStart))

		#createResultsFolders $folderResults $folderTestRunResults
		if  os.path.exists(self.folderResults) == False :
			os.mkdir(self.folderResults)
			
		

		self.folder = ("%s/%s" % (self.folderResults, self.folderTestRunResults))
	
			# If we are trying to create a folder for a demo run, then, if it already exists, delete it
		if ("*Demo" in self.folderTestRunResults) and (os.path.exists(self.folder)): 
			os.remove(folder)
			log ("Existing demo folder %s has been deleted" % self.folder)
				

		if os.path.exists(self.folder) == False:
			os.mkdir( self.folder )
		else:
			log ("%s  already exists. Aborting." % self.folder )
			exit
			

		self.fidOverallResults = open (self.fileOverallResults, "a")
		self.WriteHeaderToResultsFile()

		log ("Results will be stored in:\t\%s\%s"% (self.folderResults, self.folderTestRunResults))
	


