# script file for noise transfer in TCL
# Note:
# This function assumes a sample rate of 16 pkts/sec.
# It is likely that for real devices, the sample rate will not be exactly 16.
# The fix is probably to calculate the sample rate across the capture or make use of the timestamps directly.
# Not sure if the latter is possible.
import math

def filterIIR  (stimulus,testFile,results: list ,resultsFile,inputFreq,inputAmpl,inputSampleRate ):

	d1 = []


	XXXDebugTrace ("Using IIR filter... %f %f %f"  % (inputFreq, inputAmpl,inputSampleRate))

	# open file to write
	fileId = open (resultsFile ,"a")

	intro = "t1, d1,filtered_data_q,filtered_data2_q,filtered_data3_q,filtered_data_i,filtered_data2_i,filtered_data3_i," 
	fileId.write( intro )

	pi = 3.1415926535897931

	inputAmplitude = inputAmpl 
	inputFrequency = inputFreq
	Sample_Rate_Hz = inputSampleRate 


	d1 = results[1]
	#set d1 $results

	XXXDebugTrace ("filterIIR: Initialising...")
	Wc = (2*pi*inputFrequency)

	# Data is now coming in as a list (not an array)
	# Converting a list to array appears to be non-trivial
	# It looks as though initialising I and Q to the data values should not be required so skipping it - if anything looks odd though, this would be the palce to look
	#array set Q1 [array get d1]
	#array set I1 [array get d1]
	
	Low_Pass_Frequency_Hz = 0.01
	# 3dB radian frequency
	W3=  ((Low_Pass_Frequency_Hz / Sample_Rate_Hz) * 2 * pi)  
	# Scaled 3dB radian frequency  
	Ohmega3 = math.atan(W3/2)

	# Compute the digital filter coefficients
	A0 = (Ohmega3 / (1.0 + Ohmega3))
	A1 = A0
	B1 = -((Ohmega3 - 1.0)/(1.0+ Ohmega3))

	# Set up temporary variables to hold filter data values
	previous_input_i = 0
	previous_output_i = 0

	# Temp variables for 2nd stage filter I values
	previous_input2_i = 0
	previous_output2_i = 0

	# Temp variables for 3rd stage filter values
	previous_input3_i = 0
	previous_output3_i = 0


	previous_input_q = 0
	previous_output_q = 0

	# Temp variables for 2nd stage filter Q values
	previous_input2_q = 0
	previous_output2_q = 0

	# Temp variables for 3nd stage filter Q values
	previous_input3_q = 0
	previous_output3_q = 0

	# Variables to hold the sum of I and Q
	I_SUM = 0.0
	Q_SUM = 0.0

	# No of samples in the average
	SUM_COUNT = 0


###################################
###  DJWAT ADD
#####################################

	XXXDebugTrace ("filterIIR: Filtering... %s samples" % len(d1))

	n = (len(d1) - 1)

	#for { set i 0}  {$i < [array size d1]} {incr i} 

	d = 0 

	for i in range (n):

# Now do the I & Q mixing  
##set I1[i] [ expr [ expr $d1($i)] *cos($Wc*$i*1/$Sample_Rate_Hz) ] 
##set Q1($i) [ expr [ expr $d1($i)] *sin($Wc*$i*1/$Sample_Rate_Hz) ]
	
 
		I1 = []
		Q1 = []
		filtered_data_i = []
		d = (float)(d1[i])


		ival =(d * math.acos(Wc*i*1/Sample_Rate_Hz)) 
		qval = (d*math.asin(Wc*i*1/Sample_Rate_Hz))
		#puts "$ival $qval"
		I1.append( ival )
		Q1.append( qval )

		# Now filter the mixing products to remove the double frequency component

		filtered_data_i[i] =  A0 * I1[i] + A1*previous_input_i + B1*previous_output_i
		previous_input_i = I1[i]
		previous_output_i = filtered_data_i[i]

		# Now apply filter 2nd time on I channel

		filtered_data2_i[i] =  A0*filtered_data_i[i] + A1*previous_input2_i + B1*previous_output2_i
		previous_input2_i = filtered_data_i[i]
		previous_output2_i = filtered_data2_i[i]

		# Now apply filter 3rd time on I channel

		filtered_data3_i[i] = (A0*filtered_data2_i[i]) + (A1*previous_input3_i) + (B1*previous_output3_i)
		previous_input3_i = filtered_data2_i[i]
		previous_output3_i = filtered_data3_i[i]

		# Filter the Q channel

		filtered_data_q[i] = A0*Q1[i] + A1*previous_input_q + B1*previous_output_q
		previous_input_q = Q1[i]
		previous_output_q = filtered_data_q[i]

		# Now filter the Q channel a 2nd time

		filtered_data2_q[i] = A0*filtered_data_q[i] + A1*previous_input2_q + B1*previous_output2_q
		previous_input2_q = filtered_data_q[i]
		previous_output2_q = filtered_data2_q[i]

		# Now filter the Q channel a 3rd time

		filtered_data3_q[i] = A0*filtered_data2_q[i] + A1*previous_input3_q + B1*previous_output3_q
		previous_input3_q = filtered_data2_q[i]
		previous_output3_q = filtered_data3_q[i]

		# Now form the sum of I**2 and Q**2 and record the no of samples in the sum	
	
		# Performs the sum on the last 1/3 of the data
		#if {$i > [ expr 2.0 * [ array size d1 ] / 3 ] } 
		if i >  (2.0 * len(d1) / 3 ):
			I_SUM = I_SUM + filtered_data3_i[i]
			Q_SUM = Q_SUM + filtered_data3_q[i]
			SUM_COUNT = SUM_COUNT + 1
		
		info = "" 
		#append info $i "," $d1[i] "," $filtered_data_q[i] "," $filtered_data2_q[i] "," $filtered_data3_q[i] "," $filtered_data_i[i] "," $filtered_data2_i[i] "," $filtered_data3_i[i] "," 
		info = i + "," + d + "," + filtered_data_q[i] + "," +filtered_data2_q[i] + "," + filtered_data3_q[i] + "," + filtered_data_i[i] +"," + filtered_data2_i[i] + "," + filtered_data3_i[i] + "," 
		# puts $info
		fileId.write( info )
	

	# Amplitude (RMS) of sine wave (note multiply by 2 is to compensate for the initial mixing which halves the amplitude)

	AMPLITUDE = ( 2.0*math.sqrt( I_SUM* I_SUM + Q_SUM*Q_SUM / SUM_COUNT*SUM_COUNT ) )

	# return pk-pk value, therefore double the initial amplitude
	AMPLITUDE = 2* AMPLITUDE

	#puts "IIR: Input Amplitude: $inputAmplitude, Output Amplitude:  $AMPLITUDE "
	fileId.close()

	resultList.append(inputAmplitude)
	resultsList.append(AMPLITUDE)
	resultsList.append(inputFrequency)

	return (resultList)





