## Based on code from Gaby Zigelboim (RAD)
##
## Re-factored to separate input data from least squares algorithm and gain
## calculation
##
##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
##%% Assumptions:
##%% - Sample frequency is fixed at 16 samples/sec based on G.8275.1
##%%   Note that this means that this algorithm can ONLY be used under
##%%   script/instrument control
##%%   Questions: Can we get the CAT to check this? 
##%%              Does the capture file contain the packet rate so that the script
##%%              can check?
##%% - The algorithm will be called for each step in the noise transfer table.
##%%   The input frequency and amplitude for each step must be input to the algorithm
##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pi = 3.1415926535897931
import math
from Commonfunctions import *
from Globals import *

def filterLeastSquares ( stimulus, results, resultsFile, inputFreq, inputAmpl, inputSampleRate):
	global pi

	t = []

	y = []

	XXXDebugTrace ("Using Least Squares filter... %f %f %f" % (inputFreq, inputAmpl, inputSampleRate))

	# open file to write
	fileId = open (resultsFile, "a")

	intro = "t1" + "," + "d1" + "," 
	fileId.write( intro)


	inputAmplitude = inputAmpl
	f  = inputFreq
	Fs  = inputSampleRate 

	t = results [0]
	y = results [1]
	#array set y $results
	# y now contains the the measured amplitude in ns for each sampled point

	# Set up filter params
	#XXXDebugTrace "filterLeastSquares: Initialising..."
	sumCosCos = 0
	sumCosSin = 0
	sumSinSin = 0
	sumCosY	  = 0
	sumSinY	  = 0
	sumYY	  = 0

	N = len (y)
	#set N [array size y]


	XXXDebugTrace ("filterLeastSquares: Input data max: %f  min: %f  range: %f " % (max(y), min(y), (max(y) - min(y))))
	XXXDebugTrace ("filterLeastSquares: %d samples" % N )

	offset =  max(y) + min (y)
	offset = offset / 2
	#set y [expr $y1 - $o]
	#set y $y1

	XXXDebugTrace ("Offset: %f" %  offset)
	# (>3) minimal number of samples before starting the estimation
	#set nThresh	10

	#%% Calculate the sums for all points in the data
	#%% Note that the original alogorithm terminated early if the estimate at any
	#%% time was less than limit-3*std. We can't do this - we need to produce an actual
	#%% gain value not simply verify that it is less than the limit. 
	#%% Likewise, the original algorithm terminated if the amplitude and deviation
	#%% estimate were clearly above the limit. Again, we need an actual gain value.
	#%% Note:
	#%% - If ampEst-(sigmaEst*3)>Limit, this means that the algorithm is failing to converge
	#%%   However, we ignore this and finish since we need a result
	#%% - If sigmaEst*10 < ampEst then we have a valid result otherwise the SNR is not good
	#%%   In the latter case, it likely that the algortihm will not converge and so the result is questionable.
	#%%   The assumption is that, if this is the case, we will fail limits.
	#XXXDebugTrace "filterLeastSquares: Estimating..."

	for n in range (1,N):
		i  = n-1
		d = y[n]
		d = d - offset
		tn  = t[n]
		#XXXDebugTrace (d)


		value1 = (2*pi)
		value2 = (f*tn)
		value3 = (value1 * value2)
		cos_ = math.cos(value3)
		cos_ = 			math.cos((2*pi)*(f*tn))
		sin_ = 			math.sin((2*pi)*(f*tn))
		sumCosCos =		sumCosCos + (cos_ * cos_)
		sumCosSin =		sumCosSin + (cos_ * sin_)
		sumSinSin =		sumSinSin + (sin_ * sin_)	
		sumCosY	= 		sumCosY	 + (cos_ * d)
		sumSinY =		sumSinY	 + (sin_ * d)
		sumYY =			sumYY	 + (d * d)

		info = "" 
		info = str(i) + ","  + str(d) + ", \n" 
		fileId.write ( info)
	

	detMat =		(sumCosCos*sumSinSin) - (sumCosSin*sumCosSin)
	invMat11 =		sumSinSin / detMat
	invMat12 =		-sumCosSin / detMat 
	invMat21 =		-sumCosSin / detMat
	invMat22 =		sumCosCos / detMat
	aCos	=		(invMat11 * sumCosY)	+ (invMat12 * sumSinY)
	aSin	=		(invMat21 * sumCosY)	+ (invMat22 * sumSinY)
	sigmaSquare	= 	(sumYY - ((aCos * sumCosY) + aSin * sumSinY))/N

	ampEst  =	math.sqrt((aCos*aCos) + (aSin*aSin))
	sigmaEst =	math.sqrt(sigmaSquare/N)

	amplitude = ampEst

	if sigmaEst*10 > ampEst:
		# SNR is low. Flag this only when debugging
		XXXDebugTrace ("!!!!!!!!!!!!!!  Questionable result  !!!!!!!!!!!!!!!!!!!!!!!!!")

	amplitude = 	(2*amplitude)

	XXXDebugTrace ("LSQ: Input Amplitude: %f , Output Amplitude: %f " % (inputAmplitude, amplitude))
	revisit = 2

	fileId.close()
	
	resultList = []
	resultList.append(inputAmplitude)
	resultList.append(amplitude)
	resultList.append(inputFreq)

	return (resultList)



