#---------------TODO-------------------
#always plot full mask
#Make into a script launcher:
#  - read in defaults - allow to be editable, and sent to the script
#  - plot each individual measurements, plot each of the raw data that does into the gain,
#    different plot or clock button to open?
#--------------------------------------

import tkinter as tk
import tkinter.ttk as ttk
import textwrap

from matplotlib.figure import Figure 
from matplotlib.backends.backend_tkagg import (FigureCanvasTkAgg, NavigationToolbar2Tk)
from matplotlib.ticker import ScalarFormatter
from matplotlib.lines import Line2D
from matplotlib import pyplot as plt

from PIL import ImageTk, Image
from math import log10


class EntMeasSetup:
    # calculates and holds the measurement parameters that affect aspects of the GUI,
    def __init__(self, stim, meas, limitsOn, yAxis):               
        if meas == "BOTH" or meas == "PTP": 
            self.measPtp  = True
        else:
            self.measPtp = False
       
        if meas == "BOTH" or meas == "1PPS":  
            self.meas1pps = True 
        else:
            self.meas1pps = False

        if limitsOn == "TRUE":
            self.limitsOn = True
        else:
            self.limitsOn = False

        if yAxis == "GAIN":
            self.displayType = "GAIN"
            self.ll = "LL (gain dB)"
            self.ul = "UL (gain dB)"
            self.plotYlabel = "Gain (dB)"
        
        if yAxis == "AMPLITUDE":
            self.displayType = "AMP"
            self.ll = "LL (pk-pk ns)"
            self.ul = "UL (pk-pk ns)"
            self.plotYlabel = "Amplitude (pk-pkns)"
        
        m = meas
        if meas == "BOTH":
            m = 'PTP and 1PPS'
        self.bodePlotTitle = stim + " to " + m + " Noise Transfer"


class EntParamsTable:
    def __init__(self, params, sn, ver, d, t):
        self.paramsDict = {}
        #makes the current value of the passed variables into our table format to pass to label-table draw
        self.paramsDict = {
                "Serial No.":       {"Parameter": "Serial No.", "Current Value": sn },
                "Firmware":         {"Parameter": "Firmware",   "Current Value": ver},
                "Date":             {"Parameter": "Date",       "Current Value": d  },
                "Time":             {"Parameter": "Time",       "Current Value": t  },
        }

        #removes a few settings that we aren't so interested in - gives a bit more screen space back
        hide_params = ("Debug", "RemoteProfileFolder", "RemoteCaptureFolder", "GuiPort",
                        "Graphing", "GraphingYAxis", "Port1Config", "Port2Config",)

        for key in hide_params:
            if key in params:
                del params[key]

        #makes the parameters dictionary into our table format
        for key, value in params.items():
            self.paramsDict[key] = ({"Parameter": key,
                                     "Current Value":  value,})


class EntBodePlotTable:
    def __init__(self, stimlist):
        #uses stimlist and measurement modes create dictionary that with required columns for this measurement and 
        #populates with all currently known information about the upcoming measurements
        self.measDict = {}

        for dic in stimlist:        
                self.measDict[dic["Frequency"]] = {
                                            "Frequency (Hz)"        : dic["Frequency"],
                                            "Ampl (pk-pk ns)"       : dic["amplitude"],
                } 
                                
                if currMeas.measPtp == True:
                    self.measDict[dic["Frequency"]].update({                
                                            "PTP Meas (pk-pk ns)"    : None,
                                            "PTP Gain (dB)"          : None,
                    })

                if currMeas.meas1pps == True: 
                    self.measDict[dic["Frequency"]].update({ 
                                            "1pps Meas (pk-pk ns)"    : None,
                                            "1pps Gain (dB)"          : None,
                    })

                if currMeas.displayType == "GAIN" and currMeas.limitsOn == True:
                    self.measDict[dic["Frequency"]].update({ 
                                            currMeas.ll               : -100 if (dic["LL"]) == "NA" else round(20 * log10(float (dic["LL"] / dic["amplitude"]) ), 2),         # limit value may be a number or "NA" - if NA, val is -100, if a number, DO GAIN CALC
                                            currMeas.ul               : round(20 * log10(float (dic["UL"] / dic["amplitude"]) ), 2),                                            # limit value will always be a number, DO GAIN CALC
                    })

                if currMeas.displayType == "AMP" and currMeas.limitsOn == True:
                    self.measDict[dic["Frequency"]].update({ 
                                            currMeas.ll               : 0 if (dic["LL"]) == "NA" else (dic["LL"]),                                                              # limit value may be a number or "NA", if NA val is 0, if limit value is a number then use that number
                                            currMeas.ul               : (dic["UL"]),                                                                                              # will be the same value, which is a number
                    })

                if currMeas.limitsOn == True and currMeas.measPtp == True: 
                    self.measDict[dic["Frequency"]].update({ 
                                            "PTP Pass/Fail"           : None,
                    })

                if currMeas.limitsOn == True and currMeas.meas1pps == True:   
                    self.measDict[dic["Frequency"]].update({                   
                                            "1pps Pass/Fail"          : None,
                    })


    def update1pps(self, freq, amp_out, gain, oneppsPass):
        self.measDict[freq].update({"1pps Meas (pk-pk ns)"        : amp_out})
        self.measDict[freq].update({"1pps Gain (dB)"              : gain})
        
        if currMeas.limitsOn == True:
            if "FAIL" in oneppsPass:
                oneppsPass = "FAIL"  
            self.measDict[freq].update({"1pps Pass/Fail"          : oneppsPass})


    def updatePtp(self, freq, amp_out, gain, ptpPass):
        self.measDict[freq].update({"PTP Meas (pk-pk ns)"          : round(amp_out, 5)})
        self.measDict[freq].update({"PTP Gain (dB)"                : round(gain,5)})
        
        if currMeas.limitsOn == True:
            if "FAIL" in ptpPass:
                ptpPass = "FAIL"         
            self.measDict[freq].update({"PTP Pass/Fail"            : ptpPass})
        

class EntGUI(tk.Tk):
    #class that manages the main application window
    def __init__(self): 
        tk.Tk.__init__(self)
        self.geometry("1200x780+400+100")
        self.protocol("WM_DELETE_WINDOW", self.onClosing)
        self.title("G.8273.2 Noise Transfer")
        self.iconbitmap("./Calnex_Icon.ico")

        #main content frame with a bit of padding around the edges
        self.main_frame = tk.Frame(self, borderwidth = 0, relief = "solid")
        self.main_frame.grid_rowconfigure(1, weight=1)
        self.main_frame.grid_columnconfigure(1, weight=1)
        self.main_frame.pack(fill = "both", expand = "true", padx = 20, pady = 20)

        #lays out logo/parameters table/plot/measurement
        self.logo_frame = tk.Frame(self.main_frame, borderwidth = 0, relief = "solid")
        self.logo_frame.grid(row = 0, column = 0, sticky = "nw")

        self.params_table_frame = tk.Frame(self.main_frame, borderwidth = 0, relief = "solid")
        self.params_table_frame.grid(row=1, rowspan = 2, column=0, pady = (20,0), sticky = "n")

        self.matplotlib_frame = tk.Frame(self.main_frame,  borderwidth = 0, relief = "solid")
        self.matplotlib_frame.grid(row=0, column=1, rowspan = 2, padx = (20,0))

        self.meas_table_frame = tk.Frame(self.main_frame, borderwidth = 0, relief = "solid")
        self.meas_table_frame.grid (row=2, column=1, pady = (20,0))

        self.createCalnexLogo(self.logo_frame)
        self.drawTable(self.params_table_frame, paramsTable.paramsDict, 18, False)    
        self.drawTable(self.meas_table_frame, measTable.measDict, 10, True)
        self.createBodePlot(self.matplotlib_frame, measTable.measDict)

        self.lift()

        #self.mainloop

    def onClosing(self):
        if tk.messagebox.askokcancel("Quit", "Do you want to quit?"):
            self.destroy()
            quit()


    def createCalnexLogo(self,frame):
        self.logofile = Image.open("./Calnex.png")
        self.logo = ImageTk.PhotoImage(self.logofile)
        self.logolabel = tk.Label(frame,image=self.logo)
        self.logolabel.image = self.logo
        self.logolabel.pack()


    def drawTable(self, frame, tableDict, cell_width, header_wrap):
        #clears out all labels first, in case this is a redraw (update)
        for widget in frame.winfo_children():
            widget.destroy()

        #gets keys for measDict first item, turn them into a list - because dict_keys is a type that cannot be sliced
        #slice the 1st element, use the result as the key to the measDict then make a list of the returned keys, then enumerate the list
        for column, header in enumerate(list(tableDict[list(tableDict.keys())[0]].keys())):
            if header_wrap == True:
                header = textwrap.fill(str(header), width = cell_width, break_long_words=False, break_on_hyphens=False)
            tk.Label(frame, text=header, font = "Arial 9 bold", relief = "groove", borderwidth = 1, fg = "white", bg = "gray39", width = cell_width, padx=6, pady=6).grid(row=0, column=0+column)

        # Fill in the row values - if value is PASS or FAIL, colours the label accordingly, first column, and  is bold
        for row, element in enumerate(tableDict.values()):
            for column, (header, value) in enumerate(element.items()):
                
                row_font = "Arial 9"
                cell_colour = "white"
                if value == "PASS": cell_colour = "green3"
                if value == "FAIL": cell_colour = "tomato2"
                if column == 0: row_font = "Arial 9 bold"
                
                tk.Label(frame, text=value, font = row_font, relief = "groove", width = cell_width, bg = cell_colour, borderwidth = 1, padx=6, pady=1).grid(row=1+row, column=0+column)
        
        frame.update()


    def createBodePlot(self, frame, plotTable):   
        global entPlot

        fig = Figure(figsize = (10, 10), dpi = 100) 
        fig.suptitle(currMeas.bodePlotTitle, fontsize=16)
    
        entPlot = fig.add_subplot(111) 

        freq_x = []
        for key, value in plotTable.items():  
            freq_x.append(float(value["Frequency (Hz)"]))

        #places the limits on the graph if selected
        if currMeas.limitsOn == True:
            ll_gain_y = []
            for key, value in plotTable.items():  
                ll_gain_y.append(float(value[currMeas.ll]))
            
            ul_gain_y = []
            for key, value in plotTable.items():  
                ul_gain_y.append(float(value[currMeas.ul]))
                
            marg_diff = (abs(min(ll_gain_y) - max(ul_gain_y)) * 0.05)
            margin_lower = min(ll_gain_y) - marg_diff
            margin_upper = max(ul_gain_y) + marg_diff

            entPlot.semilogx(freq_x, ll_gain_y, label = "Upper Limit", linestyle = "solid", color = "red", linewidth = 1)
            entPlot.fill_between(freq_x, ll_gain_y, margin_lower, facecolor="red", edgecolor="brown", hatch = "xxx", alpha = 0.2)
            entPlot.semilogx(freq_x, ul_gain_y, label = "Lower Limit", linestyle = "solid", color = "red", linewidth=1)
            entPlot.fill_between(freq_x, ul_gain_y, margin_upper, facecolor="red", edgecolor="brown", hatch = "xxx", alpha = 0.2)

        entPlot.set_xscale('log')
        entPlot.set_xlabel("Frequency (Hz)")
        entPlot.set_ylabel(currMeas.plotYlabel)
        entPlot.grid(which="major", color ='lightgrey', linestyle = '-', linewidth =1)
        entPlot.grid(which="minor", color ='gainsboro', linestyle = ':', linewidth =1)
        entPlot.set_axisbelow(True)
        entPlot.xaxis.set_major_formatter(ScalarFormatter())
        entPlot.ticklabel_format(axis="x", style="plain")

        legend_elements = [ Line2D([0], [0], lw = 3, color ='red', label='ENT Thresholds'),
                            Line2D([0], [0], lw = 0, marker='P', label='PTP Noise Tfr', markerfacecolor='navy', markeredgecolor = "navy", markersize=5),
                            Line2D([0], [0], lw = 0, marker='o', label='1PPS Noise Tfr', markerfacecolor='none', markeredgecolor = "darkgreen", markersize=5),    
        ]
        
        entPlot.legend(handles=legend_elements, fontsize = "small")

        # creating the Tkinter canvas containing the Matplotlib figure
        figure_frame = tk.Frame(frame, background = "white", borderwidth = 3, relief = "groove")
        canvas = FigureCanvasTkAgg(fig, master = figure_frame)
        figure_frame.pack(fill = "both", expand = 1)

        fig.tight_layout(pad = 6)

        NavigationToolbar2Tk(canvas, figure_frame)
        canvas.get_tk_widget().pack()
        frame.update()


    def updateBodePlot(self, freq, signal_type, amp_out, gain):
        if currMeas.displayType == "AMP":
            y_val = amp_out
        elif currMeas.displayType == "GAIN":
            y_val = gain
        
        if signal_type == "PTP":
             entPlot.plot(freq, y_val, label = "PTP Noise Tfr", linestyle = 'None', markerfacecolor='navy', markeredgecolor = "navy",  marker = 'P')
        elif signal_type == "1PPS":
            entPlot.plot(freq, y_val, label = "1PPS Noise Tfr", linestyle = 'None', markerfacecolor='none', markeredgecolor = "darkgreen", marker = 'o')
        
        entPlot.figure.canvas.draw()


def configureGraphing(stim, stimlist, meas, limitsOn, yAxis, params, ser_num, fw, date, time):
    #creates instance of the class to process the data, makes the items global so updates and data accessed
    global currMeas, paramsTable, measTable, mainGui
    
    currMeas =      EntMeasSetup(stim, meas, limitsOn, yAxis)           #current measurement parameters
    paramsTable =   EntParamsTable(params, ser_num, fw, date, time)     #the params table, P-X settings & serial, etc.
    measTable =     EntBodePlotTable(stimlist)                          #the measurement frequencies, limits, results
    
    mainGui = EntGUI()


def updatePtpMeasurement(freq, amp_out, gain, ptpPass):
    measTable.updatePtp(freq, amp_out, gain, ptpPass)
    mainGui.drawTable(mainGui.meas_table_frame, measTable.measDict, 10, True)
    mainGui.updateBodePlot(freq, "PTP", amp_out, gain)


def update1ppsMeasurement(freq, amp_out, gain, oneppsPass):
    measTable.update1pps(freq, amp_out, gain, oneppsPass)
    mainGui.drawTable(mainGui.meas_table_frame, measTable.measDict, 10, True)
    mainGui.updateBodePlot(freq, "1PPS", amp_out, gain)
    

def savePlotImage(overall_results_csv):
    #gets the overall .csv filename and replaces it with.png
    fullpath_file = overall_results_csv.replace(".csv",".png")
    fig = entPlot.figure.savefig(fullpath_file)