package require Tk
package require Img
package require rbc
namespace import rbc::*

# Don't like so many globals...
vector create ptpAmplBodePlotLine
vector create 1ppsAmplBodePlotLine
global tableData
global infoTableData
global sList
global topFrame
global ptpPlotsFrame
global 1ppsPlotsFrame
global limitsOn
global yAxis

# The starting column in the bode plot table for 1pps and ptp results
# These are dependent on whether the measurement is PTP, 1pps or both
global 1ppsTableCol
global ptpTableCol
global 1ppsTablePFCol
global ptpTablePFCol

global mainWindowWidth
global mainWindowHeight

 #############################################################################
 # Configure Graphing
 # Set up the required plots and tables
proc configureGraphing {stim meas stimList params sn version date time} { 
	variable limitsOn
	variable yAxis
 variable mainWindowWidth
 variable mainWindowHeight
	
 set mainWindowHeight 700
 set mainWindowWidth 830
 
	set measPTP false
	set meas1pps false
	
	if {$meas == "PTP" || $meas == "BOTH"} {set measPTP true}
	if {$meas == "1PPS" || $meas == "BOTH"} {set meas1pps true}
	
	set m $meas
	if {$meas == "BOTH"} {set m "PTP and 1pps"}
	
	set limitsOn [dict get $params "LimitsEnabled"]
	#set yAxis GAIN
	set yAxis [dict get $params "GraphingYAxis"]
	
	createPlotWindows $measPTP $meas1pps
	createPlots "$stim to $m Noise Transfer" $stimList $measPTP $meas1pps $limitsOn
	createBodePlotTable $stimList $meas $limitsOn
	createInfoTable $params $sn $version $date $time
}

 ############################################################################
 #
 # simpleTable.tcl -- replacement of tktable for simple tables
 # by Keith Vetter, October 2005
 #
 proc simpleTable {pathName args} {    
    set legal {-rows -cols -titlerows -titlecols -vcmd
        -readonlyrows -readonlycols -variable}
    array set A {-rows 5 -cols 5 -titlerows 0 -titlecols 0
                     -readonlyrows {} -readonlycols {}}
 
    if {[llength $args] == 0} {
        error "usage: simpleTable pathName -variable varName ?options...?"
    }
 
    set idx -1
    foreach {opt value} $args {
        if {[lsearch $legal $opt] == -1} {
            error "unknown option \042$opt\042"
        }
        if {[incr idx 2] >= [llength $args]} {
            error "value for \042$opt\042 missing"
        }
        set A($opt) $value
    }
    if {! [info exists A(-variable)]} {
        error "missing the \042-variable\042 option"
    }
 
    # Now build our table
    destroy $pathName
    frame $pathName
    for {set r 0} {$r < $A(-rows)} {incr r} {
        for {set c 0} {$c < $A(-cols)} {incr c} {
            set w "$pathName.r${r},$c"
            if {$r < $A(-titlerows) || $c < $A(-titlecols)} {
                if { $::tcl_platform(platform) eq "windows" } {
                    set bg SystemDisabledText 
                } else {
                    set bg darkgrey
                }
                label $w  -textvariable $A(-variable)\($r,$c\) \
                    -relief sunken -bg $bg -fg white
            } elseif {[lsearch $A(-readonlyrows) $r] != -1 || \
                      [lsearch $A(-readonlycols) $c] != -1} {
                label $w -textvariable $A(-variable)\($r,$c\) \
                    -relief groove -bd 1
                $w config -font [lindex [$w config -font] 3]
            } else {
                entry $w -textvariable $A(-variable)\($r,$c\) \
                    -relief sunken -bd 1 -justify center -width 9
                if {[info exists A(-vcmd)]} {
                    $w config -validate key \
                        -vcmd [list $A(-vcmd) $r $c %P]
                }
            }
            grid $pathName.r${r},$c -row $r -column $c -sticky ew
        }
    }
 }

#----------------------------------------------------
# Functions to extract fields from the stimulusList
proc getFreqs {stimList} {
	set l {}
	foreach i $stimList {
		lappend l [lindex $i 0]
		#puts $l
	}
	return $l
}
proc getAmpls {stimList} {
	set l {}
	foreach i $stimList {
		lappend l [lindex $i 1]
		#puts $l
	}
	return $l
}

proc getUpperLimits {stimList} {
	variable yAxis
	set l {}
	foreach i $stimList {
		set lim [lindex $i 5]
		if {$yAxis == "GAIN"} {
			set amplIn [lindex $i 1]
			set db [format %.2f [expr {20 * log10($lim/$amplIn)}]]
			lappend l $db
			#puts $l
		} else {		
			lappend l $lim
		}
	}
	return $l
}
proc getLowerLimits {stimList} {
	variable yAxis
	set l {}
	foreach i $stimList {
		set lim [lindex $i 4]
		if {$lim == "NA"} {
			if {$yAxis == "GAIN"} {set lim -100} else {set lim 0}
			lappend l $lim
		} elseif {$yAxis == "GAIN"} {
			set amplIn [lindex $i 1]
			set db [format %.2f [expr {20 * log10($lim/$amplIn)}]]
			lappend l $db
			#puts $l
		} else {		
			lappend l $lim
		}
		#set v [lindex $i 4]
		#if {$v == "NA"} {set v 0}
		#lappend l $v
		#puts $l
	}
	return $l
}
#-----------------------------------------------------
proc createInfoTable {params sn v d t} {
	variable infoTableData
	variable topFrame
	variable reprocessIgnore
	variable reprocessOmit
 variable mainWindowWidth
 variable mainWindowHeight
	
 set reProcess [dict get $params "ReProcess"]
 #set omitted 0
 #if {$reProcess == TRUE} {
  #set omitted [llength $reprocessOmit]
 #}
 
	if {$params == ""} {
		set rows 3
	} else {
		set rows [expr {[dict size $params]+3-[llength $reprocessOmit]}]
		#set rows [expr {[dict size $params]+3-$omitted}]
	}
	unset -nocomplain table
	array set infoTableData {
		0,0 "Serial Number" 1,0 "Version" 2,0 "Date" 3,0 "Time"
	}

	set infoTableData(0,1) $sn
	set infoTableData(1,1) $v
	set infoTableData(2,1) $d
	set infoTableData(3,1) $t
	set row 4

	if {$params != ""} {
		dict for {key value} $params {
   if {$reProcess == TRUE} {
 			if {$key in $reprocessIgnore} {
 				set value "NA"
 			}
   }
			if {$key ni $reprocessOmit} {
				set infoTableData($row,0) $key
				set infoTableData($row,1) $value
				set row [incr row]
			}
		}
	}

		
	simpleTable $topFrame.infoTable -variable infoTableData -rows [expr {$rows+1}] -cols 2 -titlecols 1 -readonlycols {0 1}
	#grid $topFrame.infoTable -column 0 -row 1 -padx 10 -pady 0 -columnspan 0 -rowspan 2 -sticky n
	grid $topFrame.infoTable -column 0 -row 1 -padx 10 -pady 0 -rowspan 4 -sticky n
	update
	#positionWindow .top 50 50
 wm geometry .top ${mainWindowWidth}x${mainWindowHeight}+50+50

}

proc createBodePlotTable {stimulusList meas limitsEnabled} {
	variable tableData
	variable topFrame
	variable ptpTableCol
	variable 1ppsTableCol
	variable ptpTablePFCol
	variable 1ppsTablePFCol
	variable yAxis
 variable mainWindowWidth
 variable mainWindowHeight

	set freqs [getFreqs $stimulusList]
	set ampls [getAmpls $stimulusList]
	set uls [getUpperLimits $stimulusList]
	set lls [getLowerLimits $stimulusList]
	
	# Set the indices assuming not BOTH
	set indexLL 4
	
	set indexUL 5
	set numCols 7

	set rows [llength $freqs]
	unset -nocomplain table
	if {$meas == "BOTH"} {
		if {$limitsEnabled} {
			if {$yAxis == "GAIN"} {
				array set tableData {
					0,0 "Freq\n(Hz)" 0,1 "Ampl\n(pk-pk ns)" 0,2 "PTP Meas\n(pk-pk ns)" 0,3 "PTP Gain\n(dB)" 0,4 "1pps Meas\n(pk-pk ns)" 0,5 "1pps Gain\n(dB)" 0,6 "LL\n(gain dB)" 0,7 "UL\n(gain dB)" 0,8 " PTP\nPass/Fail" 0,9 " 1pps\nPass/Fail"				
				}
			} else {
				array set tableData {
					0,0 "Freq\n(Hz)" 0,1 "Ampl\n(pk-pk ns)" 0,2 "PTP Meas\n(pk-pk ns)" 0,3 "PTP Gain\n(dB)" 0,4 "1pps Meas\n(pk-pk ns)" 0,5 "1pps Gain\n(dB)" 0,6 "LL\n(pk-pk ns)" 0,7 "UL\n(pk-pk ns)" 0,8 " PTP\nPass/Fail" 0,9 " 1pps\nPass/Fail"				
				}
			}
			set numCols 10
		} else {
			array set tableData {
				0,0 "Freq\n(Hz)" 0,1 "Ampl\n(pk-pk ns)" 0,2 "PTP Meas\n(pk-pk ns)" 0,3 "PTP Gain\n(dB)" 0,4 "1pps Meas\n(pk-pk ns)" 0,5 "1pps Gain\n(dB)"
			}
			set numCols 6
		}
		set indexLL 6
		set indexUL 7
		set ptpTableCol 2
		set 1ppsTableCol 4
		set ptpTablePFCol 8
		set 1ppsTablePFCol 9
	}
	if {$meas == "PTP"} {
		if {$limitsEnabled} {
			if {$yAxis == "GAIN"} {
				array set tableData {
					0,0 "Freq\n(Hz)" 0,1 "Ampl\n(pk-pk ns)" 0,2 "PTP Meas\n(pk-pk ns)" 0,3 "PTP Gain\n(dB)" 0,4 "LL\n(gain dB)" 0,5 "UL\n(gain dB)" 0,6 " PTP\nPass/Fail"				
				}
			} else {
				array set tableData {
					0,0 "Freq\n(Hz)" 0,1 "Ampl\n(pk-pk ns)" 0,2 "PTP Meas\n(pk-pk ns)" 0,3 "PTP Gain\n(dB)" 0,4 "LL\n(pk-pk ns)" 0,5 "UL\n(pk-pk ns)" 0,6 " PTP\nPass/Fail"				
				}
			}
		} else {
			array set tableData {
				0,0 "Freq\n(Hz)" 0,1 "Ampl\n(pk-pk ns)" 0,2 "PTP Meas\n(pk-pk ns)" 0,3 "PTP Gain\n(dB)"
			}			
		}
		set ptpTableCol 2
		set ptpTablePFCol 6
	}
	if {$meas == "1PPS"} {
		if {$limitsEnabled} {
			if {$yAxis == "GAIN"} {
				array set tableData {
					0,0 "Freq\n(Hz)" 0,1 "Ampl\n(pk-pk ns)" 0,2 "1pps Meas\n(pk-pk ns)" 0,3 "1pps Gain\n(dB)" 0,4 "LL\n(gain dB)" 0,5 "UL\n(gain dB)" 0,6 " 1pps\nPass/Fail"				
				}
			} else {
				array set tableData {
					0,0 "Freq\n(Hz)" 0,1 "Ampl\n(pk-pk ns)" 0,2 "1pps Meas\n(pk-pk ns)" 0,3 "1pps Gain\n(dB)" 0,4 "LL\n(pk-pk ns)" 0,5 "UL\n(pk-pk ns)" 0,6 " 1pps\nPass/Fail"				
				}
			}
		} else {
			array set tableData {
				0,0 "Freq\n(Hz)" 0,1 "Ampl\n(pk-pk ns)" 0,2 "1pps Meas\n(pk-pk ns)" 0,3 "1pps Gain\n(dB)"
			}			
		}
		set 1ppsTableCol 2
		set 1ppsTablePFCol 6
	}
	
#	if {$noiseTransferType == "PTP-PTP" || $noiseTransferType == "SyncE-PTP"} {
#		set tableData(0,2) "PTP Meas\n(pk-pk ns)"
#	} else {
#		set tableData(0,2) "1PPS Meas\n(pk-pk ns)"
#	}

	for {set index 0} {$index < $rows} {incr index} {
		set row [expr {$index+1}]
		set tableData($row,0) [lindex $freqs $index]
		set tableData($row,1) [lindex $ampls $index]
		if {$limitsEnabled} {
			set tableData($row,$indexLL) [lindex $lls $index]
			set tableData($row,$indexUL) [lindex $uls $index]
		}
	}
	
	simpleTable $topFrame.t -variable tableData -rows [expr {$rows+1}] -cols $numCols -titlerows 1 -readonlycols {0 1 2 3 4 5 6 7 8 9}
	#grid $topFrame.t -column 3 -row 0 -padx 10 -columnspan 2 -sticky n -pady 40
	grid $topFrame.t -column 1 -row 4 -padx 10 -sticky n -pady 20
	update
	#positionWindow .top 50 50
 wm geometry .top ${mainWindowWidth}x${mainWindowHeight}+50+50

}

proc positionWindow {w x y} {
	set width [winfo reqwidth $w]
    set height [winfo reqheight $w]
    wm geometry $w ${width}x${height}+${x}+${y}
}

proc createPlotWindows {ptp 1pps} {
	variable topFrame
	variable ptpPlotsFrame
	variable 1ppsPlotsFrame
	variable debugTrace
	
	# Create the main window	
	wm withdraw .
	set top .top
	toplevel $top
	wm title $top "G.8273.2 Noise Transfer"
	wm resizable $top 0 0
	set topFrame $top.f
	frame $topFrame
	grid $topFrame - - - - -row 0 -column 0 -sticky ew
	
	wm iconbitmap .top -default "./Calnex_Icon.ico"
	
	# Add the logo
	set logo [image create photo -file Calnex.png]
	label $topFrame.labelLogo -image $logo -width 169 -height 94
	grid $topFrame.labelLogo -column 0 -row 0 -sticky n -pady 10
	
	# When the window is closed, kill the parent window too - closes the application
	wm protocol $top WM_DELETE_WINDOW [list destroy .]
	
	# Create the window for the detailed PTP plots if required
	if {$ptp} {
		set ptpPlots .ptpPlots
		toplevel $ptpPlots
		wm title $ptpPlots "G.8273.2 Noise Transfer - PTP Plots"
		wm resizable $ptpPlots 0 0
		set ptpPlotsFrame $ptpPlots.f
		frame $ptpPlotsFrame
		grid $ptpPlotsFrame - - - - -row 0 -column 0 -sticky ew
		if {!$debugTrace} {wm withdraw $ptpPlots}
	}
	
	if {$1pps} {
		set 1ppsPlots .1ppsPlots
		toplevel $1ppsPlots
		wm title $1ppsPlots "G.8273.2 Noise Transfer - 1pps Plots"
		wm resizable $1ppsPlots 0 0
		set 1ppsPlotsFrame $1ppsPlots.f
		frame $1ppsPlotsFrame
		grid $1ppsPlotsFrame - - - - -row 0 -column 0 -sticky ew
		if {!$debugTrace} {wm withdraw $1ppsPlots}
	}

}


# Formats the x-axis frequency labels
proc formatTickLabel {w v} {
	return [format "%f" $v]
}

proc createPlots {bodePlotTitle stimulusList ptp 1pps limitsEnabled} {
	variable sList
	variable topFrame
	variable ptpPlotsFrame
	variable 1ppsPlotsFrame
	variable debugTrace
	variable yAxis
 variable mainWindowWidth
 variable mainWindowHeight
			
	set sList $stimulusList

	# Create the graphs
	# One for the final results - the bode plot
	graph $topFrame.bodePlot -plotbackground "white"
	
	# One plot per frequency point for PTP and 1pps
	set frameNames {}
	if {$ptp} {lappend frameNames "ptp"}
	if {$1pps} {lappend frameNames "1pps"}
	#set frameNames {"ptp" "1pps"}
	foreach frameName $frameNames {
		# One each for the time error from each point
		set c 1
		set r 1
		foreach s $stimulusList {
			set freq [lindex $s 0]
			# Replace "." in frequency with underbar
			set f [ string map {"." "_"} $freq ]
			#set plotId $topFrame.dataPlot_
			if {$frameName == "ptp"} {
				set plotId $ptpPlotsFrame.dataPlot_
			} else {
				set plotId $1ppsPlotsFrame.dataPlot_
			}			
			set line "dataLine_"
			append plotId $f
			append line $f
			graph $plotId -plotbackground "white" -height 1.5i -width 2i
			$plotId configure -title "$freq Hz" -font "*-Helvetica-R-Normal-*-10-180-*"
			#$plotId axis configure x -title "Time (s)"
			#$plotId axis configure y -title "Time Error (ns)"
			$plotId grid configure -hide no -dashes { 1 1 } -minor no
			$plotId element create $line -symbol "" -color blue -label ""
			$plotId configure -cursor left_ptr

			# Only display the individual data charts in debug mode
#			if {$debugTrace} {
				grid $plotId -column $c -row $r
#			}
			
			set c [incr c]
			if {$c == 5} {
				set c 1
				set r [incr r]
			}
		}
	}
	
	#grid $topFrame.bodePlot -column 1 -row 0 -columnspan 0 -rowspan 3 -sticky ewn
	grid $topFrame.bodePlot -column 1 -row 0 -rowspan 3 -sticky ewn
	#grid .dataPlot -column 0 -row 1

	# Change the title. 
	$topFrame.bodePlot configure -title $bodePlotTitle -cursor left_ptr
	
	# Configure the axes. Bode plot X-axis (freq) is log scale. 
	$topFrame.bodePlot axis configure x -logscale yes -title "Frequency (Hz)" -command "formatTickLabel" -max 10
	if {$yAxis == "GAIN"} {
		$topFrame.bodePlot axis configure y -title "Gain (dB)"
	} else {
		$topFrame.bodePlot axis configure y -title "Amplitude (ns)"		
	}

	# Display grid lines. 
	$topFrame.bodePlot grid configure -hide no -dashes { 1 1 } -minor no
	
	# Create vectors for freq and limits	
	vector create freqVec ulVec llVec
	freqVec set [getFreqs $stimulusList]
	
	if {$limitsEnabled} {
		ulVec set [getUpperLimits $stimulusList]
		llVec set [getLowerLimits $stimulusList]
		# Draw the limit lines
		$topFrame.bodePlot element create ulLine -xdata freqVec -ydata ulVec -symbol "" -color red -label "Upper Limit" -dashes {1 1 1}
		$topFrame.bodePlot element create llLine -xdata freqVec -ydata llVec -symbol "" -color red -label "Lower Limit" -dashes {1 1 1}
	}
	
	if {$ptp} {
		$topFrame.bodePlot element create ptpAmplLine -xdata freqVec -ydata ptpAmplBodePlotLine -symbol "circle" -pixels 2 -color blue -label "Amplitude (PTP)" -weights {5.0 5.0 5.0 5.0 5.0 5.0 5.0 5.0}
		$topFrame.bodePlot element configure ptpAmplLine -weights {5.0 5.0 5.0 5.0 5.0 5.0 5.0 5.0} -linewidth 0
	}
	if {$1pps} {
		$topFrame.bodePlot element create 1ppsAmplLine -xdata freqVec -ydata 1ppsAmplBodePlotLine -symbol "plus" -pixels 5 -color green -label "Amplitude (1pps)" -weights {5.0 5.0 5.0 5.0 5.0 5.0 5.0 5.0}
		$topFrame.bodePlot element configure 1ppsAmplLine -weights {5.0 5.0 5.0 5.0 5.0 5.0 5.0 5.0} -linewidth 0
	}
	
	#createBodePlotTable $stimulusList
	# Forces an update of the chart. This is equired to make sure the UI gets a look in.
		
#	$topFrame.bodePlot element bind all <ButtonPress-1> {
#		puts "Click in bode plot"
#		wm geometry  .ptpPlots +900+50
#		wm transient .ptpPlots .
#		wm deiconify .ptpPlots
#	}
	if {$ptp && $1pps} {
		$topFrame.bodePlot legend bind all <ButtonPress-1> {
			if {[wm state .ptpPlots] != "normal"} {
				wm deiconify .ptpPlots
			} else {
				wm withdraw .ptpPlots
			}
			if {[wm state .1ppsPlots] != "normal"} {
				wm deiconify .1ppsPlots
			} else {
				wm withdraw .1ppsPlots
			}
		}
	} elseif {$ptp} {
		$topFrame.bodePlot legend bind all <ButtonPress-1> {
			if {[wm state .ptpPlots] != "normal"} {
				wm deiconify .ptpPlots
			} else {
				wm withdraw .ptpPlots
			}
		}
	} elseif {$1pps} {
		$topFrame.bodePlot legend bind all <ButtonPress-1> {
			if {[wm state .1ppsPlots] != "normal"} {
				wm deiconify .1ppsPlots
			} else {
				wm withdraw .1ppsPlots
			}
		}
	}
	
	
	update
	#positionWindow .top 50 50
 wm geometry .top ${mainWindowWidth}x${mainWindowHeight}+50+50
	if {$1pps && $ptp} {
		positionWindow .ptpPlots 900 50
		positionWindow .1ppsPlots 900 [expr {[winfo reqheight .ptpPlots] + 50 + 37}]
	} elseif {$ptp} {
		positionWindow .ptpPlots 900 50	
	} else {
		positionWindow .1ppsPlots 900 50
	}
	
}

proc update1ppsBodePlot {freq 1ppsAmpl 1ppsGain 1ppsPass} {
	variable sList
	variable tableData
	variable topFrame
	variable 1ppsTableCol
	variable 1ppsTablePFCol
	variable limitsOn
	variable yAxis
	
	if {$yAxis == "GAIN"} {
		1ppsAmplBodePlotLine append $1ppsGain		
	} else {
		1ppsAmplBodePlotLine append $1ppsAmpl		
	}
	

	set freqs [getFreqs $sList]
	set row [lsearch $freqs $freq]
	set row [expr {$row+1}]
	set tableData($row,$1ppsTableCol) [format "%0.2f" $1ppsAmpl]
	set tableData($row,[expr {$1ppsTableCol+1}]) [format "%0.2f" $1ppsGain]

	if {$limitsOn} {
		set tableData($row,$1ppsTablePFCol) $1ppsPass
		set passFailCol $1ppsTablePFCol
	    set w "$topFrame.t.r${row},$passFailCol"
		if {[lsearch $1ppsPass "FAIL*"] == 0} {
			$w configure -bg red
		} else {
			$w configure -bg green
		}
	}
	update
}

proc updatePtpBodePlot {freq ptpAmpl ptpGain ptpPass} {
	variable sList
	variable tableData
	variable topFrame
	variable ptpTableCol
	variable ptpTablePFCol
	variable limitsOn
	variable yAxis
	
	if {$yAxis == "GAIN"} {
		ptpAmplBodePlotLine append $ptpGain		
	} else {
		ptpAmplBodePlotLine append $ptpAmpl		
	}

	set freqs [getFreqs $sList]
	set row [lsearch $freqs $freq]
	set row [expr {$row+1}]
	set tableData($row,$ptpTableCol) [format "%0.2f" $ptpAmpl]
	set tableData($row,[expr {$ptpTableCol+1}]) [format "%0.2f" $ptpGain]
	
	if {$limitsOn} {
		set tableData($row,$ptpTablePFCol) $ptpPass
		set passFailCol $ptpTablePFCol
	    set w "$topFrame.t.r${row},$passFailCol"
		if {[lsearch $ptpPass "FAIL*"] == 0} {
			$w configure -bg red
		} else {
			$w configure -bg green
		}
	}
	update
}


proc updateDataPlot {x y freq meas} {
	variable topFrame
	variable ptpPlotsFrame
	variable 1ppsPlotsFrame
	#XXXDebugTrace "updateDataPlot: $meas"
	
	set f [ string map {"." "_"} $freq ]
	#set plotId $topFrame.dataPlot_
	if {$meas == "PTP" || $meas == "T4"} {
		set plotId $ptpPlotsFrame.dataPlot_
	} else {
		set plotId $1ppsPlotsFrame.dataPlot_
	}
	#XXXDebugTrace "updateDataPlot: plotId - $plotId"
	set line "dataLine_"
	append line $f
	append plotId $f
	set xVec "xVec_"
	append xVec $f
	append xVec "_$meas"
	set yVec "yVec_"
	append yVec $f
	append yVec "_$meas"
	
	vector create $xVec $yVec
	$xVec set $x
	$yVec set $y
	$plotId element configure $line -xdata $xVec -ydata $yVec
	
	update
}

proc bltCaptureWindow { win } {

	# Bring the window to the front
	winop raise $win
	
	# Make an empty photo image
	set image [image create photo]

	# Snapshot of window/widget
	winop snap $win $image

	return $image
}
 
proc savePlotWindow {path filename} {
	variable topFrame
	
	set image [bltCaptureWindow $topFrame]
	set file $path/$filename
	append file ".bmp"

	#package require Img
	#$image write Screenshot.gif -format gif ;# Only if 256 colours or less
	#$image write Screenshot.png -format png
	#$image write Screenshot.jpg -format jpeg
	$image write $file -format bmp
}
