################################################################################
#																			   #
#   paragonwl.py															   #
#																			   #
#   Python  interface to Calnex Paragon										   #
#																			   #
#   Copyright (c) Calnex Solutions Ltd 2008 - 2019							   #
#																			   # 
#   The contents of this file should not be copied or distributed			   #
#   without permission being granted by Calnex Solutions Ltd.				   #
#																			   #
#   All rights reserved.													   #
#																			   #
################################################################################

# The provided commands are described below. Errors are signalled using the
# error method.

# Interface functions follow below - do not modify these 


from __future__ import print_function
import socket, sys,time, shlex

from measurements import *

# PRIVATE Members and Functions
# -----------------------------
_revStr = "$Revision: 25438 $"

# Expected reply constants
_PARAGON_READY = "__PARAGON__";
_PARAGON_ERROR = "ERROR: ";

# Private module variables
_socket	= None;
_lastErr   = "";
_connected = False;
_instrumentType = "";
_instrumentIP = "";
_instrument = "";

################################################################################
#
#  Socket handling functions
#
################################################################################

#set [namespace current]::__lastError ""

def logInfo ( msg ):
	print ( msg )

# Debug enable/disable
_DEBUG = False;
#_DEBUG = True;

def _debug(msg):
	if _DEBUG == True:
		print(msg)
		sys.stdout.flush()

# ======================================================================================================================================
#   Class declaration
#
#=======================================================================================================================================

class  ParagonX:
	paragonSocket = None;
	_lastError = "";
	calculateRequired = False
	idn = ""
	serialNumber = 0
	measurementConfiguration = "NONE"
	pxType= ""




	def __init__(self):
		self.calculateRequired = False 
		connected = 0
		self.paragonSocket = socket.socket()
		self.serialNumber = "Not Available"
		self.idn = "Not Available"
		self.version = "Not Available"
		self.pxType = "Not Available"
		self.dictionaryMeasurements = {}
	
	
	
	def __del__(self):
		del self.paragonSocket
	
	def setMeasurementConfiguration (self,meas): 
		self.measurementConfiguration  = meas


	def getMeasurementConfiguration (self):
		return self.measurementConfiguration

	###########################################################################################################
	

	def  addMeasurement ( self,name ,measurementAnalysis1 ,timeMeasurement1 , filter = "" ):

	######################
	# 
	# DJWAT REVISIT
	# ######################	

	#my selectSlot $timeMeasurement1
		self.selectSlot (timeMeasurement1)
		
		if measurementAnalysis1 == "TE":
			meas = TEMeasurementAnalysis( self, timeMeasurement1 )
		
		elif  measurementAnalysis1 == "ATE":
			meas =  ATEMeasurementAnalysis (self, timeMeasurement1) 
		
		elif measurementAnalysis1 == "DTEMTIELF_TDEVLF" :
		
			filterTypesList = [ 'MTIELF', 'TDEVLF' ]
		
			meas  = DTEMTIELF_TDEVLFMeasurementAnalysis (self, timeMeasurement1 )
			if filter == "MTIELF" or filter == "TDEVLF":
				meas.setFilterType ( filter )
			elif filter == "":
				error (" Error: Filter type was not defined, possible filter types are %s " % filterTypesList ) 
			else:
				error (" Error: Incorrect filter name: %s , possible filter types are  %s" % (filter,filterTypesList) )  
		
		
		elif  measurementAnalysis1 == "DTELF_HF":
			filterTypesList = [ 'LF', 'HF' ] 
			meas = DTELF_HFMeasurementAnalysis (self , timeMeasurement1 )
			
			if  filter == "LF" or filter == "HF": 
				meas.setFilterType ( filter )
			elif filter == "":
				error ( " Error: Filter type was not defined, possible filter types are %s" % filterTypesList ) 
			else:
				error ( " Error: Incorrect filter name: %s, possible filter types are %s " % (filter,filterTypesList ) ) 
			 
		
		elif  measurementAnalysis1 == "TETC":
		
			meas = TETCMeasurementAnalysis(self, timeMeasurement1 )
			meas.setMeasurementAnalysis (filter )
		
		else:
			error( " Error: Incorrect measurement analysis name: %s " % measurementAnalysis1 )			
	
	
		self.dictionaryMeasurements['%sname' % name ] = meas 
		return (meas)


	def  getMeasurement (self,name) : 

		return ( self.dictionaryMeasurements ['%s'% name])   


	def getDictionaryMeasurements (self):
	
		return (self.dictionaryMeasurements)


	def clearDictionaryMeasurements (self):
		self.dictionaryMeasurements = {}


	def selectSlot ( self,timeMeasurement ):
		self.paragonset ("Cat SelectSlot", timeMeasurement)
		self.paragonset ("Cat EnabledSlot",  "True")

	###################################################################################################

	def calculate (self):
		if  self.calculateRequired == True:
			self.paragonset ("Cat Calculate")
			self.waitforcat()
			self.calculateRequired = False
	


	def calculateRequiredSetTrue (self):
		self.calculateRequired =  True



	def calculateSpecified( self,timeMeasurement,measurementAnalysis):
	
		measurementAnalysisBCList = [ "TE," "ATE", "DTE", "DTEHF", "DTEMTIE" ,"DTETDEV" ]
		measurementAnalysisTCList = [ "FWD_CF_ACCURACY", "FWD_LATENCY", "FWD_CF_DELTA" ,"REV_CF_ACCURACY", "REV_LATENCY", "REV_CF_DELTA", "TWOWAY_CF_ACCURACY" ]
	
		if self.calculateRequired == True:
		
			if  measurementAnalysis in measurementAnalysisBCList:
				self.paragonset (" Cat TE AllEnable", "False")
				self.paragonset (" Cat ATE AllEnable", "False")
				self.paragonset (" Cat DTE AllEnable", "False")
				self.paragonset (" Cat DTEHF AllEnable", "False")
				self.paragonset (" Cat DTEMTIE AllEnable", "False")
				self.paragonset (" Cat DTETDEV AllEnable", "False")
			
			elif measurementAnalysis in measurementAnalysisTCList :
				self.paragonset (" Cat FWD_CF_ACCURACY AllEnable", "False")
				self.paragonset (" Cat FWD_LATENCY AllEnable", "False")
				self.paragonset (" Cat FWD_CF_DELTA AllEnable", "False")
				self.paragonset (" Cat REV_CF_ACCURACY AllEnable", "False")
				self.paragonset (" Cat REV_LATENCY AllEnable", "False")
				self.paragonset (" Cat REV_CF_DELTA AllEnable", "False")
				self.paragonset (" Cat TWOWAY_CF_ACCURACY AllEnable", "False")
		
		
			self.paragonset (" Cat SelectSlot", timeMeasurement )
			self.paragonset (" Cat EnabledSlot True" )
			self.paragonset (" Cat %s  Enable" % measurementAnalysis ,"True" )
			self.paragonset (" Cat Calculate" )
			self.waitforcat()
			self.calculateRequired = False
		
	


	#####################################################################
	#####################################################################
	###  CONNECTION  METHODS
	#####################################################################

	
	def getConnection(self):
		connected = "FALSE"
		print (connected,end='\n',flush= True)
	 

	# Get the instrument type from the Idn string
	def getInstrumentDetails(self):
		
		self.idn = self.paragonget("Idn")
		idnList = self.idn.split(',')

		self.recallserialNumber = idnList[2]
		self.pxType = idnList[1]

		versionS = idnList[3]
		#versionList = versionS.split(';')
		#verionO = versionList[1]
		#versionF =verionO.split('=')
		versionF = versionS.split('=')
		self.version = versionF[1]

		#logInfo $version
		logInfo("Idn = %s\n" % self.idn)


	
	def getInstrumentSerialNumber(self):
		return self.serialNumber

	def getInstrumentType(self):
		 return (self.pxType)

	def getInstrumentVersion(self):
		return(self.version)

		# Makes a connection to the specified GUI and instrument.
		#  @param instAddress required  This is the IP address of the Paragon instrument.
		#  @param guiAddress  optional  This specifies the IP address of the PC that is hosting
		#							   the GUI; if this parameter is not specified then a
		#							   connection to the local machine is assumed.
		#  @param guiport	 optional  This specifies the remote control TCP Port used by the
		#							   Paragon client application; if this parameter is not
		#							   specified then port 9000 is assumed.
		#  @param instPort	optional  This specifies the TCP Port on the Paragon instrument;
		#							   if this parameter is not specified then port 9990 is assumed.
		#  @throw Exception
		## Modified to follow the same process as the Tcl wrapper; also updated to be more robust
		## especially around connecting in demo mode
		return version

	def connect(self, instAddress, guiAddress = "localhost", instPort= 9990, guiport  = 9000, rc = 1):
		global _connected, _lastErr
		global _instrumentType, _instrumentIP
		global _revStr
		
		status = 0
		error = ""

		_debug ("instAddress: " + instAddress + " guiAddress: " + guiAddress)

		self._gui_connect(guiAddress, guiport)
		
		try:
			self._remote_control_on(rc)
		
			connStatus = self._is_connected("")
			_debug ("connect: connStatus = " + str(connStatus))
			
			if connStatus < 2:
				instStatus = self._is_connected(instAddress)
				_debug ("connect: instStatus = " + str(instStatus))
				if connStatus == 1 and instStatus == 0:
					currIp = self.paragonget("RmtIpAddress")
					error = "ERROR: another instrument (%s) is already connected" % currIp;
				elif instStatus == 0:
					self._instrument_connect(instAddress, instPort)
					_connected = True
			else:
				_connected = True
		
		except Exception as e:
			status = 1
			_debug ("Exception while attempting to connect: " + str (e.args))
		
		_debug ("Status: " + str(status) + " Error: " + error)
		
		if (error != ""):
			_lastErr = error
			self._check_for_error("connect")
			
		if (status == 0):
		#	if _connected == 1:
			_debug("connected")
			self.getInstrumentDetails()
			_instrumentType = self.getInstrumentType()
			_instrumentIP = instAddress
			idnString = self.paragonget ("Idn")
			revParts = _revStr.split()
			#print("Connected to " + _instrumentType)
			print(idnString + ' (' + revParts[1] + ')',end='\n',flush= True)
			
		else:
			_debug("not connected")
			err = _lastErr
			self.disconnect()
			_lastErr = err
			self._check_for_error("connect")

	## Disconnnects the currently connected instrument.
	def disconnect(self):

		global _socket, _connected
		global _instrumentType, _instrumentIP
	
		#if _connected == True:

		try: self._instrument_disconnect()
		except: pass
		try: self._remote_control_off()
		except: pass
		try: _socket.close()
		except: pass

		_socket	= None
		_connected = False
		_instrumentType = ""
		_instrumentIP = ""
		
		print("Disconnected.",end='\n',flush= True)

	## Returns 1 if there is a connection to the instrument; 0 otherwise	
	def isconnected(self,instrumentIP=""):
		resp = 0
	
		# Send generates an error if not connected to the GUI. For isconnected, we should simply return false
		if (instrumentIP == ""):
			ip = _instrumentIP
		else:
			ip = instrumentIP
		
		if (ip == ""):
			resp = 0
		else:
			guiConn = self._isguiconnected()
			#print ("isconnected: guiConn = %d" % guiConn)
			if (guiConn == 0):
				resp = 0
			else:
				resp = self._is_connected(instrumentIP)
				self._check_for_error("isconnected")
		
		return resp

	
	########################################
	## CONNECTION 
	#########################################

	def _gui_connect(self,address, guiport):
		global paragonSocket
			
		print("Connecting to %s:%s... " %(address, guiport))

		# Create a new socket and connect to the GUI
		self.paragonSocket.connect((address, guiport))
			
		print("Success.")

		# Flush and check for errors
		self._receive()
		self._check_for_error("_gui_connect")

	def _receive(self):
		global _lastErr, _socket

		response = "";
		_lastErr = "";
			
		# Create file from socket for convenience
		# and read each line from it
		file = self.paragonSocket.makefile()
		for line in file:
			_debug("Received: %s" % line)		
			# Errors are returned as a string beginning 'ERROR: '
			# Ready is signalled by '__PARAGON__' prompt
			# Anything else is considered a response
			if line.find(_PARAGON_ERROR) >= 0:
				# Got an error
				_lastErr = line[len(_PARAGON_ERROR):len(line)]
			elif line.find(_PARAGON_READY) >= 0:
				# Got the ready prompt, OK to leave
				break
			else:
				# Got a response. Store it and continue
				response = line
		file.close()
			
		return response
			
	#if sys.version_info < (3, 0): # py2
	#	def _send(self,data):
	#		global _socket
	#			
	#		_debug("Sending %s... " % data)
	#		self.paragonSocket.sendall("%s\n" % data)
	#		_debug("Done.")	
	#		return self._receive().strip()
	#else: # py3
	def _send(self,data):
		global _socket
			
		_debug("Sending %s... " % data)
		self.paragonSocket.sendall(bytes("%s\n" % data, "UTF-8"))
		_debug("Done.")	


		return self._receive().strip()



	def _is_connected(self,instrumentIP):
		response = self._send("paragon.remoteprint(paragon.isconnected(\"%s\"))" % instrumentIP)
		self._check_for_error("_is_connected")
		return int(response)



	def _check_for_error(self,label):
		global _lastErr
		
		if len(_lastErr) > 0:
			raise Exception("%s : %s" %(label, _lastErr))


	def _build_cmd_from_args(self,*args):
		cmd = ""
		for arg in args:
			# Indexed: 1st char is '#'
			if arg[0] == '#':
				arg = "__IDX____#__" + arg[1:len(arg)]
			# Append argument
			cmd = cmd + "__#__" + arg
		return cmd


	def _remote_control_on(self,rc):
		self._send("paragon.on(%s)" % rc)
		self._check_for_error("_remote_control_on")

	def _remote_control_off(self):
		self._send("paragon.off()")
		self._check_for_error("_remote_control_off")

	def _instrument_connect(self,instAddr, instPort):
		print("Connecting to instrument %s:%s... " %(instAddr, instPort),end='\n',flush= True)
		self._send("paragon.connect(\"%s\",\"%s\")" %(instAddr, instPort));
		self._check_for_error("_instrument_connect %s:%s" %(instAddr, instPort))

	def _instrument_disconnect(self):
		self._send("paragon.disconnect()")
		self._check_for_error("_instrument_disconnect")

	################################################################################
	# The following routines are used to make the Rst process more robust
	# There are occasions where Rst has issues when the instrument is active
	# These routines try to make sure that the box is in an idle state before the 
	# Rst is sent.
	# Note: The Rst command is intercepted in paragonset; anything that is active on
	# the instrument is then stopped and then the Rst is passed on.
	# Since this file is also used for P100G, a number of the commands may fail
	# if any given feature has not yet been implemented. This is avoided by making
	# sure that errors are ignored in the appropriate places

	##
	# Check to see if any wander generation is running
	# If so, stop it
	def _stopWander(self):
		_debug("stopWander")
		isFreqOffsetRunning = "FALSE"
		tolSingleState  	= "IDLE"
		tolTableState  		= "IDLE"
		tolMtieTdevState  	= "IDLE"
		synceTransientState = "IDLE"
		transferSingleState = "IDLE"
		transferTableState 	= "IDLE"
		transferTdevState 	= "IDLE"
	
		try: isFreqOffsetRunning = self.paragonget("WanderGeneration FrequencyOffset Enable")
		except: pass
		try: tolSingleState = self.paragonget("WanderGeneration Tolerance Single State")
		except: pass
		try: tolTableState = self.paragonget("WanderGeneration Tolerance Table State")
		except: pass
		try: tolMtieTdevState = self.paragonget("WanderGeneration Tolerance MtieTdev State")
		except: pass
		try: synceTransientState = self.paragonget("WanderGeneration SyncETransient State")
		except: pass
		try: transferSingleState = self.paragonget("WanderGeneration Transfer Single State")
		except: pass
		try: transferTableState = self.paragonget("WanderGeneration Transfer Table State")
		except: pass
		try: transferTdevState = self.paragonget("WanderGeneration Transfer TDEV State")
		except: pass
	
		if (isFreqOffsetRunning != "FALSE"):
			self.paragonset("WanderGeneration FrequencyOffset Enable", "FALSE")
	
		if (tolSingleState != "IDLE"):
			self.paragonset("WanderGeneration Tolerance Single Enable", "FORCESTOP")
	
		if (tolTableState != "IDLE"):
			self.paragonset("WanderGeneration Tolerance Table Enable", "FORCESTOP")
	
		if (tolMtieTdevState != "IDLE"):
			self.paragonset("WanderGeneration Tolerance MtieTdev Enable", "STOP")
	
		if (synceTransientState != "IDLE"):
			self.paragonset("WanderGeneration SyncETransient Enable", "STOP")
	
		if (transferSingleState != "IDLE"):
			try: self.paragonset("WanderGeneration Transfer Single CalibrateEnable", "FORCESTOP")
			except: pass
			try: self.paragonset("WanderGeneration Transfer Single GenerateEnable", "FORCESTOP")
			except: pass
	
		if (transferTableState != "IDLE"):
			try: self.paragonset("WanderGeneration Transfer Table CalibrateEnable", "FORCESTOP")
			except: pass
			try: self.paragonset("WanderGeneration Transfer Table GenerateEnable", "FORCESTOP")
			except: pass
	
		if (transferTdevState != "IDLE"):
			self.paragonset("WanderGeneration Transfer TDEV Enable", "STOP")

	def _stopPacketGeneration(self):
		_debug("stopPacketGeneration")
		isPort0Running = "FALSE"
		isPort1Running = "FALSE"
	
		try: isPort0Running = self.paragonget("PacketGeneration #0 Enable")
		except: pass
		try: isPort1Running = self.paragonget("PacketGeneration #1 Enable")
		except: pass

		if (isPort0Running != "FALSE"):
			self.paragonset("PacketGeneration #0 Enable", "FALSE")
		if (isPort1Running != "FALSE"):
			self.paragonset("PacketGeneration #1 Enable", "FALSE")

	# Check to see if any jitter generation is running
	# If so, stop it
	def _stopJitter(self):
		_debug("stopJitter")
		tolSingleState  	= "IDLE"
		tolTableState  		= "IDLE"
		maxTolTableState  	= "IDLE"
	
		try: tolSingleState = self.paragonget ("Jitter Tolerance Single State")
		except: pass
		try: tolTableState  = self.paragonget ("Jitter Tolerance Table State")
		except: pass
		try: maxTolTableState = self.paragonget ("Jitter MaxTolerable Table State")
		except: pass
	
		if (tolSingleState != "IDLE"):
			self.paragonset ("Jitter Tolerance Single Enable", "FORCESTOP")
		if (tolTableState != "IDLE"):
			self.paragonset ("Jitter Tolerance Table Enable", "FORCESTOP")
		if (maxTolTableState != "IDLE"):
			self.paragonset ("Jitter MaxTolerable Table Enable", "FORCESTOP")
		
	def _stopMSE(self):
		_debug("stopMSE")
		isMaster0Running = "FALSE"
		isMaster1Running = "FALSE"
		isSlaveRunning 	 = "FALSE"
	
		try: isMaster0Running = self.paragonget ("MasterSlave Master #0 Enabled")
		except: pass
		try: isMaster1Running = self.paragonget ("MasterSlave Master #1 Enabled")
		except: pass
		try: isSlaveRunning   = self.paragonget ("MasterSlave Slave Enabled")
		except: pass
	
		if (isMaster0Running != "FALSE"):
			self.paragonset ("MasterSlave Master #0 Enabled", "FALSE")
		if (isMaster1Running != "FALSE"):
			# Looks like the get doesn't return the correct status so ignore errors
			try: self.paragonset ("MasterSlave Master #1 Enabled", "FALSE")
			except: pass
		if (isSlaveRunning != "FALSE"):
			try: self.paragonset ("MasterSlave Slave Enabled", "FALSE")
			except: pass
		
	def _stopCapture(self):
		_debug("stopCapture")
		#set retry 0
		self.stopcapture()
		#while {[paragonget InstrumentStatus Capture IsRunning] == TRUE && $retry < 6} {
		#	after 500
		#	stopcapture
		#	incr retry
		#}

	def _stopImpairment(self):
		_debug("stopImpairment")
		try: self.stopimpairment()
		except: pass
	
	## Before executing Rst, make sure that the instrument is idle
	# - If a capture is in progress then we not idle.
	# - If Thru mode is active we are not idle.
	# - If import-export is active we are not idle.
	#		We can't check for this and so we ignore it - unlikely to happen
	# - If MSE is active we are not idle.
	# - If impairments are running we are not idle.
	# - If wander generation is active we are not idle.
	# - If Jitter generation is active we are not idle.
	# - If Packet Generation is active we are not idle.
	def _doResetPreparation(self):
		_debug("doResetPreparation")
	
		# Stop anything that is running
		self._stopCapture()
		self._stopImpairment()
		self._stopWander()
		self._stopPacketGeneration()
		self._stopJitter()
		self._stopMSE()
		
	
	def _rstTranslation(self,arguments):
		#set args [join $arguments]

		# We want bypass set to false.
		# Once we are done with the other reset actions, we want to execute the Rst normally
		bypass = False

		#if {[lindex $args 0 ] == "Rst"} {
		if "Rst" in arguments:
			_debug("rstTranslation." + arguments + ".")
			self._doResetPreparation()
	
		return bypass
	
	
	#######################################################
	##  Instrument functions
	#######################################################

	## Query an instrument setting. Most of the instrument's settings are read
	#  using by this command. (Refer to the remote control manual for the complete list)
	#  @param arg_str required  Setting to query (e.g "Physical LineRate").
	#  @return Setting value.
	#  @throw Exception
	def paragonget(self,arg_str):
		_debug("paragonget " + arg_str)
		wireCmd = "paragon.CSettingParameter(\"__GET__"
		wireCmd += self._build_cmd_from_args(*shlex.split(arg_str)) + "\")"
			
		self._send(wireCmd)
		self._check_for_error("paragonget %s" % wireCmd)
		
		response = self._send("print(paragon.CSettingParameter_resp())")
		self._check_for_error("paragonget %s" % response)
		
		# Ignore the type ID character at the start of the response
		return response[1:len(response)]

	## Set an instrument setting. Most of the instrument's settings are controlled
	#  using by this command. (Refer to the remote control manual for the complete list)
	#  @param arg_str required  Setting to set (e.g "Physical LineRate").
	#  @param value   required  The value to set.
	#  @throw Exception
	def paragonset(self,arg_str, value = None):
		_debug("paragonset " + arg_str)
		isBypass = False
			
		wireCmd = "paragon.CSettingParameter(\"__SET__"

		# Parse the incoming command such that specific actions can be taken for specific commands
		# In most cases, when a command is found, it will bypass further processing
		if (_instrumentType == "Paragon-X"):
			isBypass = self._rstTranslation(arg_str)
			# Only send the command is it hasn't already been dealt with
		if (isBypass == False):
			wireCmd += self._build_cmd_from_args(*shlex.split(arg_str))
			if (value != None):
				wireCmd += "__#__" + value
			wireCmd += "\")"
			self._send(wireCmd)
			self._check_for_error("paragonset %s" % wireCmd)
		## Starts an ALL byte capture.
		#  @throw Exception
	
	def startpacketcapture(self):
		self._send("paragon.startpacketcapture()")
		self._check_for_error("startpacketcapture")

	## Starts timing capture.
	#  @throw Exception
	def starttimingcapture(self):
		self._send("paragon.starttimingcapture()")
		self._check_for_error("starttimingcapture")

	## Stops current capture.
	#  @throw Exception
	def stopcapture(self):
		self._send("paragon.stopcapture()")
		self._check_for_error("stopcapture")

	## Enables the impairment as configured in the settings.
	#  @throw Exception
	def startimpairment(self):
		self._send("paragon.startimpairment()")
		self._check_for_error("startimpairment")

	## Disables the impairment as configured in the settings.
	#  @throw Exception
	def stopimpairment(self):
		self._send("paragon.stopimpairment()")
		self._check_for_error("stopimpairment")

	## Starts Time of Day capture.
	#  @throw Exception
	def starttodcapture(self):
		self._send("paragon.starttodcapture()")
		self._check_for_error("starttodcapture")

	## Stops Time Of Day capture.
	#  @throw Exception
	def stoptodcapture(self):
		self._send("paragon.stoptodcapture()")
		self._check_for_error("stoptodcapture")

	## Select the SONET STS1 tributary to impair.
	#  @param trib required  STS1 tributary number.
	#  @throw Exception
	def setsonetowsts1trib(self,trib):
		self._send("paragon.CSettingParameter(\"__SET____#___SonetSts1Trib__#__%s\")" % trib)
		self._check_for_error("setsonetowsts1trib %s" % trib)

	## Select the SONET VT1.5 tributary to impair.
	#  @param trib required  VT1.5 tributary number.
	#  @throw Exception
	def setsonetowvttrib(self,trib):
		self._send("paragon.CSettingParameter(\"__SET____#___SonetVtTrib__#__%s\")" % trib)
		self._check_for_error("setsonetowvttrib %s" % trib)

	## Set the SONET delay
	#  @param delay required  Delay in uS.
	#  @throw Exception
	def setsonetdelay(self,delay):
		self._send("paragon.CSettingParameter(\"__SET____#___SonetDelay__#__%s\")" % delay)
		self._check_for_error("setsonetdelay %s" % delay)

	## Saves the displayed captured data to a file.
	#  @param filename required  Path and filename for a file on the local PC's
	#							file system. filename must be suffixed with a
	#							valid filename extension:
	#							- ".csv"
	#							- ".cpd"
	#  @throw Exception
	def exportdata(self,filename):
		filename = filename.replace("\\", "/")
		self._send("paragon.exportdata(\"%s\")" % filename)
		self._check_for_error("exportdata %s" % filename)
		## Loads the captured data from a file.
		#  The importdata is equivalent to using the GUI toolbar menu item File>Import. 
		#  This is not port specific and in the bi-directional context is not intended
		#  to be used for loading impairment data for replay.
		#  @param filename required  Path and filename for a file on the local PC's
		#							file system. filename must be suffixed with a
		#							valid filename extension:
		#							- ".csv"
		#							- ".cpd"
	
	#  @throw Exception
	def importdata(self,filename):
		filename = filename.replace("\\", "/")
		self._send("paragon.importdata(\"%s\")" % filename)
		self._check_for_error("importdata %s" % filename)

		## Stores the instrument settings to the file passed in using the filename
		#  parameter. Settings are stored in the file system of the machine hosting
		#  the GUI. The stored settings file can be recalled using the 'recall' command.
		#  If the specified file already exists, then it will be overwritten.
		#  @param filename required  Path and filename for a file on the local PC's
		#							file system.
	
	#  @throw Exception
	def store(self,filename):
		filename = filename.replace("\\", "/")
		self._send("saveSettings(\"%s\")" % filename)
		self._check_for_error("store %s" % filename)

		## Recalls the instrument settings from the file passed in using the filename
		#  parameter. Settings are stored in the file system of the machine hosting
		#  the GUI. A stored settings file can be created by from the Paragon GUI
		#  using Save on the Setup menu.
		#  @param filename required  Path and filename for a file on the local PC's
		#							file system.
	
	#  @throw Exception
	def recall(self,filename):
		filename = filename.replace("\\", "/")
		self._send("loadSettings(\"%s\")" % filename)
		self._check_for_error("recall %s" % filename)

		## Loads the capture data for replay. It will load files specifically to replay
		#  delay profiles against defined message types on defined ports.
		#  The loaded file will be replayed against incoming traffic on the given port.
		#  The importimpairmentdata command is equivalent to using the "Import-Port1" and
		#  "Import-Port2" buttons in the Add Impairments and Delay window
		#  @param port	 required  The Ethernet port (1 or 2).
		#  @param filename required  Path to a capture file on the local PC's
		#							file system.
		#  @throw Exception
	def importimpairmentdata(self,port, filename):
		filename = filename.replace("\\", "/")
		self._send("paragon.importimpairmentdata(\"%s\",\"%s\")" %(port, filename))
		self._check_for_error("importimpairmentdata %s %s" %(port, filename))

		## Wait for the completion of the current CAT operation. 
		#  If a script is waiting for previous open operation or waiting for end of
		#  previous calculation then this command should be always used. It is
		#  recommended to call this after the calls below otherwise there is
		#  possibility to get settings conflict error.
		#  @throw Exception
	def waitforcat(self):
		status = self.paragonget("Cat Status")[0]
		while(status == "1"):
			status = self.paragonget("Cat Status")[0]
			time.sleep(1)
		_debug("Wait operation completed")


